@@ -9,22 +9,31 @@
  * This file is part of the GNUstep Base Library.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  * 
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA. */
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02111 USA. */
+
+#import	<GNUstepBase/GSVersionMacros.h>
+
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
 #include <Foundation/NSObject.h>
 #include <Foundation/NSZone.h>
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 /* To easily un-inline functions for debugging */
 #ifndef	INLINE
 #define INLINE inline
@@ -919,3 +928,9 @@ GSIMapInitWithZoneAndCapacity(GSIMapTable map, NSZone *zone, size_t capacity)
   GSIMapMoreNodes(map, capacity);
 }
 
+#if	defined(__cplusplus)
+}
+#endif
+
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
+
