@@ -9,16 +9,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -26,8 +26,11 @@
    AutogsdocSource: Additions/GSMime.m
 */
 
-#ifndef __GSMIME_H__
-#define __GSMIME_H__
+#ifndef __GSMime_h_GNUSTEP_BASE_INCLUDE
+#define __GSMime_h_GNUSTEP_BASE_INCLUDE
+#include <GNUstepBase/GSVersionMacros.h>
+
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
 #ifdef NeXT_Foundation_LIBRARY
 #include <Foundation/Foundation.h>
@@ -36,6 +39,10 @@
 #include	<Foundation/NSString.h>
 #endif
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 @class	NSArray;
 @class	NSMutableArray;
 @class	NSData;
@@ -225,4 +232,10 @@
 - (void) setIsHttp;
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
+
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
+
+#endif	/* __GSMime_h_GNUSTEP_BASE_INCLUDE */
