@@ -14,25 +14,29 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    AutogsdocSource: Additions/GSXML.m
 
 */
 
-#ifndef __GSXML_H__
-#define __GSXML_H__
+#ifndef __GSXML_h_GNUSTEP_BASE_INCLUDE
+#define __GSXML_h_GNUSTEP_BASE_INCLUDE
+#include <GNUstepBase/GSVersionMacros.h>
+
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
 #ifndef NeXT_Foundation_LIBRARY
 #include <Foundation/NSObject.h>
@@ -42,8 +46,9 @@
 #include <Foundation/Foundation.h>
 #endif
 
-#ifndef	STRICT_MACOS_X
-#ifndef	STRICT_OPENSTEP
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class GSXMLAttribute;
 @class GSXMLDocument;
@@ -739,9 +744,11 @@
 - (void) completedXMLRPC: (GSXMLRPC*)sender;
 @end
 
+#if	defined(__cplusplus)
+}
+#endif
 
-#endif	/* STRICT_MACOS_X */
-#endif	/* STRICT_OPENSTEP */
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
 
-#endif /* __GSXML_H__ */
+#endif /* __GSXML_h_GNUSTEP_BASE_INCLUDE */
 
