@@ -12,16 +12,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 
@@ -31,28 +31,31 @@
 
 #ifndef __Unicode_h_OBJECTS_INCLUDE
 #define __Unicode_h_OBJECTS_INCLUDE
+#include <GNUstepBase/GSVersionMacros.h>
+#include <GNUstepBase/GSObjCRuntime.h>
 
-#include <Foundation/NSString.h>	/* For string encodings */
+#import <Foundation/NSString.h>	/* For standard string encodings */
 
-/*
- * Private API used internally by NSString etc.
- */
-#ifndef	NO_GNUSTEP
-#include "GSObjCRuntime.h"
+#if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
-GS_EXPORT NSStringEncoding *GetAvailableEncodings(void);
-GS_EXPORT NSStringEncoding GetDefEncoding(void);
+#if GS_API_VERSION(GS_API_NONE,011500)
+/* Deprecated functions */
 GS_EXPORT NSStringEncoding GSEncodingFromLocale(const char *clocale);
 GS_EXPORT NSStringEncoding GSEncodingForRegistry(NSString *registry, 
-						 NSString *encoding);
-GS_EXPORT NSString* GSEncodingName(NSStringEncoding encoding);
-
+  NSString *encoding);
 GS_EXPORT unichar uni_tolower(unichar ch);
 GS_EXPORT unichar uni_toupper(unichar ch);
+
 GS_EXPORT unsigned char uni_cop(unichar u);
 GS_EXPORT BOOL uni_isnonsp(unichar u);
 GS_EXPORT unichar *uni_is_decomp(unichar u);
+GS_EXPORT unsigned GSUnicode(const unichar *chars, unsigned length,
+  BOOL *isASCII, BOOL *isLatin1);
+#endif
 
 
 /*
@@ -64,8 +67,6 @@ GS_EXPORT unichar *uni_is_decomp(unichar u);
 #define	GSUniBOM	0x08
 #define	GSUniShortOk	0x10
 
-GS_EXPORT unsigned GSUnicode(const unichar *chars, unsigned length,
-  BOOL *isASCII, BOOL *isLatin1);
 GS_EXPORT BOOL GSFromUnicode(unsigned char **dst, unsigned int *size,
   const unichar *src, unsigned int slen, NSStringEncoding enc, NSZone *zone,
   unsigned int options);
@@ -73,22 +74,10 @@ GS_EXPORT BOOL GSToUnicode(unichar **dst, unsigned int *size,
   const unsigned char *src, unsigned int slen, NSStringEncoding enc,
   NSZone *zone, unsigned int options);
 
-/*
- *  The next functions are deprecated and will be removed in a future
- *  release.  Use GSFromUnicode() and GSToUnicode() instead.
- */
-GS_EXPORT NSString* GetEncodingName(NSStringEncoding encoding);
-GS_EXPORT unichar chartouni(unsigned char c);
-GS_EXPORT unsigned char unitochar(unichar u);
-GS_EXPORT unichar encode_chartouni(unsigned char c, NSStringEncoding enc);
-GS_EXPORT unsigned char encode_unitochar(unichar u, NSStringEncoding enc);
-GS_EXPORT unsigned encode_unitochar_strict(unichar u, NSStringEncoding enc);
-GS_EXPORT int encode_ustrtocstr(char *dst, int dl, const unichar *src, int sl, 
-  NSStringEncoding enc, BOOL strict);
-GS_EXPORT int encode_cstrtoustr(unichar *dst, int dl, const char *src, int sl, 
-  NSStringEncoding enc);
-
-
+#if	defined(__cplusplus)
+}
 #endif
 
+#endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
+
 #endif /* __Unicode_h_OBJECTS_INCLUDE */
