@@ -8,30 +8,35 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */ 
 
 #ifndef __NSConnection_h_GNUSTEP_BASE_INCLUDE
 #define __NSConnection_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSTimer.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSMapTable.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSDictionary.h>
+#import	<Foundation/NSString.h>
+#import	<Foundation/NSTimer.h>
+#import	<Foundation/NSRunLoop.h>
+#import	<Foundation/NSMapTable.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSDistantObject;
 @class NSPort;
@@ -295,4 +300,8 @@ GS_EXPORT NSString * const NSConnectionDidInitializeNotification; /* OPENSTEP */
  */
 GS_EXPORT NSString * const NSFailedAuthenticationException; /* MacOS-X  */
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSConnection_h_GNUSTEP_BASE_INCLUDE */
