@@ -9,28 +9,33 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #ifndef __NSDebug_h_GNUSTEP_BASE_INCLUDE
 #define __NSDebug_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
 #include <errno.h>
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
 
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 /*
  *	Functions for debugging object allocation/deallocation
  *
@@ -207,8 +212,8 @@ GS_EXPORT BOOL NSDeallocateZombies;
 
 
 #ifdef GSDIAGNOSE
-#include	<Foundation/NSObjCRuntime.h>
-#include	<Foundation/NSProcessInfo.h>
+#import	<Foundation/NSObjCRuntime.h>
+#import	<Foundation/NSProcessInfo.h>
 
 /**
    <p>NSDebugLLog() is the basic debug logging macro used to display
@@ -380,7 +385,7 @@ GS_EXPORT BOOL NSDeallocateZombies;
 
 
 #ifdef GSWARN
-#include	<Foundation/NSObjCRuntime.h>
+#import	<Foundation/NSObjCRuntime.h>
 
 /**
    <p>NSWarnLog() is the basic debug logging macro used to display
@@ -462,4 +467,8 @@ GS_EXPORT void *NSReturnAddress(int offset);
  */
 GS_EXPORT unsigned NSCountFrames(void);
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif
