@@ -7,34 +7,40 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    */
 
 #ifndef __NSDecimal_h_GNUSTEP_BASE_INCLUDE
 #define __NSDecimal_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#ifndef	STRICT_OPENSTEP
+#include <GNUstepBase/GSConfig.h>
 
-#include <GSConfig.h>
+#import	<Foundation/NSObject.h>
+
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
 #if	USE_GMP
 #include <gmp.h>
 #endif
 
-#include <Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 /**
  *  Enumerated type for specifying decimal rounding behavior.  Can be one of
@@ -45,9 +51,9 @@
  *  [(NSDecimalNumberBehaviors)] protocol.
  */
 typedef	enum {
+  NSRoundPlain,		/* Round .5 up		*/
   NSRoundDown,
   NSRoundUp,
-  NSRoundPlain,		/* Round .5 up		*/
   NSRoundBankers	/* Make last digit even	*/
 } NSRoundingMode;
 
@@ -69,9 +75,9 @@ typedef	enum {
  */
 typedef enum {
   NSCalculationNoError = 0,
+  NSCalculationLossOfPrecision,
   NSCalculationUnderflow,	/* result became zero */
   NSCalculationOverflow,
-  NSCalculationLossOfPrecision,
   NSCalculationDivideByZero
 } NSCalculationError;
 
@@ -245,6 +251,10 @@ GS_EXPORT void
 NSDecimalFromString(NSDecimal *result, NSString *numberValue, 
 		    NSDictionary *locale);
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif
 #endif
 
