@@ -7,30 +7,34 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #ifndef __NSDecimalNumber_h_GNUSTEP_BASE_INCLUDE
 #define __NSDecimalNumber_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include	<Foundation/NSObject.h>
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#ifndef	STRICT_OPENSTEP
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSDecimal.h>
+#import	<Foundation/NSValue.h>
 
-#include	<Foundation/NSDecimal.h>
-#include	<Foundation/NSValue.h>
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class	NSDecimalNumber;
 
@@ -404,5 +408,9 @@
 - (NSDecimal) decimalValue;
 @end
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif
 #endif
