@@ -7,19 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
     AutogsdocSource: NSFileHandle.m
     AutogsdocSource: NSPipe.m
@@ -27,10 +27,17 @@
 
 #ifndef __NSFileHandle_h_GNUSTEP_BASE_INCLUDE
 #define __NSFileHandle_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSData.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSRange.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
+@class NSData;
+@class NSString;
 
 @interface NSFileHandle : NSObject
 
@@ -160,7 +167,7 @@ GS_EXPORT NSString * const NSFileHandleOperationException;
 
 
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 // GNUstep class extensions
 
@@ -237,4 +244,8 @@ GS_EXPORT NSString * const GSFileHandleWriteCompletionNotification;
 GS_EXPORT NSString * const GSFileHandleNotificationError;
 #endif
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSFileHandle_h_GNUSTEP_BASE_INCLUDE */
