@@ -10,29 +10,34 @@
  * This file is part of the GNUstep Base Library.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  * 
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, 02111 USA.
  */ 
 
 #ifndef __NSHashTable_h_GNUSTEP_BASE_INCLUDE
 #define __NSHashTable_h_GNUSTEP_BASE_INCLUDE 1
+#import	<GNUstepBase/GSVersionMacros.h>
 
 /**** Included Headers *******************************************************/
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSArray.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSString.h>
+#import	<Foundation/NSArray.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 /**** Type, Constant, and Macro Definitions **********************************/
 
@@ -138,4 +143,8 @@ NSHashRemove(NSHashTable *table, const void *element);
 GS_EXPORT NSString *
 NSStringFromHashTable(NSHashTable *table);
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSHashTable_h_GNUSTEP_BASE_INCLUDE */
