@@ -8,25 +8,32 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */ 
 
 #ifndef __NSKeyValueCoding_h_GNUSTEP_BASE_INCLUDE
 #define __NSKeyValueCoding_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include	<Foundation/NSObject.h>
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSArray;
 @class NSMutableArray;
@@ -34,8 +41,6 @@
 @class NSError;
 @class NSString;
 
-#ifndef	STRICT_OPENSTEP
-
 /** An exception for an unknown key in [NSObject(NSKeyValueCoding)]. */
 GS_EXPORT NSString* const NSUndefinedKeyException;
 
@@ -353,6 +358,11 @@ GS_EXPORT NSString* const NSUndefinedKeyException;
 
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
+
+#endif	/* GS_API_MACOSX */
+
 #endif
 
