@@ -11,9 +11,9 @@
    This file is part of the GNUstep Objective-C Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,15 +23,21 @@
    If you are interested in a warranty or support for this source code,
    contact Scott Christley <scottc@net-community.com> for more information.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 */ 
 
-#ifndef _GNUstep_H_NSLock
-#define _GNUstep_H_NSLock
+#ifndef __NSLock_h_GNUSTEP_BASE_INCLUDE
+#define __NSLock_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 /**
  * Protocol defining lock and unlock operations.
@@ -196,7 +202,7 @@
 
 @end
 
-#ifndef NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /**
  * Returns IDENT which will be initialized
@@ -284,6 +290,11 @@
 + (id) newLockAt: (id *)location;
 @end
 
-#endif /* NO_GNUSTEP */
+#endif	/* GS_API_NONE */
+
+#if	defined(__cplusplus)
+}
+#endif
+
+#endif /* __NSLock_h_GNUSTEP_BASE_INCLUDE */
 
-#endif /* _GNUstep_H_NSLock*/
