@@ -10,28 +10,34 @@
  * This file is part of the GNUstep Base Library.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  * 
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111 USA.  */ 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02111 USA.
+ */ 
 
 #ifndef __NSMapTable_h_GNUSTEP_BASE_INCLUDE
 #define __NSMapTable_h_GNUSTEP_BASE_INCLUDE 1
+#import	<GNUstepBase/GSVersionMacros.h>
 
 /**** Included Headers *******************************************************/
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSArray.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSString.h>
+#import	<Foundation/NSArray.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 /**** Type, Constant, and Macro Definitions **********************************/
 
@@ -205,4 +211,8 @@ NSMapRemove(NSMapTable *table, const void *key);
 
 GS_EXPORT NSString *NSStringFromMapTable (NSMapTable *table);
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSMapTable_h_GNUSTEP_BASE_INCLUDE */
