@@ -29,27 +29,34 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 */
 
-#ifndef __NSNotificationQueue_h__
-#define __NSNotificationQueue_h__
+#ifndef __NSNotificationQueue_h_GNUSTEP_BASE_INCLUDE
+#define __NSNotificationQueue_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSNotification.h>
+#import	<Foundation/NSObject.h>
 
-@class NSMutableArray;
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
+@class NSArray;
+@class NSNotification;
+@class NSNotificationCenter;
 
 /*
  * Posting styles into notification queue
@@ -67,9 +74,9 @@
  </example>
  */
 typedef enum {
-  NSPostWhenIdle,	
-  NSPostASAP,		
-  NSPostNow		
+  NSPostWhenIdle = 1,
+  NSPostASAP = 2,
+  NSPostNow = 3
 } NSPostingStyle;
 
 /**
@@ -84,9 +91,9 @@ typedef enum {
  </example>
  */
 typedef enum {
-  NSNotificationNoCoalescing = 0,	
-  NSNotificationCoalescingOnName = 1,	
-  NSNotificationCoalescingOnSender = 2,	
+  NSNotificationNoCoalescing = 0,
+  NSNotificationCoalescingOnName = 1,
+  NSNotificationCoalescingOnSender = 2,
 } NSNotificationCoalescing;
 
 /*
@@ -126,4 +133,8 @@ struct _NSNotificationQueueList;
 
 @end
 
-#endif /* __NSNotificationQueue_h__ */
+#if	defined(__cplusplus)
+}
+#endif
+
+#endif /* __NSNotificationQueue_h_GNUSTEP_BASE_INCLUDE */
