@@ -7,19 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
   AutogsdocSource: NSPort.m
   AutogsdocSource: NSSocketPort.m
@@ -28,9 +28,10 @@
 
 #ifndef __NSPort_h_GNUSTEP_BASE_INCLUDE
 #define __NSPort_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include	<Foundation/NSObject.h>
-#include	<Foundation/NSMapTable.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSMapTable.h>
 
 #if	defined(__MINGW32__)
 #include	<winsock2.h>
@@ -40,6 +41,10 @@
 #define	SOCKET	int
 #endif
 
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 @class	NSMutableArray;
 @class	NSConnection;
 @class	NSDate;
@@ -133,7 +138,7 @@ GS_EXPORT NSString * const NSPortTimeoutException; /* OPENSTEP */
  */
 - (BOOL) isValid;
 
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 /**
  * Adds to run loop as input source to be notified for input in given mode.
  * This method is for use by subclasses.
@@ -183,7 +188,7 @@ GS_EXPORT NSString* const NSPortDidBecomeInvalidNotification;
 
 #define	PortBecameInvalidNotification NSPortDidBecomeInvalidNotification
 
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
 /**
  * Native socket type.
@@ -312,6 +317,10 @@ typedef SOCKET NSSocketNativeHandle;
 
 #endif
 
+#if	defined(__cplusplus)
+}
+#endif
+
 
 #endif /* __NSPort_h_GNUSTEP_BASE_INCLUDE */
 
