@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -24,9 +24,16 @@
 
 #ifndef __NSPredicate_h_GNUSTEP_BASE_INCLUDE
 #define __NSPredicate_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include	<Foundation/NSObject.h>
-#include	<Foundation/NSArray.h>
+#if	OS_API_VERSION(100400, GS_API_LATEST)
+
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSArray.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @interface NSPredicate : NSObject <NSCoding, NSCopying>
 
@@ -47,4 +54,9 @@
 - (NSArray *) filteredArrayUsingPredicate: (NSPredicate *)predicate;
 @end
 
-#endif /* __NSPredicate_h_GNUSTEP_BASE_INCLUDE */
+#if	defined(__cplusplus)
+}
+#endif
+
+#endif	/* 100400 */
+#endif	/* __NSPredicate_h_GNUSTEP_BASE_INCLUDE */
