@@ -7,26 +7,32 @@
  * This file is part of the GNUstep Base Library.
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  * 
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111 USA. */ 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02111 USA.
+ */ 
 
 #ifndef __NSRange_h_GNUSTEP_BASE_INCLUDE
 #define __NSRange_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
 /**** Included Headers *******************************************************/
 
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSException;
 @class NXConstantString;
@@ -35,14 +41,14 @@
 
 #ifndef MAX
 #define MAX(a,b) \
-       ({typeof(a) _MAX_a = (a); typeof(b) _MAX_b = (b);  \
+       ({__typeof__(a) _MAX_a = (a); __typeof__(b) _MAX_b = (b);  \
          _MAX_a > _MAX_b ? _MAX_a : _MAX_b; })
 #define	GS_DEFINED_MAX
 #endif
 
 #ifndef MIN
 #define MIN(a,b) \
-       ({typeof(a) _MIN_a = (a); typeof(b) _MIN_b = (b);  \
+       ({__typeof__(a) _MIN_a = (a); __typeof__(b) _MIN_b = (b);  \
          _MIN_a < _MIN_b ? _MIN_a : _MIN_b; })
 #define	GS_DEFINED_MIN
 #endif
@@ -81,7 +87,7 @@ struct _NSRange
   unsigned int length;
 };
 
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 /** Pointer to an NSRange structure. */
 typedef NSRange *NSRangePointer;
 #endif
@@ -222,7 +228,7 @@ GS_EXPORT NSRange NSRangeFromString(NSString *aString);
 #undef	MIN
 #endif
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 /**
  * To be used inside a method for making sure that a range does not specify
  * anything outside the size of an array/string.  Raises exception if range
@@ -242,4 +248,8 @@ if (INDEX >= OVER) \
                GSNameFromSelector(_cmd), INDEX]
 #endif
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSRange_h_GNUSTEP_BASE_INCLUDE */
