@@ -7,19 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    AutogsdocSource: NSSet.m
    AutogsdocSource: NSCountedSet.m
@@ -28,8 +28,13 @@
 
 #ifndef _NSSet_h_GNUSTEP_BASE_INCLUDE
 #define _NSSet_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSArray, NSString, NSEnumerator, NSDictionary;
 
@@ -39,7 +44,7 @@
 + (id) setWithArray: (NSArray*)objects;
 + (id) setWithObject: (id)anObject;
 + (id) setWithObjects: (id)firstObject, ...;
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 + (id) setWithObjects: (id*)objects
 		count: (unsigned)count;
 #endif
@@ -66,7 +71,7 @@
 
 - (void) makeObjectsPerform: (SEL)aSelector;
 - (void) makeObjectsPerform: (SEL)aSelector withObject: (id)argument;
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (void) makeObjectsPerformSelector: (SEL)aSelector;
 - (void) makeObjectsPerformSelector: (SEL)aSelector withObject: (id)argument;
 #endif
@@ -86,7 +91,7 @@
 - (void) minusSet: (NSSet*)other;
 - (void) removeAllObjects;
 - (void) removeObject: (id)anObject;
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (void) setSet: (NSSet*)other;
 #endif
 - (void) unionSet: (NSSet*)other;
@@ -98,7 +103,7 @@
 
 @end
 
-#ifndef NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /**
  * Utility methods for using a counted set to handle uniquing of objects.
@@ -173,6 +178,10 @@ void	GSUPurge(unsigned count);
  */
 id	GSUSet(id anObject, unsigned count);
 
-#endif /* NO_GNUSTEP */
+#endif	/* GS_API_NONE */
+
+#if	defined(__cplusplus)
+}
+#endif
 
 #endif
