@@ -5,16 +5,16 @@
    Date: 2006
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -23,11 +23,16 @@
 
 #ifndef __NSStream_h_GNUSTEP_BASE_INCLUDE
 #define __NSStream_h_GNUSTEP_BASE_INCLUDE
-
-#include <Foundation/NSObject.h>
+#import	<GNUstepBase/GSVersionMacros.h>
 
 #if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)
 
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 typedef enum {   
   NSStreamStatusNotOpen = 0,   
   NSStreamStatusOpening = 1,
@@ -331,6 +336,10 @@ GS_EXPORT NSString * const NSStreamSOCKSProxyVersionKey;
 - (void) stream: (NSStream*)sStream handleEvent: (NSStreamEvent)anEvent;
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
 
-#endif /* __NSStream_h_GNUSTEP_BASE_INCLUDE */
+#endif	/* 100200 */
+
+#endif	/* __NSStream_h_GNUSTEP_BASE_INCLUDE */
