@@ -7,29 +7,34 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #ifndef __NSTask_h_GNUSTEP_BASE_INCLUDE
 #define __NSTask_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSFileHandle.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSString.h>
+#import	<Foundation/NSArray.h>
+#import	<Foundation/NSDictionary.h>
+#import	<Foundation/NSFileHandle.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @interface NSTask : NSObject <GCFinalization>
 {
@@ -77,7 +82,7 @@
  *	Obtaining task state
  */
 - (BOOL) isRunning;
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (int) processIdentifier;
 #endif
 - (int) terminationStatus;
@@ -87,14 +92,14 @@
  */
 - (void) interrupt;
 - (void) launch;
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) resume;
 - (BOOL) suspend;
 #endif
 - (void) terminate;
 - (void) waitUntilExit;
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 - (BOOL) usePseudoTerminal;
 - (NSString*) validatedLaunchPath;
 #endif
@@ -107,4 +112,8 @@
  */
 GS_EXPORT NSString* const NSTaskDidTerminateNotification;
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSTask_h_GNUSTEP_BASE_INCLUDE */
