@@ -4,25 +4,30 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
   */
 
 #ifndef __NSTimeZone_h_GNUSTEP_BASE_INCLUDE
 #define __NSTimeZone_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class	NSArray;
 @class	NSDate;
@@ -47,13 +52,13 @@
 + (NSArray*) timeZoneArray;
 - (NSArray*) timeZoneDetailArray;
 
-#ifndef	NO_GNUSTEP
+#if	OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 /* Returns an dictionary that maps abbreviations to the array
    containing all the time zone names that use the abbreviation. */
 + (NSDictionary*) abbreviationMap;
 #endif
 
-#ifndef	STRICT_OPENSTEP
+#if	OS_API_VERSION(GS_API_MACOSX, GS_API_NONE)
 + (void) resetSystemTimeZone;
 + (NSTimeZone*) systemTimeZone;
 + (NSTimeZone*) timeZoneWithName: (NSString*)name data: (NSData*)data;
@@ -70,7 +75,7 @@
 - (int) secondsFromGMTForDate: (NSDate*)aDate;
 #endif
 
-#ifndef	STRICT_MACOS_X
+#if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
 - (NSTimeZoneDetail*) timeZoneDetailForDate: (NSDate*)date;
 - (NSString*) timeZoneName;
 #endif
@@ -84,7 +89,7 @@
 + (NSTimeZone*) timeZoneWithAbbreviation: (NSString*)abbreviation;  
 @end
 
-#ifndef	STRICT_MACOS_X
+#if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
 @interface NSTimeZoneDetail : NSTimeZone
 - (BOOL) isDaylightSavingTimeZone;
 - (NSString*) timeZoneAbbreviation;
@@ -92,5 +97,9 @@
 @end
 #endif
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif  /* __NSTimeZone_h_GNUSTEP_BASE_INCLUDE*/
 
