@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -24,11 +24,16 @@
 
 #ifndef __NSURLCache_h_GNUSTEP_BASE_INCLUDE
 #define __NSURLCache_h_GNUSTEP_BASE_INCLUDE
-
-#include <Foundation/NSObject.h>
+#import	<GNUstepBase/GSVersionMacros.h>
 
 #if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
 
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 @class NSData;
 @class NSDictionary;
 @class NSURLRequest;
@@ -189,5 +194,10 @@ typedef enum
 
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
+
+#endif
+
 #endif
