@@ -7,27 +7,32 @@
    This file is part of the GNUstep Library.
    
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 */
 
-#ifndef _NSURLHandle_h__
-#define _NSURLHandle_h__
+#ifndef __NSURLHandle_h_GNUSTEP_BASE_INCLUDE
+#define __NSURLHandle_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#if	OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#ifndef	STRICT_OPENSTEP
+#import	<Foundation/NSObject.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSData;
 @class NSString;
@@ -66,7 +71,7 @@ GS_EXPORT NSString * const NSHTTPPropertyRedirectionHeadersKey;
  */
 GS_EXPORT NSString * const NSHTTPPropertyErrorPageDataKey;
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /**
  * Key for passing to [NSURLHandle]'s <code>propertyForKey..</code> methods to
@@ -210,6 +215,11 @@ typedef enum
 
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
 
 #endif
+
+#endif /* __NSURLHandle_h_GNUSTEP_BASE_INCLUDE */
+
