@@ -7,26 +7,32 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */ 
 
 #ifndef __NSValue_h_GNUSTEP_BASE_INCLUDE
 #define __NSValue_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSGeometry.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSGeometry.h>
+#import	<Foundation/NSRange.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSString;
 
@@ -90,7 +96,7 @@
  */
 + (NSValue*) valueWithSize: (NSSize)size;
 
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 /**
  * Synonym for value:withObjCType: .
  */
@@ -106,7 +112,7 @@
  * both contents and declared type of the two values must match.
  */
 - (BOOL) isEqualToValue: (NSValue*)other;
-#endif
+#endif	/* GS_API_MACOSX */
 
 // Accessing Data 
 
@@ -318,7 +324,7 @@
 - (BOOL) isEqualToNumber: (NSNumber*)otherNumber;
 @end
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /**
  * GNUstep specific (non-standard) additions to the NSNumber class.
@@ -363,4 +369,8 @@ GSNumberInfo	*GSNumberInfoFromObject(NSNumber *o);
 unsigned	GSSmallHash(int n);
 #endif
 
+#if	defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSValue_h_GNUSTEP_BASE_INCLUDE */
