@@ -8,26 +8,27 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 
    <title>NSPathUtilities function reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
 #include "GNUstepBase/preface.h"
 #include "GNUstepBase/GSFunctions.h"
+#include "GNUstepBase/GSCategories.h"
 #include "Foundation/Foundation.h"
 
 NSString *
@@ -40,6 +41,8 @@ GSFindNamedFile(NSArray *paths, NSString *aName, NSString *anExtension)
   NSCParameterAssert(aName != nil);
   NSCParameterAssert(paths != nil);
 
+GSOnceFLog(@"deprecated ... trivial to code directly");
+
   /* make up the name with extension if given */
   if (anExtension != nil)
     {
