@@ -11,21 +11,21 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 
    <title>GSObjCRuntime function and macro reference</title>
-   $Date: 2006-08-05 23:27:44 -0600 (Sat, 05 Aug 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -117,14 +117,6 @@ GSAllocateMutexAt(objc_mutex_t *request)
   objc_mutex_unlock(local_lock);
 }
 
-/**  Deprecated ... use GSObjCFindVariable() */
-BOOL
-GSFindInstanceVariable(id obj, const char *name,
-		       const char **type, unsigned int *size, int *offset)
-{
-  return GSObjCFindVariable(obj, name, type, size, offset);
-}
-
 /**
  * This function is used to locate information about the instance
  * variable of obj called name.  It returns YES if the variable
@@ -274,16 +266,10 @@ GSObjCVariableNames(id obj)
   return array;
 }
 
-/**  Deprecated ... use GSObjCGetVariable() */
-void
-GSGetVariable(id obj, int offset, unsigned int size, void *data)
-{
-  GSObjCGetVariable(obj, offset, size, data);
-}
 /**
  * Gets the value from an instance variable in obj<br />
  * This function performs no checking ... you should use it only where
- * you are providing information from a call to GSFindVariable()
+ * you are providing information from a call to GSObjCFindVariable()
  * and you know that the data area provided is the correct size.
  */
 void
@@ -292,12 +278,6 @@ GSObjCGetVariable(id obj, int offset, unsigned int size, void *data)
   memcpy(data, ((void*)obj) + offset, size);
 }
 
-/**  Deprecated ... use GSObjCSetVariable() */
-void
-GSSetVariable(id obj, int offset, unsigned int size, const void *data)
-{
-  GSObjCSetVariable(obj, offset, size, data);
-}
 /**
  * Sets the value in an instance variable in obj<br />
  * This function performs no checking ... you should use it only where
@@ -1475,13 +1455,6 @@ GSObjCAddClassBehavior(Class receiver, Class behavior)
 #endif
 
 
-/**  Deprecated ... use GSObjCGetVal() */
-id
-GSGetValue(NSObject *self, NSString *key, SEL sel,
-	   const char *type, unsigned size, int offset)
-{
-  return GSObjCGetVal(self, [key UTF8String], sel, type, size, offset);
-}
 /**
  * This is used internally by the key-value coding methods, to get a
  * value from an object either via an accessor method (if sel is
@@ -1801,13 +1774,6 @@ GSObjCGetValue(NSObject *self, NSString *key, SEL sel,
   return GSObjCGetVal(self, [key UTF8String], sel, type, size, offset);
 }
 
-/**  Deprecated ... use GSObjCSetVal() */
-void
-GSSetValue(NSObject *self, NSString *key, id val, SEL sel,
-	   const char *type, unsigned size, int offset)
-{
-  GSObjCSetVal(self, [key UTF8String], val, sel, type, size, offset);
-}
 /**
  * This is used internally by the key-value coding methods, to set a
  * value in an object either via an accessor method (if sel is
