@@ -8,21 +8,21 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2006-08-13 05:25:13 -0600 (Sun, 13 Aug 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -36,6 +36,8 @@
 // For private method _decodeArrayOfObjectsForKey:
 #include "Foundation/NSKeyedArchiver.h"
 
+#include "GSPrivate.h"
+
 static SEL	eqSel;
 static SEL	oaiSel;
 
@@ -54,39 +56,10 @@ static Class	GSInlineArrayClass;
 @interface GSArrayEnumeratorReverse : GSArrayEnumerator
 @end
 
-
-@interface GSArray : NSArray
-{
-@public
-  id		*_contents_array;
-  unsigned	_count;
-}
-@end
-
-@interface GSInlineArray : GSArray
-{
-}
-@end
-
-@interface GSMutableArray : NSMutableArray
-{
-@public
-  id		*_contents_array;
-  unsigned	_count;
-  unsigned	_capacity;
-  int		_grow_factor;
-}
-@end
-
 @interface GSMutableArray (GSArrayBehavior)
 - (void) _raiseRangeExceptionWithIndex: (unsigned)index from: (SEL)sel;
 @end
 
-@interface GSPlaceholderArray : NSArray
-{
-}
-@end
-
 @implementation GSArray
 
 - (void) _raiseRangeExceptionWithIndex: (unsigned)index from: (SEL)sel
