@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -141,7 +141,7 @@ static objc_mutex_t  ff_callback_map_lock = NULL;
 static vacallReturnTypeInfo returnTypeInfo [STATIC_CALLBACK_LIST_SIZE];
 
 /* Function that implements the actual forwarding */
-void
+static void
 GSInvocationCallback(void *callback_data, va_alist args);
 
 /*
@@ -168,7 +168,7 @@ gs_offset(const char *type, int index)
 /* Determines if the structure type can be returned entirely in registers.
    See the avcall or vacall man pages for more info. FIXME: I'm betting
    this won't work if a structure contains another structure */
-int
+static int
 gs_splittable (const char *type)
 {
   int i, numtypes;
@@ -297,7 +297,7 @@ gs_find_by_receiver_best_typed_sel (id receiver, SEL sel)
   Only passes the first part.  Is used for determining
   the return type for the vacall macros.
 */
-void
+static void
 gs_sel_type_to_callback_type (const char *sel_type,
   vacallReturnTypeInfo *vatype)
 {
@@ -771,7 +771,7 @@ gs_protocol_selector(const char *types)
  * information.
  */
 
-void
+static void
 GSInvocationCallback (void *callback_data, va_alist args)
 {
   id			obj;
