@@ -7,18 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 #include "Foundation/NSException.h"
 #include "Foundation/NSCoder.h"
@@ -44,7 +45,7 @@ typedef void (*ffi_closure_fun) (ffi_cif*,void*,void**,void*);
 
 typedef void (*f_fun) ();
 
-void GSFFIInvocationCallback(ffi_cif*, void*, void **, void*);
+static void GSFFIInvocationCallback(ffi_cif*, void*, void **, void*);
 
 /*
  * If we are using the GNU ObjC runtime we could
@@ -407,7 +408,7 @@ gs_protocol_selector(const char *types)
   return NO;
 }
 
-void
+static void
 GSFFIInvocationCallback(ffi_cif *cif, void *retp, void **args, void *user)
 {
   id			obj;
