@@ -7,18 +7,19 @@
    This file is part of the GNUstep Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 */
 
 #include "config.h"
@@ -36,6 +37,7 @@
 #include "Foundation/NSFileHandle.h"
 #include "Foundation/NSDebug.h"
 #include "GNUstepBase/GSMime.h"
+#include "GSPrivate.h"
 
 GS_EXPORT NSString * const GSTelnetNotification;
 GS_EXPORT NSString * const GSTelnetErrorKey;
@@ -972,14 +974,12 @@ static NSLock			*urlLock = nil;
 						      protocol: @"tcp"];
   if (sock == nil)
     {
-      extern int	errno;
-
       /*
        * Tell superclass that the load failed - let it do housekeeping.
        */
       [self backgroundLoadDidFailWithReason: [NSString stringWithFormat:
-	@"Unable to connect to %@:%@ ... %s",
-	host, port, GSLastErrorStr(errno)]];
+	@"Unable to connect to %@:%@ ... %@",
+	host, port, [NSError _last]]];
       return;
     }
   cHandle = [[GSTelnetHandle alloc] initWithHandle: sock isConnected: NO];
