@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -42,6 +42,7 @@
 #include "Foundation/NSProcessInfo.h"
 #include "Foundation/NSUserDefaults.h"
 #include "Foundation/NSDebug.h"
+#include "GSPrivate.h"
 
 #include "../Tools/gdomap.h"
 
@@ -797,7 +798,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == -1)
     {
-      NSLog(@"unable to create socket - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to create socket - %@", [NSError _last]);
       RELEASE(self);
       return nil;
     }
@@ -811,8 +812,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       if (bind(net, (struct sockaddr *)&lsin, sizeof(lsin)) == -1)
 	{
-	  NSLog(@"unable to bind to port %s:%d - %s", inet_ntoa(lsin.sin_addr),
-	    GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+	  NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(lsin.sin_addr),
+	    GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
 	  (void) close(net);
 	  RELEASE(self);
 	  return nil;
@@ -830,9 +831,9 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  if (errno != EINPROGRESS)
 	    {
-	      NSLog(@"unable to make connection to %s:%d - %s",
+	      NSLog(@"unable to make connection to %s:%d - %@",
 		inet_ntoa(sin.sin_addr),
-		GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+		GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
 	      RELEASE(self);
 	      return nil;
 	    }
@@ -896,7 +897,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == -1)
     {
-      NSLog(@"unable to create socket - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to create socket - %@", [NSError _last]);
       RELEASE(self);
       return nil;
     }
@@ -913,8 +914,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (bind(net, (struct sockaddr *)&sin, sizeof(sin)) == -1)
     {
-      NSLog(@"unable to bind to port %s:%d - %s", inet_ntoa(sin.sin_addr),
-	GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+      NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(sin.sin_addr),
+	GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
       (void) close(net);
       RELEASE(self);
       return nil;
@@ -922,7 +923,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (listen(net, 256) == -1)
     {
-      NSLog(@"unable to listen on port - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to listen on port - %@", [NSError _last]);
       (void) close(net);
       RELEASE(self);
       return nil;
@@ -930,7 +931,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(net, (struct sockaddr*)&sin, &size) == -1)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
       (void) close(net);
       RELEASE(self);
       return nil;
@@ -1095,8 +1096,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
       if (fstat(desc, &sbuf) < 0)
 	{
-          NSLog(@"unable to get status of descriptor %d - %s",
-	    desc, GSLastErrorStr(errno));
+          NSLog(@"unable to get status of descriptor %d - %@",
+	    desc, [NSError _last]);
 	}
       else
 	{
@@ -1317,8 +1318,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (len < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to read from descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to read from descriptor - %@",
+                  [NSError _last]];
     }
   return d;
 }
@@ -1342,8 +1343,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (len < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to read from descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to read from descriptor - %@",
+                  [NSError _last]];
     }
   return d;
 }
@@ -1352,51 +1353,34 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   NSMutableData	*d;
   int		got;
+  char		buf[READ_SIZE];
 
   [self checkRead];
   if (isNonBlocking == YES)
     {
       [self setNonBlocking: NO];
     }
-  if (len <= 65536)
-    {
-      char	*buf;
 
-      buf = NSZoneMalloc(NSDefaultMallocZone(), len);
-      d = [NSMutableData dataWithBytesNoCopy: buf length: len];
-      got = [self read: [d mutableBytes] length: len];
-      if (got < 0)
+  d = [NSMutableData dataWithCapacity: len < READ_SIZE ? len : READ_SIZE];
+  do
+    {
+      int	chunk = len > sizeof(buf) ? sizeof(buf) : len;
+
+      got = [self read: buf length: chunk];
+      if (got > 0)
+	{
+	  [d appendBytes: buf length: got];
+	  len -= got;
+	}
+      else if (got < 0)
 	{
 	  [NSException raise: NSFileHandleOperationException
-		      format: @"unable to read from descriptor - %s",
-		      GSLastErrorStr(errno)];
+		      format: @"unable to read from descriptor - %@",
+		      [NSError _last]];
 	}
-      [d setLength: got];
     }
-  else
-    {
-      char	buf[READ_SIZE];
+  while (len > 0 && got > 0);
 
-      d = [NSMutableData dataWithCapacity: 0];
-      do
-	{
-	  int	chunk = len > sizeof(buf) ? sizeof(buf) : len;
-
-	  got = [self read: buf length: chunk];
-	  if (got > 0)
-	    {
-	      [d appendBytes: buf length: got];
-	      len -= got;
-	    }
-	  else if (got < 0)
-	    {
-	      [NSException raise: NSFileHandleOperationException
-			  format: @"unable to read from descriptor - %s",
-			  GSLastErrorStr(errno)];
-	    }
-	}
-      while (len > 0 && got > 0);
-    }
   return d;
 }
 
@@ -1437,8 +1421,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (rval < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to write to descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to write to descriptor - %@",
+                  [NSError _last]];
     }
 }
 
@@ -1543,8 +1527,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
   return (unsigned long long)result;
 }
@@ -1567,8 +1551,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
   return (unsigned long long)result;
 }
@@ -1591,8 +1575,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
 }
 
@@ -1927,8 +1911,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  NSString	*s;
 
-	  s = [NSString stringWithFormat: @"Accept attempt failed - %s",
-		GSLastErrorStr(errno)];
+	  s = [NSString stringWithFormat: @"Accept attempt failed - %@",
+	    [NSError _last]];
 	  [readInfo setObject: s forKey: GSFileHandleNotificationError];
 	}
       else
@@ -1995,8 +1979,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
             {
 	      NSString	*s;
 
-	      s = [NSString stringWithFormat: @"Read attempt failed - %s",
-		    GSLastErrorStr(errno)];
+	      s = [NSString stringWithFormat: @"Read attempt failed - %@",
+		[NSError _last]];
 	      [readInfo setObject: s forKey: GSFileHandleNotificationError];
 	      [self postReadNotification];
 	    }
@@ -2023,16 +2007,26 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (operation == GSFileHandleConnectCompletionNotification
     || operation == GSSOCKSConnect)
     { // Connection attempt completed.
+      extern int errno;
       int	result;
+      int	rval;
       unsigned	len = sizeof(result);
 
-      if (getsockopt(descriptor, SOL_SOCKET, SO_ERROR,
-        (char*)&result, &len) == 0 && result != 0)
+      rval = getsockopt(descriptor, SOL_SOCKET, SO_ERROR, (char*)&result, &len);
+      if (rval != 0)
         {
           NSString	*s;
 
-          s = [NSString stringWithFormat: @"Connect attempt failed - %s",
-		GSLastErrorStr(result)];
+          s = [NSString stringWithFormat: @"Connect attempt failed - %@",
+	    [NSError _last]];
+          [info setObject: s forKey: GSFileHandleNotificationError];
+	}
+      else if (result != 0)
+        {
+          NSString	*s;
+
+          s = [NSString stringWithFormat: @"Connect attempt failed - %@",
+	    [NSError _systemError: result]];
           [info setObject: s forKey: GSFileHandleNotificationError];
         }
       else
@@ -2065,7 +2059,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	          NSString	*s;
 
 	          s = [NSString stringWithFormat:
-	        	@"Write attempt failed - %s", GSLastErrorStr(errno)];
+		    @"Write attempt failed - %@", [NSError _last]];
 	          [info setObject: s forKey: GSFileHandleNotificationError];
 	          [self postWriteNotification];
 	        }
@@ -2141,8 +2135,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	    }
 	  if (fcntl(descriptor, F_SETFL, e) < 0)
 	    {
-	      NSLog(@"unable to set non-blocking mode for %d - %s",
-		descriptor, GSLastErrorStr(errno));
+	      NSLog(@"unable to set non-blocking mode for %d - %@",
+		descriptor, [NSError _last]);
 	    }
 	  else
 	    {
@@ -2151,8 +2145,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	}
       else
 	{
-	  NSLog(@"unable to get non-blocking mode for %d - %s",
-	    descriptor, GSLastErrorStr(errno));
+	  NSLog(@"unable to get non-blocking mode for %d - %@",
+	    descriptor, [NSError _last]);
 	}
     }
 }
@@ -2170,11 +2164,11 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
-      str = [NSString stringWithCString: (char*)inet_ntoa(sin.sin_addr)];
+      str = [NSString stringWithUTF8String: (char*)inet_ntoa(sin.sin_addr)];
     }
   return str;
 }
@@ -2187,7 +2181,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
