@@ -23,16 +23,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -73,7 +73,8 @@
 #include "Foundation/NSZone.h"
 #include "Foundation/NSDebug.h"
 #include "GNUstepBase/GSLocale.h"
-#include "GSFormat.h"
+
+#include "GSPrivate.h"
 
 #include <string.h>		// for strstr()
 #include <sys/stat.h>
@@ -218,10 +219,10 @@ enum
 /* Digits.  */
 
 /* Lower-case digits.  */
-const char _itowa_lower_digits[36]
+static const char _itowa_lower_digits[36]
 	= "0123456789abcdefghijklmnopqrstuvwxyz";
 /* Upper-case digits.  */
-const char _itowa_upper_digits[36]
+static const char _itowa_upper_digits[36]
 	= "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 
@@ -758,9 +759,13 @@ parse_one_spec (const unichar *format, size_t posn, struct printf_spec *spec,
   return nargs;
 }
 
+static inline void GSStrAppendUnichar(GSStr s, unichar u)
+{
+  GSPrivateStrAppendUnichars(s, &u, 1);
+}
 
 #define	outchar(Ch)		GSStrAppendUnichar(s, Ch)
-#define outstring(String, Len)	GSStrAppendUnichars(s, String, Len)
+#define outstring(String, Len)	GSPrivateStrAppendUnichars(s, String, Len)
 
 /* For handling long_double and longlong we use the same flag.  If
    `long' and `long long' are effectively the same type define it to
@@ -798,7 +803,7 @@ static unichar *group_number (unichar *, unichar *, const char *, NSString *);
 
 /* The function itself.  */
 void
-GSFormat (GSStr s, const unichar *format, va_list ap,
+GSPrivateFormat (GSStr s, const unichar *format, va_list ap,
 NSDictionary *locale)
 {
   /* The character used as thousands separator.  */
@@ -1690,9 +1695,10 @@ NSDictionary *locale)
 
     LABEL (form_strerror):
       /* Print description of error ERRNO.  */
-      string =
-	(unichar *) GSLastErrorStr(save_errno);
-      is_long = 0;		/* This is no wide-char string.  */
+      errno = save_errno;
+      string = (unichar *)[[[NSError _last] localizedDescription]
+	cStringUsingEncoding: NSUnicodeStringEncoding];
+      is_long = 1;		/* This is a unicode string.  */
       goto LABEL (print_string);
     LABEL (form_character):
       /* Character.  */
@@ -1735,7 +1741,7 @@ NSDictionary *locale)
 	  {
 	    /* Write "(null)" if there's space.  */
 	    if (prec == -1
-		|| prec >= (int) (sizeof (null) / sizeof (null[0])) - 1)
+	      || prec >= (int) (sizeof (null) / sizeof (null[0])) - 1)
 	      {
 		string = (unichar *) null;
 		len = (sizeof (null) / sizeof (null[0])) - 1;
@@ -1757,8 +1763,8 @@ NSDictionary *locale)
 
 	    if (enc == GSUndefinedEncoding)
 	      {
-	        enc = GetDefEncoding();
-		byteEncoding = GSIsByteEncoding(enc);
+	        enc = [NSString defaultCStringEncoding];
+		byteEncoding = GSPrivateIsByteEncoding(enc);
 	      }
 
 	    len = strlen(str);	// Number of bytes to convert.
