@@ -8,16 +8,16 @@
    This file is part of the GNUstep Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -25,26 +25,27 @@
 
 #include "config.h"
 #include "Foundation/NSArray.h"
-#include "Foundation/NSString.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSValue.h"
+#include "Foundation/NSByteOrder.h"
 #include "Foundation/NSData.h"
+#include "Foundation/NSDebug.h"
+#include "Foundation/NSException.h"
+#include "Foundation/NSFileHandle.h"
+#include "Foundation/NSHost.h"
+#include "Foundation/NSLock.h"
+#include "Foundation/NSMapTable.h"
+#include "Foundation/NSNotification.h"
+#include "Foundation/NSPathUtilities.h"
+#include "Foundation/NSProcessInfo.h"
+#include "Foundation/NSRunLoop.h"
+#include "Foundation/NSString.h"
 #include "Foundation/NSURL.h"
 #include "Foundation/NSURLHandle.h"
-#include "Foundation/NSNotification.h"
-#include "Foundation/NSRunLoop.h"
-#include "Foundation/NSByteOrder.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSFileHandle.h"
-#include "Foundation/NSDebug.h"
-#include "Foundation/NSHost.h"
-#include "Foundation/NSProcessInfo.h"
-#include "Foundation/NSPathUtilities.h"
-#include "Foundation/NSMapTable.h"
+#include "Foundation/NSValue.h"
 #include "GNUstepBase/GSMime.h"
 #include "GNUstepBase/GSLock.h"
 #include "NSCallBacks.h"
 #include "GSURLPrivate.h"
+#include "GSPrivate.h"
 
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -55,6 +56,9 @@
 #ifdef HAVE_SYS_FCNTL_H
 #include <sys/fcntl.h>		// For O_WRONLY, etc
 #endif
+#ifdef	HAVE_SYS_SOCKET_H
+#include <sys/socket.h>		// For MSG_PEEK, etc
+#endif
 
 /*
  * Implement map keys for strings with case insensitive comparisons,
@@ -81,7 +85,7 @@ typedef void (*NSMT_retain_func_t)(NSMapTable *, const void *);
 typedef void (*NSMT_release_func_t)(NSMapTable *, void *);
 typedef NSString *(*NSMT_describe_func_t)(NSMapTable *, const void *);
 
-const NSMapTableKeyCallBacks writeKeyCallBacks =
+static const NSMapTableKeyCallBacks writeKeyCallBacks =
 {
   (NSMT_hash_func_t) _non_retained_id_hash,
   (NSMT_is_equal_func_t) _non_retained_id_is_equal,
@@ -210,7 +214,10 @@ static NSString	*httpVersion = @"1.1";
  */
 @implementation GSHTTPURLHandle
 
+#define	MAX_CACHED	16
+
 static NSMutableDictionary	*urlCache = nil;
+static NSMutableArray		*urlOrder = nil;
 static NSLock			*urlLock = nil;
 
 static Class			sslClass = 0;
@@ -278,7 +285,12 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       //NSLog(@"Lookup for handle for '%@'", page);
       [urlLock lock];
       obj = [urlCache objectForKey: page];
-      AUTORELEASE(RETAIN(obj));
+      if (obj != nil)
+        {
+	  [urlOrder removeObjectIdenticalTo: obj];
+	  [urlOrder addObject: obj];
+          AUTORELEASE(RETAIN(obj));
+	}
       [urlLock unlock];
       //NSLog(@"Found handle %@", obj);
     }
@@ -290,6 +302,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   if (self == [GSHTTPURLHandle class])
     {
       urlCache = [NSMutableDictionary new];
+      urlOrder = [NSMutableArray new];
       urlLock = [GSLazyLock new];
       debugLock = [GSLazyLock new];
       debugFile = [NSString stringWithFormat: @"%@/GSHTTP.%d",
@@ -309,14 +322,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
     {
       NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
 
-      /*
-       * We might be in an idle state with an outstandng read on the
-       * socket, keeping the connection alive, but waiting for the
-       * remote end to drop it.
-       */
-      [nc removeObserver: self
-		    name: NSFileHandleReadCompletionNotification
-		  object: sock];
+      [nc removeObserver: self name: nil object: sock];
       [sock closeFile];
       DESTROY(sock);
     }
@@ -350,10 +356,23 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       connectionState = idle;
       if (cached == YES)
         {
-	  NSString	*page = [newUrl absoluteString];
+	  NSString		*page = [newUrl absoluteString];
+	  GSHTTPURLHandle	*obj;
 
 	  [urlLock lock];
+	  obj = [urlCache objectForKey: page];
 	  [urlCache setObject: self forKey: page];
+	  if (obj != nil)
+	    {
+	      [urlOrder removeObjectIdenticalTo: obj];
+	    }
+	  [urlOrder addObject: self];
+	  while ([urlOrder count] > MAX_CACHED)
+	    {
+	      obj = [urlOrder objectAtIndex: 0];
+	      [urlCache removeObjectForKey: [obj->url absoluteString]];
+	      [urlOrder removeObjectAtIndex: 0];
+	    }
 	  [urlLock unlock];
 	  //NSLog(@"Cache handle %@ for '%@'", self, page);
 	}
@@ -433,20 +452,28 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  NSURLCredential	*cred;
 	  NSString		*method;
 
-	  /*
-	   * Create credential from user and password
-	   * stored in the URL.
+	  /* Create credential from user and password stored in the URL.
+	   * Returns nil if we have no username or password.
 	   */
 	  cred = [[NSURLCredential alloc]
 	    initWithUser: [u user]
 	    password: [u password]
 	    persistence: NSURLCredentialPersistenceForSession];
 
-	  authentication = [GSHTTPAuthentication
-	    authenticationWithCredential: cred
-	    inProtectionSpace: space];
-
-	  RELEASE(cred);
+	  if (cred == nil)
+	    {
+	      authentication = nil;
+	    }
+	  else
+	    {
+	      /* Create authentication from credential ... returns nil if
+	       * we have no credential.
+	       */
+	      authentication = [GSHTTPAuthentication
+		authenticationWithCredential: cred
+		inProtectionSpace: space];
+	      RELEASE(cred);
+	    }
 
 	  method = [request objectForKey: GSHTTPPropertyMethodKey];
 	  if (method == nil)
@@ -464,8 +491,12 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  auth = [authentication authorizationForAuthentication: nil
 							 method: method
 							   path: [u path]];
-
-	  NSMapInsert(wProperties, (void*)@"Authorization", (void*)auth);
+	  /* If authentication is nil then auth will also be nil
+	   */
+	  if (auth != nil)
+	    {
+	      [self writeProperty: auth forKey: @"Authorization"];
+	    }
 	}
     }
 
@@ -512,6 +543,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSDictionary		*dict = [not userInfo];
   NSData		*d;
   NSRange		r;
+  unsigned		readCount;
   BOOL			complete = NO;
 
   RETAIN(self);
@@ -519,6 +551,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
   d = [dict objectForKey: NSFileHandleNotificationDataItem];
   if (debug == YES) debugRead(self, d);
+  readCount = [d length];
 
   if (connectionState == idle)
     {
@@ -529,12 +562,11 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
        */
       if (debug == YES && [d length] != 0)
 	{
-	  NSLog(@"%@ %s Unexpected data from remote!",
-	    NSStringFromSelector(_cmd), keepalive?"K":"");
+	  NSLog(@"%@ %s Unexpected data (%*.*s) from remote!",
+	    NSStringFromSelector(_cmd), keepalive?"K":"",
+	    [d length], [d length], [d bytes]);
 	}
-      [nc removeObserver: self
-		    name: NSFileHandleReadCompletionNotification
-		  object: sock];
+      [nc removeObserver: self name: nil object: sock];
       [sock closeFile];
       DESTROY(sock);
     }
@@ -585,16 +617,16 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  GSMimeHeader	*info;
 	  NSString	*val;
 	  float		ver;
+	  int		code;
 
 	  connectionState = idle;
+	  [nc removeObserver: self name: nil object: sock];
 
 	  ver = [[[document headerNamed: @"http"] value] floatValue];
 	  val = [[document headerNamed: @"connection"] value];
 	  if (ver < 1.1 || (val != nil && [val isEqual: @"close"] == YES))
 	    {
-	      [nc removeObserver: self
-			    name: NSFileHandleReadCompletionNotification
-			  object: sock];
+	      [nc removeObserver: self name: nil object: sock];
 	      [sock closeFile];
 	      DESTROY(sock);
 	    }
@@ -604,7 +636,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	   */
 	  info = [document headerNamed: @"http"];
 	  val = [info objectForKey: NSHTTPPropertyStatusCodeKey];
-	  if ([val intValue] == 401 && self->challenged < 2)
+	  code = [val intValue];
+	  if (code == 401 && self->challenged < 2)
 	    {
 	      GSMimeHeader	*ah;
 
@@ -613,33 +646,48 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 		{
 	          NSURLProtectionSpace	*space;
 		  NSString		*ac;
-		  NSURLCredential	*cred;
 		  GSHTTPAuthentication	*authentication;
 		  NSString		*method;
-		  NSString		*a;
+		  NSString		*auth;
 
 		  ac = [ah value];
 		  space = [GSHTTPAuthentication
 		    protectionSpaceForAuthentication: ac requestURL: url];
+		  if (space == nil)
+		    {
+		      authentication = nil;
+		    }
+		  else
+		    {
+		      NSURLCredential	*cred;
 
-		  /*
-		   * Create credential from user and password
-		   * stored in the URL.
-		   */
-		  cred = [[NSURLCredential alloc]
-		    initWithUser: [url user]
-		    password: [url password]
-		    persistence: NSURLCredentialPersistenceForSession];
+		      /*
+		       * Create credential from user and password
+		       * stored in the URL.
+		       * Returns nil if we have no username or password.
+		       */
+		      cred = [[NSURLCredential alloc]
+			initWithUser: [url user]
+			password: [url password]
+			persistence: NSURLCredentialPersistenceForSession];
 
-		  /*
-		   * Get the digest object and ask it for a header
-		   * to use for authorisation.
-		   */
-		  authentication = [GSHTTPAuthentication
-		    authenticationWithCredential: cred
-		    inProtectionSpace: space];
-
-		  RELEASE(cred);
+		      if (cred == nil)
+		        {
+			  authentication = nil;
+			}
+		      else
+		        {
+			  /*
+			   * Get the digest object and ask it for a header
+			   * to use for authorisation.
+			   * Returns nil if we have no credential.
+			   */
+			  authentication = [GSHTTPAuthentication
+			    authenticationWithCredential: cred
+			    inProtectionSpace: space];
+			  RELEASE(cred);
+			}
+		    }
 
 		  method = [request objectForKey: GSHTTPPropertyMethodKey];
 		  if (method == nil)
@@ -654,12 +702,12 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 			}
 		    }
 
-		  a = [authentication authorizationForAuthentication: ac
-							  method: method
-							    path: [url path]];
-		  if (a != nil)
+		  auth = [authentication authorizationForAuthentication: ac
+		    method: method
+		    path: [url path]];
+		  if (auth != nil)
 		    {
-		      [self writeProperty: a forKey: @"Authorization"];
+		      [self writeProperty: auth forKey: @"Authorization"];
 		      [self _tryLoadInBackground: u];
 		      return;	// Retrying.
 		    }
@@ -688,8 +736,17 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  bodyPos = 0;
 	  DESTROY(wData);
 	  NSResetMapTable(wProperties);
-	  [self didLoadBytes: [d subdataWithRange: r]
-		loadComplete: YES];
+	  if (code >= 200 && code < 300)
+	    {
+	      [self didLoadBytes: [d subdataWithRange: r]
+		    loadComplete: YES];
+	    }
+	  else
+	    {
+	      [self didLoadBytes: [d subdataWithRange: r]
+		    loadComplete: NO];
+	      [self cancelLoadInBackground];
+	    }
 	}
       else
 	{
@@ -705,8 +762,23 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 		    loadComplete: NO];
 	    }
 	}
-      if (sock != nil
-	&& (connectionState == reading || connectionState == idle))
+
+      if (complete == NO && readCount == 0)
+        {
+	  /* The read failed ... dropped, but parsing is not complete.
+	   * The request was sent, so we can't know whether it was
+	   * lost in the network or the remote end received it and
+	   * the response was lost.
+	   */
+	  if (debug == YES)
+	    {
+	      NSLog(@"HTTP response not received - %@", parser);
+	    }
+	  [self endLoadInBackground];
+          [self backgroundLoadDidFailWithReason: @"Response parse failed"];
+	}
+
+      if (sock != nil && connectionState == reading)
 	{
           if ([sock readInProgress] == NO)
 	    {
@@ -780,16 +852,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   if (connectionState != idle)
     {
       NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
-      NSString			*name;
 
-      if (connectionState == connecting)
-	name = GSFileHandleConnectCompletionNotification;
-      else if (connectionState == writing)
-	name = GSFileHandleWriteCompletionNotification;
-      else
-	name = NSFileHandleReadCompletionNotification;
-
-      [nc removeObserver: self name: name object: sock];
+      [nc removeObserver: self name: nil object: sock];
       [sock closeFile];
       DESTROY(sock);
       connectionState = idle;
@@ -989,12 +1053,12 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	   * then we may try again with a new connection.
 	   */
 	  nc = [NSNotificationCenter defaultCenter];
-	  [nc removeObserver: self
-			name: GSFileHandleWriteCompletionNotification
-		      object: sock];
+	  [nc removeObserver: self name: nil object: sock];
 	  [sock closeFile];
 	  DESTROY(sock);
 	  connectionState = idle;
+	  if (debug)
+	    NSLog(@"%@ restart on new connection", NSStringFromSelector(_cmd));
 	  [self _tryLoadInBackground: u];
 	  return;
 	}
@@ -1134,6 +1198,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   [dat setLength: 0];
   RELEASE(document);
   RELEASE(parser);
+  [pageInfo removeAllObjects];
   parser = [GSMimeParser new];
   document = RETAIN([parser mimeDocument]);
 
@@ -1172,6 +1237,77 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       port = @"80";
     }
 
+  /* An existing socket with keepalive may have been closed by the other
+   * end.  The portable way to detect it is to run the runloop once to
+   * allow us to be sent a notification about end-of-file.
+   * On unix systems (google told me it is not reliable on windows) we can
+   * simply peek on the file descriptor for a much more efficient check.
+   */
+  if (sock != nil)
+    {
+#if	defined(__MINGW__)
+      NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
+      NSRunLoop			*loop = [NSRunLoop currentRunLoop];
+      NSFileHandle		*test = RETAIN(sock);
+      
+      if (debug)
+        {
+	  NSLog(@"%@ check for reusable socket", NSStringFromSelector(_cmd));
+	}
+      [nc addObserver: self
+	     selector: @selector(bgdRead:)
+		 name: NSFileHandleReadCompletionNotification
+	       object: test];
+      if ([test readInProgress] == NO)
+	{
+	  [test readInBackgroundAndNotify];
+	}
+      [loop acceptInputForMode: NSDefaultRunLoopMode
+		    beforeDate: nil];
+      [nc removeObserver: self
+		    name: nil
+		  object: test];
+      RELEASE(test);
+#else
+      int fd = [sock fileDescriptor];
+
+      if (debug)
+        {
+	  NSLog(@"%@ check for reusable socket", NSStringFromSelector(_cmd));
+	}
+      if (fd >= 0)
+        {
+	  extern int	errno;
+	  int		result;
+	  unsigned char	c;
+
+#if     !defined(MSG_DONTWAIT)
+#define MSG_DONTWAIT    0
+#endif
+	  result = recv(fd, &c, 1, MSG_PEEK | MSG_DONTWAIT);
+	  if (result == 0 || (result < 0 && errno != EAGAIN && errno != EINTR))
+	    {
+	      DESTROY(sock);
+	    }
+	}
+      else
+        {
+	  DESTROY(sock);
+	}
+#endif
+      if (debug)
+	{
+	  if (sock == nil)
+	    {
+	      NSLog(@"%@ socket closed by remote", NSStringFromSelector(_cmd));
+	    }
+	  else
+	    {
+	      NSLog(@"%@ socket is still open", NSStringFromSelector(_cmd));
+	    }
+	}
+    }
+
   if (sock == nil)
     {
       keepalive = NO;	// New connection
@@ -1260,8 +1396,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	   * Tell superclass that the load failed - let it do housekeeping.
 	   */
 	  [self backgroundLoadDidFailWithReason:
-	    [NSString stringWithFormat: @"Unable to connect to %@:%@ ... %s",
-	    host, port, GSLastErrorStr(errno)]];
+	    [NSString stringWithFormat: @"Unable to connect to %@:%@ ... %@",
+	    host, port, [NSError _last]]];
 	  return;
 	}
       RETAIN(sock);
@@ -1271,7 +1407,11 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 		 name: GSFileHandleConnectCompletionNotification
 	       object: sock];
       connectionState = connecting;
-      if (debug) NSLog(@"%@ start connect", NSStringFromSelector(_cmd));
+      if (debug)
+        {
+          NSLog(@"%@ start connect to %@:%@",
+	    NSStringFromSelector(_cmd), host, port);
+	}
     }
   else
     {
@@ -1285,7 +1425,11 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 		    name: NSFileHandleReadCompletionNotification
 		  object: sock];
 
-      keepalive = YES;	// Reusing a connection.
+      /* Reusing a connection. Set flag to say that it has been kept
+       * alive and we don't know if the other end has dropped it
+       * until we write to it and read some response.
+       */
+      keepalive = YES;
       method = [request objectForKey: GSHTTPPropertyMethodKey];
       if (method == nil)
 	{
