@@ -9,16 +9,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
   
    This library is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
   
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
@@ -167,9 +167,10 @@ static inline void GSeq_normalize(GSeq seq)
 		  notdone = NO;
 		  for (i = 1; i < count; i++)
 		    {
-		      if (uni_cop(*second))
+		      if (GSPrivateUniCop(*second))
 			{
-			  if (uni_cop(*first) > uni_cop(*second))
+			  if (GSPrivateUniCop(*first)
+			    > GSPrivateUniCop(*second))
 			    {
 			      unichar	tmp = *first;
 
@@ -177,7 +178,8 @@ static inline void GSeq_normalize(GSeq seq)
 			      *second = tmp;
 			      notdone = YES;
 			    }
-			  else if (uni_cop(*first) == uni_cop(*second))
+			  else if (GSPrivateUniCop(*first)
+			    == GSPrivateUniCop(*second))
 			    {
 			      if (*first > *second)
 				{
