@@ -12,21 +12,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2006-03-08 04:28:59 -0700 (Wed, 08 Mar 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -45,12 +46,23 @@
 #include "Foundation/NSDebug.h"
 #include "Foundation/NSValue.h"
 #include "Foundation/NSNull.h"
+#include "Foundation/NSUserDefaults.h"
 // For private method _decodeArrayOfObjectsForKey:
 #include "Foundation/NSKeyedArchiver.h"
 #include "GNUstepBase/GSCategories.h"
 #include "GSPrivate.h"
 
-extern BOOL	GSMacOSXCompatiblePropertyLists(void);
+static BOOL GSMacOSXCompatiblePropertyLists(void)
+{
+#if	defined(HAVE_LIBXML)
+  if (GSPrivateDefaultsFlag(NSWriteOldStylePropertyLists) == YES)
+    return NO;
+  return GSPrivateDefaultsFlag(GSMacOSXCompatible);
+#else
+  return NO;
+#endif
+}
+
 extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 
 @interface NSArrayEnumerator : NSEnumerator
@@ -67,19 +79,6 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 
 
 
-@class	GSArray;
-@interface GSArray : NSObject	// Help the compiler
-@end
-@class	GSInlineArray;
-@interface GSInlineArray : NSObject	// Help the compiler
-@end
-@class	GSMutableArray;
-@interface GSMutableArray : NSObject	// Help the compiler
-@end
-@class	GSPlaceholderArray;
-@interface GSPlaceholderArray : NSObject	// Help the compiler
-@end
-
 static Class NSArrayClass;
 static Class GSArrayClass;
 static Class GSInlineArrayClass;
@@ -1202,10 +1201,11 @@ compare(id elem1, id elem2, void* context)
  */
 - (BOOL) writeToFile: (NSString *)path atomically: (BOOL)useAuxiliaryFile
 {
-  NSDictionary	*loc = GSUserDefaultsDictionaryRepresentation();
+  NSDictionary	*loc;
   NSString	*desc = nil;
   NSData	*data;
 
+  loc = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
   if (GSMacOSXCompatiblePropertyLists() == YES)
     {
       GSPropertyListMake(self, loc, YES, NO, 2, &desc);
@@ -1228,10 +1228,11 @@ compare(id elem1, id elem2, void* context)
  */
 - (BOOL) writeToURL: (NSURL *)url atomically: (BOOL)useAuxiliaryFile
 {
-  NSDictionary	*loc = GSUserDefaultsDictionaryRepresentation();
+  NSDictionary	*loc;
   NSString	*desc = nil;
   NSData	*data;
 
+  loc = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
   if (GSMacOSXCompatiblePropertyLists() == YES)
     {
       GSPropertyListMake(self, loc, YES, NO, 2, &desc);
