@@ -8,22 +8,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSAutoreleasePool class reference</title>
-   $Date: 2006-06-17 11:20:22 -0600 (Sat, 17 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -185,7 +185,7 @@ static IMP	initImp;
 {
   unsigned count = 0;
   struct autorelease_array_list *released = _released_head;
-  while (released && released->count)
+  while (released != 0)
     {
       count += released->count;
       released = released->next;
@@ -199,7 +199,7 @@ static IMP	initImp;
   struct autorelease_array_list *released = _released_head;
   unsigned int i;
 
-  while (released && released->count)
+  while (released != 0)
     {
       for (i = 0; i < released->count; i++)
 	if (released->objects[i] == anObject)
@@ -380,7 +380,7 @@ static IMP	initImp;
        * releasing. */
       while (released != 0)
 	{
-	  id	*objects = released->objects;
+	  id	*objects = (id*)(released->objects);
 
 	  for (i = 0; i < released->count; i++)
 	    {
