@@ -8,21 +8,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSCoder class reference</title>
-   $Date: 2006-05-15 06:07:35 -0600 (Mon, 15 May 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -410,105 +411,6 @@
 
 @end
 
-@implementation NSCoder (GNUstep)
-- (void) decodeArrayOfObjCType: (const char*)type
-		         count: (unsigned)count
-			    at: (void*)buf
-		      withName: (id*)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  if (name)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  [self decodeArrayOfObjCType: type count: count at: buf];
-}
-
-/** <override-dummy />
- */
-- (void) decodeIndent
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-}
-
-- (void) decodeObjectAt: (id*)anObject
-	       withName: (id*)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self decodeValueOfObjCType: @encode(id) at: anObject withName: name];
-}
-
-- (void) decodeValueOfCType: (const char*)type
-			 at: (void*)buf
-		   withName: (id*)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self decodeValueOfObjCType: type at: buf withName: name];
-}
-
-- (void) decodeValueOfObjCType: (const char*)type
-			    at: (void*)buf
-		      withName: (id*)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  if (name != 0)
-    {
-      *name = [self decodeObject];
-    }
-  else
-    {
-      (void)[self decodeObject];
-    }
-  [self decodeValueOfObjCType: type at: buf];
-}
-
-- (void) encodeArrayOfObjCType: (const char*)type
-		         count: (unsigned)count
-			    at: (const void*)buf
-		      withName: (id)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self encodeObject: name];
-  [self encodeArrayOfObjCType: type count: count at: buf];
-}
-
-/** <override-dummy />
- */
-- (void) encodeIndent
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-}
-
-- (void) encodeValueOfCType: (const char*)type
-			 at: (const void*)buf
-		   withName: (id)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self encodeValueOfObjCType: type at: buf withName: name];
-}
-
-- (void) encodeValueOfObjCType: (const char*)type
-			    at: (const void*)buf
-		      withName: (id)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self encodeObject: name];
-  [self encodeValueOfObjCType: type at: buf];
-}
-
-- (void) encodeObjectAt: (id*)anObject
-	       withName: (id)name
-{
-  GSOnceMLog(@"Deprecated, use NSKeyedArchiver methods");
-  [self encodeValueOfObjCType: @encode(id) at: anObject withName: name];
-}
-
-@end
-
 
 
 #include	"GSPrivate.h"
