@@ -11,22 +11,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSConnection class reference</title>
-   $Date: 2006-08-25 09:49:01 -0600 (Fri, 25 Aug 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -540,7 +540,7 @@ static NSLock	*cached_proxies_gate = nil;
 
       if (connection_table_gate == nil)
 	{
-	  connection_table_gate = [GSLazyLock new];
+	  connection_table_gate = [GSLazyRecursiveLock new];
 	}
       if (cached_proxies_gate == nil)
 	{
@@ -701,6 +701,7 @@ static NSLock	*cached_proxies_gate = nil;
 {
   if (debug_connection)
     NSLog(@"deallocating %@", self);
+  [self gcFinalize];
   [super dealloc];
 }
 
@@ -1279,19 +1280,24 @@ static NSLock	*cached_proxies_gate = nil;
 
 - (void) release
 {
-  /*
-   *	If this would cause the connection to be deallocated then we
-   *	must perform all necessary work (done in [-gcFinalize]).
-   *	We bracket the code with a retain and release so that any
-   *	retain/release pairs in the code won't cause recursion.
+  /* We lock the connection table while checking, to prevent
+   * another thread from grabbing this connection while we are
+   * checking it.
+   * If we are going to deallocate the object, we first remove
+   * it from the table so that no other thread will find it
+   * and try to use it while it is being deallocated.
    */
-  if ([self retainCount] == 1)
+  M_LOCK(connection_table_gate);
+  if (NSDecrementExtraRefCountWasZero(self))
     {
-      [super retain];
-      [self gcFinalize];
-      [super release];
+      NSHashRemove(connection_table, self);
+      M_UNLOCK(connection_table_gate);
+      [self dealloc];
+    }
+  else
+    {
+      M_UNLOCK(connection_table_gate);
     }
-  [super release];
 }
 
 /**
