@@ -9,22 +9,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSData class reference</title>
-   $Date: 2006-08-06 22:14:58 -0600 (Sun, 06 Aug 2006) $ $Revision$
+   $Date: 2007-10-02 23:37:01 -0600 (Tue, 02 Oct 2007) $ $Revision: 25515 $
    */
 
 /* NOTES	-	Richard Frith-Macdonald 1997
@@ -80,6 +80,7 @@
 #include "Foundation/NSRange.h"
 #include "Foundation/NSURL.h"
 #include "Foundation/NSZone.h"
+#include "GSPrivate.h"
 #include <stdio.h>
 #include <string.h>		/* for memset() */
 #ifdef HAVE_UNISTD_H
@@ -163,8 +164,7 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
 
   if (theFile == 0)		/* We failed to open the file. */
     {
-      NSWarnFLog(@"Open (%@) attempt failed - %s", path,
-      GSLastErrorStr(errno));
+      NSDebugFLog(@"Open (%@) attempt failed - %@", path, [NSError _last]);
       goto failure;
     }
 	
@@ -174,8 +174,8 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
   c = fseek(theFile, 0L, SEEK_END);
   if (c != 0)
     {
-      NSWarnFLog(@"Seek to end of file (%@) failed - %s", path,
-      GSLastErrorStr(errno));
+      NSWarnFLog(@"Seek to end of file (%@) failed - %@", path,
+	[NSError _last]);
       goto failure;
     }
 	
@@ -186,8 +186,7 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
   fileLength = ftell(theFile);
   if (fileLength == -1)
     {
-      NSWarnFLog(@"Ftell on %@ failed - %s", path,
-      GSLastErrorStr(errno));
+      NSWarnFLog(@"Ftell on %@ failed - %@", path, [NSError _last]);
       goto failure;
     }
 	
@@ -198,8 +197,8 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
   c = fseek(theFile, 0L, SEEK_SET);
   if (c != 0)
     {
-      NSWarnFLog(@"Fseek to start of file (%@) failed - %s", path,
-      GSLastErrorStr(errno));
+      NSWarnFLog(@"Fseek to start of file (%@) failed - %@", path,
+	[NSError _last]);
       goto failure;
     }
 	
@@ -224,8 +223,8 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
 	    }
 	  if (tmp == 0)
 	    {
-	      NSLog(@"Malloc failed for file (%@) of length %d - %s", path,
-		fileLength + c, GSLastErrorStr(errno));
+	      NSLog(@"Malloc failed for file (%@) of length %d - %@", path,
+		fileLength + c, [NSError _last]);
 	      goto failure;
 	    }
 	  memcpy(tmp + fileLength, buf, c);
@@ -237,16 +236,16 @@ readContentsOfFile(NSString* path, void** buf, unsigned int* len, NSZone* zone)
       tmp = NSZoneMalloc(zone, fileLength);
       if (tmp == 0)
 	{
-	  NSLog(@"Malloc failed for file (%@) of length %d - %s", path,
-	  fileLength, GSLastErrorStr(errno));
+	  NSLog(@"Malloc failed for file (%@) of length %d - %@", path,
+	    fileLength, [NSError _last]);
 	  goto failure;
 	}
 	    
       c = fread(tmp, 1, fileLength, theFile);
       if (c != (int)fileLength)
 	{
-	  NSWarnFLog(@"read of file (%@) contents failed - %s", path,
-	  GSLastErrorStr(errno));
+	  NSWarnFLog(@"read of file (%@) contents failed - %@", path,
+	    [NSError _last]);
 	  goto failure;
 	}
     }
@@ -852,8 +851,7 @@ failure:
       strcat(thePath, "XXXXXX");
       if ((desc = mkstemp(thePath)) < 0)
 	{
-          NSWarnMLog(@"mkstemp (%s) failed - %s", thePath,
-	    GSLastErrorStr(errno));
+          NSWarnMLog(@"mkstemp (%s) failed - %@", thePath, [NSError _last]);
           goto failure;
 	}
       mask = umask(0);
@@ -880,9 +878,9 @@ failure:
       wcscat(wthePath, L"XXXXXX");
       if (_wmktemp(wthePath) == 0)
 	{
-	  NSWarnMLog(@"mktemp (%@) failed - %s",
-	  [NSString stringWithCharacters:wthePath length:wcslen(wthePath)],
-	  GSLastErrorStr(errno));
+	  NSWarnMLog(@"mktemp (%@) failed - %@",
+	  [NSString stringWithCharacters: wthePath length: wcslen(wthePath)],
+	    [NSError _last]);
 	  goto failure;
 	}
 #else
@@ -890,8 +888,7 @@ failure:
       strcat(thePath, "XXXXXX");
       if (mktemp(thePath) == 0)
 	{
-          NSWarnMLog(@"mktemp (%s) failed - %s", thePath,
-	    GSLastErrorStr(errno));
+          NSWarnMLog(@"mktemp (%s) failed - %@", thePath, [NSError _last]);
           goto failure;
 	}
 #endif
@@ -918,11 +915,11 @@ failure:
       /* Something went wrong; we weren't
        * even able to open the file. */
 #if defined(__MINGW32__)
-      NSWarnMLog(@"Open (%@) failed - %s",
+      NSWarnMLog(@"Open (%@) failed - %@",
 	[NSString stringWithCharacters: wthePath length: wcslen(wthePath)],
-	GSLastErrorStr(errno));
+	  [NSError _last]);
 #else
-      NSWarnMLog(@"Open (%s) failed - %s", thePath, GSLastErrorStr(errno));
+      NSWarnMLog(@"Open (%s) failed - %@", thePath, [NSError _last]);
 #endif
       goto failure;
     }
@@ -936,11 +933,11 @@ failure:
                                  * some reason. */
     {
 #if defined(__MINGW32__)
-      NSWarnMLog(@"Fwrite (%@) failed - %s",
-	[NSString stringWithCharacters:wthePath length:wcslen(wthePath)],
-	GSLastErrorStr(errno));
+      NSWarnMLog(@"Fwrite (%@) failed - %@",
+	[NSString stringWithCharacters: wthePath length: wcslen(wthePath)],
+	[NSError _last]);
 #else
-      NSWarnMLog(@"Fwrite (%s) failed - %s", thePath, GSLastErrorStr(errno));
+      NSWarnMLog(@"Fwrite (%s) failed - %@", thePath, [NSError _last]);
 #endif
       goto failure;
     }
@@ -953,11 +950,11 @@ failure:
                                  * so we need to deal with it. */
     {
 #if defined(__MINGW32__)
-      NSWarnMLog(@"Fclose (%@) failed - %s",
-	[NSString stringWithCharacters:wthePath length:wcslen(wthePath)],
-	GSLastErrorStr(errno));
+      NSWarnMLog(@"Fclose (%@) failed - %@",
+	[NSString stringWithCharacters: wthePath length: wcslen(wthePath)],
+	[NSError _last]);
 #else
-      NSWarnMLog(@"Fclose (%s) failed - %s", thePath, GSLastErrorStr(errno));
+      NSWarnMLog(@"Fclose (%s) failed - %@", thePath, [NSError _last]);
 #endif
       goto failure;
     }
@@ -1038,14 +1035,15 @@ failure:
       if (c != 0)               /* Many things could go wrong, I guess. */
         {
 #if defined(__MINGW32__)
-          NSWarnMLog(@"Rename ('%@' to '%@') failed - %s",
-	    [NSString stringWithCharacters: wthePath length:wcslen(wthePath)],
-	    [NSString stringWithCharacters:
-	    wtheRealPath length:wcslen(wtheRealPath)],
-	    GSLastErrorStr(errno));
+          NSWarnMLog(@"Rename ('%@' to '%@') failed - %@",
+	    [NSString stringWithCharacters: wthePath
+				    length: wcslen(wthePath)],
+	    [NSString stringWithCharacters: wtheRealPath
+				    length: wcslen(wtheRealPath)],
+	    [NSError _last]);
 #else
-	  NSWarnMLog(@"Rename ('%s' to '%s') failed - %s",
-	    thePath, theRealPath, GSLastErrorStr(errno));
+	  NSWarnMLog(@"Rename ('%s' to '%s') failed - %@",
+	    thePath, theRealPath, [NSError _last]);
 #endif
           goto failure;
         }
@@ -1511,7 +1509,7 @@ failure:
 
 - (void) encodeWithCoder: (NSCoder*)coder
 {
-  if([coder allowsKeyedCoding])
+  if ([coder allowsKeyedCoding])
     {
       [coder encodeObject: self];
     }
@@ -1525,7 +1523,7 @@ failure:
 {
   id	obj = nil;
 
-  if([coder allowsKeyedCoding])
+  if ([coder allowsKeyedCoding])
     {
       obj = [coder decodeObject];
     }
@@ -2436,7 +2434,7 @@ failure:
 
 - (Class) classForCoder
 {
-  return dataMalloc;		/* Will not be static data when decoded. */
+  return NSDataAbstract;
 }
 
 /* Basic methods	*/
@@ -2918,7 +2916,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 #endif
   if (fd < 0)
     {
-      NSWarnMLog(@"unable to open %@ - %s", path, GSLastErrorStr(errno));
+      NSWarnMLog(@"unable to open %@ - %@", path, [NSError _last]);
       RELEASE(self);
       return nil;
     }
@@ -2926,7 +2924,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   length = lseek(fd, 0, SEEK_END);
   if (length < 0)
     {
-      NSWarnMLog(@"unable to seek to eof %@ - %s", path, GSLastErrorStr(errno));
+      NSWarnMLog(@"unable to seek to eof %@ - %@", path, [NSError _last]);
       close(fd);
       RELEASE(self);
       return nil;
@@ -2934,7 +2932,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   /* Position at start of file. */
   if (lseek(fd, 0, SEEK_SET) != 0)
     {
-      NSWarnMLog(@"unable to seek to sof %@ - %s", path, GSLastErrorStr(errno));
+      NSWarnMLog(@"unable to seek to sof %@ - %@", path, [NSError _last]);
       close(fd);
       RELEASE(self);
       return nil;
@@ -2942,7 +2940,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   bytes = mmap(0, length, PROT_READ, MAP_SHARED, fd, 0);
   if (bytes == MAP_FAILED)
     {
-      NSWarnMLog(@"mapping failed for %s - %s", path, GSLastErrorStr(errno));
+      NSWarnMLog(@"mapping failed for %s - %@", path, [NSError _last]);
       close(fd);
       RELEASE(self);
       self = [dataMalloc allocWithZone: NSDefaultMallocZone()];
@@ -2969,15 +2967,15 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       struct shmid_ds	buf;
 
       if (shmctl(shmid, IPC_STAT, &buf) < 0)
-        NSLog(@"[NSDataShared -dealloc] shared memory control failed - %s",
-		GSLastErrorStr(errno));
+        NSLog(@"[NSDataShared -dealloc] shared memory control failed - %@",
+	  [NSError _last]);
       else if (buf.shm_nattch == 1)
 	if (shmctl(shmid, IPC_RMID, &buf) < 0)	/* Mark for deletion. */
-          NSLog(@"[NSDataShared -dealloc] shared memory delete failed - %s",
-		GSLastErrorStr(errno));
+          NSLog(@"[NSDataShared -dealloc] shared memory delete failed - %@",
+	    [NSError _last]);
       if (shmdt(bytes) < 0)
-        NSLog(@"[NSDataShared -dealloc] shared memory detach failed - %s",
-		GSLastErrorStr(errno));
+        NSLog(@"[NSDataShared -dealloc] shared memory detach failed - %@",
+	  [NSError _last]);
       bytes = 0;
       length = 0;
       shmid = -1;
@@ -2993,8 +2991,8 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       shmid = shmget(IPC_PRIVATE, bufferSize, IPC_CREAT|VM_RDONLY);
       if (shmid == -1)			/* Created memory? */
 	{
-	  NSLog(@"[-initWithBytes:length:] shared mem get failed for %u - %s",
-	    bufferSize, GSLastErrorStr(errno));
+	  NSLog(@"[-initWithBytes:length:] shared mem get failed for %u - %@",
+	    bufferSize, [NSError _last]);
 	  RELEASE(self);
 	  self = [dataMalloc allocWithZone: NSDefaultMallocZone()];
 	  return [self initWithBytes: aBuffer length: bufferSize];
@@ -3003,8 +3001,8 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
     bytes = shmat(shmid, 0, 0);
     if (bytes == (void*)-1)
       {
-	NSLog(@"[-initWithBytes:length:] shared mem attach failed for %u - %s",
-		  bufferSize, GSLastErrorStr(errno));
+	NSLog(@"[-initWithBytes:length:] shared mem attach failed for %u - %@",
+	  bufferSize, [NSError _last]);
 	bytes = 0;
 	RELEASE(self);
 	self = [dataMalloc allocWithZone: NSDefaultMallocZone()];
@@ -3022,21 +3020,23 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   shmid = anId;
   if (shmctl(shmid, IPC_STAT, &buf) < 0)
     {
-      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory control failed - %s", GSLastErrorStr(errno));
+      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory "
+        @"control failed - %@", [NSError _last]);
       RELEASE(self);	/* Unable to access memory. */
       return nil;
     }
   if (buf.shm_segsz < bufferSize)
     {
-      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory segment too small");
+      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory "
+        @"segment too small");
       RELEASE(self);	/* Memory segment too small. */
       return nil;
     }
   bytes = shmat(shmid, 0, 0);
   if (bytes == (void*)-1)
     {
-      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory attach failed - %s",
-		GSLastErrorStr(errno));
+      NSLog(@"[NSDataShared -initWithShmID:length:] shared memory "
+        @"attach failed - %s", [NSError _last]);
       bytes = 0;
       RELEASE(self);	/* Unable to attach to memory. */
       return nil;
@@ -3070,7 +3070,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (Class) classForCoder
 {
-  return mutableDataMalloc;
+  return NSMutableDataAbstract;
 }
 
 - (id) copy
@@ -3162,7 +3162,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       if (bytes == 0)
 	{
 	  NSLog(@"[NSMutableDataMalloc -initWithCapacity:] out of memory "
-	    @"for %u bytes - %s", size, GSLastErrorStr(errno));
+	    @"for %u bytes - %@", size, [NSError _last]);
 	  RELEASE(self);
 	  return nil;
 	}
@@ -3658,20 +3658,20 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       if (shmctl(shmid, IPC_STAT, &buf) < 0)
 	{
 	  NSLog(@"[NSMutableDataShared -dealloc] shared memory "
-	    @"control failed - %s", GSLastErrorStr(errno));
+	    @"control failed - %@", [NSError _last]);
 	}
       else if (buf.shm_nattch == 1)
 	{
 	  if (shmctl(shmid, IPC_RMID, &buf) < 0)	/* Mark for deletion. */
 	    {
 	      NSLog(@"[NSMutableDataShared -dealloc] shared memory "
-		@"delete failed - %s", GSLastErrorStr(errno));
+		@"delete failed - %@", [NSError _last]);
 	    }
 	}
       if (shmdt(bytes) < 0)
 	{
 	  NSLog(@"[NSMutableDataShared -dealloc] shared memory "
-	    @"detach failed - %s", GSLastErrorStr(errno));
+	    @"detach failed - %@", [NSError _last]);
 	}
       bytes = 0;
       length = 0;
@@ -3695,24 +3695,21 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
 
 - (id) initWithCapacity: (unsigned int)bufferSize
 {
-  int	e;
-
   shmid = shmget(IPC_PRIVATE, bufferSize, IPC_CREAT|VM_ACCESS);
   if (shmid == -1)			/* Created memory? */
     {
       NSLog(@"[NSMutableDataShared -initWithCapacity:] shared memory "
-	@"get failed for %u - %s", bufferSize, GSLastErrorStr(errno));
+	@"get failed for %u - %@", bufferSize, [NSError _last]);
       RELEASE(self);
       self = [mutableDataMalloc allocWithZone: NSDefaultMallocZone()];
       return [self initWithCapacity: bufferSize];
     }
 
   bytes = shmat(shmid, 0, 0);
-  e = errno;
   if (bytes == (void*)-1)
     {
       NSLog(@"[NSMutableDataShared -initWithCapacity:] shared memory "
-	@"attach failed for %u - %s", bufferSize, GSLastErrorStr(e));
+	@"attach failed for %u - %@", bufferSize, [NSError _last]);
       bytes = 0;
       RELEASE(self);
       self = [mutableDataMalloc allocWithZone: NSDefaultMallocZone()];
@@ -3732,7 +3729,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   if (shmctl(shmid, IPC_STAT, &buf) < 0)
     {
       NSLog(@"[NSMutableDataShared -initWithShmID:length:] shared memory "
-	@"control failed - %s", GSLastErrorStr(errno));
+	@"control failed - %@", [NSError _last]);
       RELEASE(self);	/* Unable to access memory. */
       return nil;
     }
@@ -3747,7 +3744,7 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
   if (bytes == (void*)-1)
     {
       NSLog(@"[NSMutableDataShared -initWithShmID:length:] shared memory "
-	@"attach failed - %s", GSLastErrorStr(errno));
+	@"attach failed - %@", [NSError _last]);
       bytes = 0;
       RELEASE(self);	/* Unable to attach to memory. */
       return nil;
@@ -3769,8 +3766,8 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
       if (newid == -1)			/* Created memory? */
 	{
 	  [NSException raise: NSMallocException
-	    format: @"Unable to create shared memory segment (size:%u) - %s.",
-	    size, GSLastErrorStr(errno)];
+	    format: @"Unable to create shared memory segment (size:%u) - %@.",
+	    size, [NSError _last]];
 	}
       tmp = shmat(newid, 0, 0);
       if ((intptr_t)tmp == -1)			/* Attached memory? */
@@ -3786,20 +3783,20 @@ getBytes(void* dst, void* src, unsigned len, unsigned limit, unsigned *pos)
           if (shmctl(shmid, IPC_STAT, &buf) < 0)
 	    {
 	      NSLog(@"[NSMutableDataShared -setCapacity:] shared memory "
-		@"control failed - %s", GSLastErrorStr(errno));
+		@"control failed - %@", [NSError _last]);
 	    }
           else if (buf.shm_nattch == 1)
 	    {
 	      if (shmctl(shmid, IPC_RMID, &buf) < 0)	/* Mark for deletion. */
 		{
 		  NSLog(@"[NSMutableDataShared -setCapacity:] shared memory "
-		    @"delete failed - %s", GSLastErrorStr(errno));
+		    @"delete failed - %@", [NSError _last]);
 		}
 	    }
 	  if (shmdt(bytes) < 0)				/* Detach memory. */
 	    {
               NSLog(@"[NSMutableDataShared -setCapacity:] shared memory "
-		@"detach failed - %s", GSLastErrorStr(errno));
+		@"detach failed - %@", [NSError _last]);
 	    }
 	}
       bytes = tmp;
