@@ -8,21 +8,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSDecimal class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include <math.h>
@@ -236,7 +237,7 @@ GSDecimalCompare(const GSDecimal *leftOperand, const GSDecimal *rightOperand)
 static NSComparisonResult
 NSSimpleCompare(const NSDecimal *leftOperand, const NSDecimal *rightOperand);
 
-void
+static void
 GSDecimalRound(GSDecimal *result, int scale, NSRoundingMode mode)
 {
   int i;
@@ -332,7 +333,7 @@ GSDecimalRound(GSDecimal *result, int scale, NSRoundingMode mode)
   GSDecimalCompact(result);
 }
 
-NSCalculationError
+static NSCalculationError
 GSDecimalNormalize(GSDecimal *n1, GSDecimal *n2, NSRoundingMode mode)
 {
   // Both are valid numbers and the exponents are not equal
@@ -673,7 +674,7 @@ NSDecimalMultiply(NSDecimal *result, const NSDecimal *l, const NSDecimal *r,
   return error;
 }
 
-NSCalculationError
+static NSCalculationError
 GSSimpleDivide(NSDecimal *result, const NSDecimal *l, const NSDecimal *r,
   NSRoundingMode mode);
 
@@ -796,7 +797,7 @@ NSDecimalMultiplyByPowerOf10(NSDecimal *result, const NSDecimal *n, short power,
   return NSCalculationNoError;
 }
 
-NSString*
+static NSString*
 GSDecimalString(const GSDecimal *number, NSDictionary *locale)
 {
   int i;
@@ -902,7 +903,7 @@ NSDecimalMin(NSDecimal *result)
 }
 
 // Give back the value of a NSDecimal as a double
-double
+static double
 GSDecimalDouble(GSDecimal *number)
 {
   double d = 0.0;
@@ -931,7 +932,7 @@ GSDecimalDouble(GSDecimal *number)
 
 
 // Create a NSDecimal with a cMantissa, exponent and a negative flag
-void
+static void
 GSDecimalFromComponents(GSDecimal *result, unsigned long long mantissa,
 			short exponent, BOOL negative)
 {
@@ -963,7 +964,7 @@ GSDecimalFromComponents(GSDecimal *result, unsigned long long mantissa,
 }
 
 // Create a NSDecimal from a string using the local
-void
+static void
 GSDecimalFromString(GSDecimal *result, NSString *numberValue,
 		    NSDictionary *locale)
 {
@@ -1157,7 +1158,7 @@ NSDecimalNormalize(NSDecimal *n1, NSDecimal *n2, NSRoundingMode mode)
 */
 }
 
-NSCalculationError
+static NSCalculationError
 GSSimpleAdd(NSDecimal *result, const NSDecimal *left, const NSDecimal *right,
 	     NSRoundingMode mode)
 {
@@ -1235,7 +1236,7 @@ GSSimpleMultiply(NSDecimal *result, NSDecimal *left, NSDecimal *right, NSRoundin
   return error;
 }
 
-NSCalculationError
+static NSCalculationError
 GSSimpleDivide(NSDecimal *result, const NSDecimal *left, const NSDecimal *right,
 	       NSRoundingMode mode)
 {
@@ -1535,7 +1536,7 @@ GSSimpleMultiply(NSDecimal *result, NSDecimal *l, NSDecimal *r, NSRoundingMode m
   return error;
 }
 
-NSCalculationError
+static NSCalculationError
 GSSimpleDivide(NSDecimal *result, const NSDecimal *l, const NSDecimal *r,
 	       NSRoundingMode mode)
 {
