@@ -8,26 +8,30 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSDecimalNumber class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
-#include "Foundation/NSException.h"
+#include "Foundation/NSCoder.h"
 #include "Foundation/NSDecimal.h"
 #include "Foundation/NSDecimalNumber.h"
+#include "Foundation/NSException.h"
+#include "Foundation/NSPortCoder.h"
+
 #include "GSPrivate.h"
 
 // shared default behavior for NSDecimalNumber class
@@ -260,7 +264,7 @@ static NSDecimalNumber *one;
 - (id) initWithString: (NSString*)numberValue
 {
   return [self initWithString: numberValue
-		       locale: GSUserDefaultsDictionaryRepresentation()];
+    locale: GSPrivateDefaultLocale()];
 }
 
 - (id) initWithString: (NSString*)numberValue
@@ -662,6 +666,32 @@ static NSDecimalNumber *one;
   return [[isa defaultBehavior] scale];
 }
 
+- (Class) classForCoder
+{
+  return [NSDecimalNumber class];
+}
+
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aCoder
+{
+  if ([aCoder isByref] == NO)
+    return self;
+  return [super replacementObjectForPortCoder: aCoder];
+}
+
+- (void) encodeWithCoder: (NSCoder*)coder
+{
+  NSString	*s = [self descriptionWithLocale: nil];
+
+  [coder encodeObject: s];
+}
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  NSString	*s = [coder decodeObject];
+
+  return [self initWithString: s locale: nil];
+}
+
 @end
 
 @implementation NSNumber (NSDecimalNumber)
