@@ -7,21 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSDistributedLock class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -31,6 +32,7 @@
 #include "Foundation/NSException.h"
 #include "Foundation/NSValue.h"
 #include "Foundation/NSDebug.h"
+#include "GSPrivate.h"
 
 #include <fcntl.h>
 
@@ -76,13 +78,13 @@ static NSFileManager	*mgr = nil;
 
       if ([mgr removeFileAtPath: _lockPath handler: nil] == NO)
 	{
-	  const char	*err = GSLastErrorStr(errno);
+	  NSString	*err = [[NSError _last] localizedDescription];
 
 	  attributes = [mgr fileAttributesAtPath: _lockPath traverseLink: YES];
 	  if ([modDate isEqual: [attributes fileModificationDate]] == YES)
 	    {
 	      [NSException raise: NSGenericException
-		      format: @"Failed to remove lock directory '%@' - %s",
+		      format: @"Failed to remove lock directory '%@' - %@",
 		      _lockPath, err];
 	    }
 	}
@@ -201,8 +203,8 @@ static NSFileManager	*mgr = nil;
 				   attributes: attributesToSet];
 	  if (locked == NO)
 	    {
-	      NSLog(@"Failed to create lock directory '%@' - %s",
-		    _lockPath, GSLastErrorStr(errno));
+	      NSLog(@"Failed to create lock directory '%@' - %@",
+		    _lockPath, [NSError _last]);
 	    }
 	}
     }
@@ -257,8 +259,8 @@ static NSFileManager	*mgr = nil;
       if ([mgr removeFileAtPath: _lockPath handler: nil] == NO)
 	{
 	  [NSException raise: NSGenericException
-		      format: @"Failed to remove lock directory '%@' - %s",
-			  _lockPath, GSLastErrorStr(errno)];
+		      format: @"Failed to remove lock directory '%@' - %@",
+			  _lockPath, [NSError _last]];
 	}
     }
   else
