@@ -7,22 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSDistributedNotificationCenter class reference</title>
-   $Date: 2006-06-04 00:42:10 -0600 (Sun, 04 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include	"config.h"
@@ -710,17 +710,29 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 	  if (_type == GSNetworkNotificationCenterType)
 	    {
 	      args = [NSArray arrayWithObjects:
-		@"-GSNetwork", @"YES", nil];
+		@"-GSNetwork", @"YES",
+		@"--auto",
+		nil];
 	    }
 	  else if (_type == GSPublicNotificationCenterType)
 	    {
 	      args = [NSArray arrayWithObjects:
-		@"-GSPublic", @"YES", nil];
+		@"-GSPublic", @"YES",
+		@"--auto",
+		nil];
 	    }
 	  else if ([host length] > 0)
 	    {
 	      args = [NSArray arrayWithObjects:
-		@"-NSHost", host, nil];
+		@"-NSHost", host,
+		@"--auto",
+		nil];
+	    }
+	  else
+	    {
+	      args = [NSArray arrayWithObjects:
+		@"--auto",
+		nil];
 	    }
 	  [NSTask launchedTaskWithLaunchPath: cmd arguments: args];
 
