@@ -7,18 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #include	<Foundation/NSDictionary.h>
@@ -26,11 +27,25 @@
 #include	<Foundation/NSError.h>
 #include	<Foundation/NSCoder.h>
 
+NSString* const NSFilePathErrorKey = @"NSFilePathErrorKey";
 NSString* const NSLocalizedDescriptionKey = @"NSLocalizedDescriptionKey";
+NSString* const NSStringEncodingErrorKey = @"NSStringEncodingErrorKey";
+NSString* const NSURLErrorKey = @"NSURLErrorKey";
 NSString* const NSUnderlyingErrorKey = @"NSUnderlyingErrorKey";
+
+NSString* const NSLocalizedFailureReasonErrorKey
+  = @"NSLocalizedFailureReasonErrorKey";
+NSString* const NSLocalizedRecoveryOptionsErrorKey
+  = @"NSLocalizedRecoveryOptionsErrorKey";
+NSString* const NSLocalizedRecoverySuggestionErrorKey
+  = @"NSLocalizedRecoverySuggestionErrorKey";
+NSString* const NSRecoveryAttempterErrorKey
+  = @"NSRecoveryAttempterErrorKey";
+
 NSString* const NSMACHErrorDomain = @"NSMACHErrorDomain";
 NSString* const NSOSStatusErrorDomain = @"NSOSStatusErrorDomain";
 NSString* const NSPOSIXErrorDomain = @"NSPOSIXErrorDomain";
+NSString* const NSCocoaErrorDomain = @"NSCocoaErrorDomain";
 
 @implementation	NSError
 
@@ -64,6 +79,11 @@ NSString* const NSPOSIXErrorDomain = @"NSPOSIXErrorDomain";
   [super dealloc];
 }
 
+- (NSString*) description
+{
+  return [self localizedDescription];
+}
+
 - (NSString*) domain
 {
   return _domain;
@@ -131,7 +151,7 @@ NSString* const NSPOSIXErrorDomain = @"NSPOSIXErrorDomain";
   return self;
 }
 
-- (NSString *)localizedDescription
+- (NSString *) localizedDescription
 {
   NSString	*desc = [_userInfo objectForKey: NSLocalizedDescriptionKey];
 
@@ -142,6 +162,26 @@ NSString* const NSPOSIXErrorDomain = @"NSPOSIXErrorDomain";
   return desc;
 }
 
+- (NSString *) localizedFailureReason
+{
+  return [_userInfo objectForKey: NSLocalizedFailureReasonErrorKey];
+}
+
+- (NSArray *) localizedRecoveryOptions
+{
+  return [_userInfo objectForKey: NSLocalizedRecoveryOptionsErrorKey];
+}
+
+- (NSString *) localizedRecoverySuggestion
+{
+  return [_userInfo objectForKey: NSLocalizedRecoverySuggestionErrorKey];
+}
+
+- (id) recoveryAttempter
+{
+  return [_userInfo objectForKey: NSRecoveryAttempterErrorKey];
+}
+
 - (NSDictionary*) userInfo
 {
   return _userInfo;
