@@ -7,21 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSFileHandle class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -709,13 +710,12 @@ NSString * const NSFileHandleOperationException
 {
   if (NSFileHandle_ssl_class == 0)
     {
+      NSString  *path;
       NSBundle	*bundle;
-      NSString	*path;
 
-      path = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
-	NSSystemDomainMask, NO) lastObject];
-      path = [path stringByAppendingPathComponent: @"Bundles"];
+      path = [[NSBundle bundleForClass: [NSObject class]] bundlePath];
       path = [path stringByAppendingPathComponent: @"SSL.bundle"];
+
       bundle = [NSBundle bundleWithPath: path];
       NSFileHandle_ssl_class = [bundle principalClass];
       if (NSFileHandle_ssl_class == 0 && bundle != nil)
