@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -68,7 +68,7 @@ static NSHTTPCookieStorage   *storage = nil;
 	  o = (NSHTTPCookieStorage*)
 	    NSAllocateObject(self, 0, NSDefaultMallocZone());
 	  o->_NSHTTPCookieStorageInternal = (Internal*)
-	    NSZoneMalloc(NSDefaultMallocZone(), sizeof(Internal));
+	    NSZoneCalloc(NSDefaultMallocZone(), 1, sizeof(Internal));
 	  inst->_policy = NSHTTPCookieAcceptPolicyAlways;
 	  inst->_cookies = [NSMutableSet new];
 	  storage = o;
@@ -111,6 +111,8 @@ static NSHTTPCookieStorage   *storage = nil;
 
 - (void) setCookie: (NSHTTPCookie *)cookie
 {
+  NSAssert([cookie isKindOfClass: [NSHTTPCookie class]] == YES,
+    NSInvalidArgumentException);
   [this->_cookies addObject: cookie];
 }
 
