@@ -9,21 +9,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSHost class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
   */
 
 #include "config.h"
@@ -204,7 +205,7 @@ static NSMutableDictionary	*_hostCache = nil;
 	  break;
 	}
 
-      h_name = [NSString stringWithCString: entry->h_name];
+      h_name = [NSString stringWithUTF8String: entry->h_name];
       [names addObject: h_name];
 
       if (entry->h_aliases != 0)
@@ -212,7 +213,7 @@ static NSMutableDictionary	*_hostCache = nil;
 	  i = 0;
 	  while ((ptr = entry->h_aliases[i++]) != 0)
 	    {
-	      [names addObject: [NSString stringWithCString: ptr]];
+	      [names addObject: [NSString stringWithUTF8String: ptr]];
 	    }
 	}
       if (entry->h_addr_list != 0)
@@ -223,7 +224,7 @@ static NSMutableDictionary	*_hostCache = nil;
 	      NSString	*addr;
 
 	      memcpy((void*)&in.s_addr, (const void*)ptr, entry->h_length);
-	      addr = [NSString stringWithCString: (char*)inet_ntoa(in)];
+	      addr = [NSString stringWithUTF8String: (char*)inet_ntoa(in)];
 	      [addresses addObject: addr];
 	    }
 	}
