@@ -8,22 +8,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSInvocation class reference</title>
-   $Date: 2006-05-15 06:07:35 -0600 (Mon, 15 May 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "Foundation/NSException.h"
@@ -648,13 +648,13 @@ _arg_addr(NSInvocation *inv, int index)
   char buffer[1024];
 
   snprintf (buffer, 1024, "<%s %p selector: %s target: %s>", \
-                GSClassNameFromObject(self), \
-                self, \
-                _selector ? GSNameFromSelector(_selector) : "nil", \
-                _target ?   GSNameFromClass([_target class]) : "nil" \
-               );
+    GSClassNameFromObject(self), \
+    self, \
+    _selector ? GSNameFromSelector(_selector) : "nil", \
+    _target ?   GSNameFromClass([_target class]) : "nil" \
+   );
 
-  return [NSString stringWithCString: buffer];
+  return [NSString stringWithUTF8String: buffer];
 }
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
