@@ -6,22 +6,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSKeyValueCoding informal protocol reference</title>
-   $Date: 2006-05-15 06:07:35 -0600 (Mon, 15 May 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -38,7 +38,11 @@
 #include "Foundation/NSKeyValueCoding.h"
 #include "Foundation/NSNull.h"
 
-NSString* const NSUndefinedKeyException = @"NSUndefinedKeyException";
+/* For backward compatibility NSUndefinedKeyException is actually the same
+ * as the older NSUnknownKeyException
+ */
+NSString* const NSUnknownKeyException = @"NSUnknownKeyException";
+NSString* const NSUndefinedKeyException = @"NSUnknownKeyException";
 
 
 static void
@@ -107,7 +111,7 @@ SetValueForKey(NSObject *self, id anObject, const char *key, unsigned size)
 	    }
 	  else
 	    {
-	      GSOnceFLog(@"Key-value access using _setKey: isdeprecated:");
+	      GSOnceFLog(@"Key-value access using _setKey: is deprecated:");
 	    }
 	}
     }
@@ -270,28 +274,32 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
     {
       [self unableToSetNilForKey: aKey];
     }
-
-  [NSException raise: NSInvalidArgumentException
-	      format: @"%@ -- %@ 0x%x: Given nil value to set for key \"%@\"",
-    NSStringFromSelector(_cmd), NSStringFromClass([self class]), self, aKey];
+  else
+    {
+      [NSException raise: NSInvalidArgumentException
+        format: @"%@ -- %@ 0x%x: Given nil value to set for key \"%@\"",
+        NSStringFromSelector(_cmd), NSStringFromClass([self class]),
+        self, aKey];
+    }
 }
 
 
 - (void) setValue: (id)anObject forKey: (NSString*)aKey
 {
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		key[size+1];
 
   [aKey getCString: key
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(key);
   SetValueForKey(self, anObject, key, size);
 }
 
 
 - (void) setValue: (id)anObject forKeyPath: (NSString*)aKey
 {
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		buf[size+1];
   unsigned	start = 0;
   unsigned	end = 0;
@@ -299,7 +307,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 
   [aKey getCString: buf
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(buf);
   while (o != nil)
     {
       end = start;
@@ -307,9 +316,9 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 	{
 	  end++;
 	}
-      aKey = [[NSString alloc] initWithBytes:  buf + start
-				      length:  end - start
-				    encoding: NSASCIIStringEncoding];
+      aKey = [[NSString alloc] initWithBytes: buf + start
+				      length: end - start
+				    encoding: NSUTF8StringEncoding];
       AUTORELEASE(aKey);
       if (end >= size)
 	{
@@ -344,7 +353,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
     (anObject ? (id)anObject : (id)@"(nil)"), @"NSTargetObjectUserInfoKey",
     (aKey ? (id)aKey : (id)@"(nil)"), @"NSUnknownUserInfoKey",
     nil];
-  exp = [NSException exceptionWithName: NSInvalidArgumentException
+  exp = [NSException exceptionWithName: NSUndefinedKeyException
 				reason: @"Unable to set nil value for key"
 			      userInfo: dict];
   [exp raise];
@@ -372,7 +381,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
       return [self valueForKey: aKey];
     }
 
-  size = [aKey length];
+  size = [aKey length] * 8;
   if (size > 0)
     {
       SEL		sel = 0;
@@ -387,7 +396,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
       strcpy(buf, "_get");
       [aKey getCString: key
 	     maxLength: size+1
-	      encoding: NSASCIIStringEncoding];
+	      encoding: NSUTF8StringEncoding];
+      size = strlen(key);
       strcpy(&buf[4], key);
       lo = buf[4];
       hi = islower(lo) ? toupper(lo) : lo;
@@ -455,7 +465,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
       return;
     }
 
-  size = [aKey length];
+  size = [aKey length] * 8;
   if (size > 0)
     {
       SEL		sel;
@@ -470,7 +480,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
       strcpy(buf, "_set");
       [aKey getCString: key
 	     maxLength: size+1
-	      encoding: NSASCIIStringEncoding];
+	      encoding: NSUTF8StringEncoding];
+      size = strlen(key);
       strcpy(&buf[4], key);
       lo = buf[4];
       hi = islower(lo) ? toupper(lo) : lo;
@@ -543,13 +554,19 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
   SEL		sel = 0;
   const char	*type = 0;
   int		off;
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		key[size+1];
 
   GSOnceMLog(@"This method is deprecated, use -setValue:forKey:");
+  if (anObject == nil)
+    {
+      [NSException raise: NSInvalidArgumentException
+                  format: @"Attempt to set nil value for key '%@'", aKey];
+    }
   [aKey getCString: key
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(key);
   if (size > 0)
     {
       const char	*name;
@@ -648,7 +665,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 {
   unsigned	size;
 
-  if (aValue == 0 || (size = [aKey length]) == 0)
+  if (aValue == 0 || (size = [aKey length] * 8) == 0)
     {
       [NSException raise: NSInvalidArgumentException format: @"nil argument"];
     }
@@ -661,7 +678,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
       strcpy(name, "validate");
       [aKey getCString: &name[8]
 	     maxLength: size+1
-	      encoding: NSASCIIStringEncoding];
+	      encoding: NSUTF8StringEncoding];
+      size = strlen(&name[8]);
       strcpy(&name[size+8], ":error:");
       if (islower(name[8]))
 	{
@@ -681,7 +699,7 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
             forKeyPath: (NSString*)aKey
                  error: (NSError**)anError
 {
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		buf[size+1];
   unsigned	start = 0;
   unsigned	end = 0;
@@ -689,7 +707,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 
   [aKey getCString: buf
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(buf);
   while (o != nil)
     {
       end = start;
@@ -701,9 +720,9 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 	{
 	  break;
 	}
-      aKey = [[NSString alloc] initWithBytes:  buf + start
-				      length:  end - start
-				    encoding: NSASCIIStringEncoding];
+      aKey = [[NSString alloc] initWithBytes: buf + start
+				      length: end - start
+				    encoding: NSUTF8StringEncoding];
       AUTORELEASE(aKey);
       o = [o valueForKey: aKey];
       start = ++end;
@@ -739,19 +758,20 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 
 - (id) valueForKey: (NSString*)aKey
 {
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		key[size+1];
 
   [aKey getCString: key
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(key);
   return ValueForKey(self, key, size);
 }
 
 
 - (id) valueForKeyPath: (NSString*)aKey
 {
-  unsigned	size = [aKey length];
+  unsigned	size = [aKey length] * 8;
   char		buf[size+1];
   unsigned	start = 0;
   unsigned	end = 0;
@@ -759,7 +779,8 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 
   [aKey getCString: buf
 	 maxLength: size+1
-	  encoding: NSASCIIStringEncoding];
+	  encoding: NSUTF8StringEncoding];
+  size = strlen(buf);
   while (start < size && o != nil)
     {
       end = start;
@@ -767,9 +788,9 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 	{
 	  end++;
 	}
-      aKey = [[NSString alloc] initWithBytes:  buf + start
-				      length:  end - start
-				    encoding: NSASCIIStringEncoding];
+      aKey = [[NSString alloc] initWithBytes: buf + start
+				      length: end - start
+				    encoding: NSUTF8StringEncoding];
       AUTORELEASE(aKey);
       o = [o valueForKey: aKey];
       start = ++end;
