@@ -7,21 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSLog reference</title>
-   $Date: 2006-02-22 02:30:55 -0700 (Wed, 22 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -144,7 +145,7 @@ _NSLog_standard_printf_handler (NSString* message)
 
   OutputDebugStringW(null_terminated_buf);
 
-  if ((GSUserDefaultsFlag(GSLogSyslog) == YES
+  if ((GSPrivateDefaultsFlag(GSLogSyslog) == YES
     || write(_NSLogDescriptor, buf, len) != (int)len) && !IsDebuggerPresent())
     {
       static HANDLE eventloghandle = 0;
@@ -170,7 +171,7 @@ _NSLog_standard_printf_handler (NSString* message)
 #else      
       
 #if	defined(HAVE_SYSLOG)
-  if (GSUserDefaultsFlag(GSLogSyslog) == YES
+  if (GSPrivateDefaultsFlag(GSLogSyslog) == YES
     || write(_NSLogDescriptor, buf, len) != (int)len)
     {
       null_terminated_buf = objc_malloc (sizeof (char) * (len + 1));
@@ -302,9 +303,9 @@ NSLogv (NSString* format, va_list args)
     }
 
 #ifdef	HAVE_SYSLOG
-  if (GSUserDefaultsFlag(GSLogSyslog) == YES)
+  if (GSPrivateDefaultsFlag(GSLogSyslog) == YES)
     {
-      if (GSUserDefaultsFlag(GSLogThread) == YES)
+      if (GSPrivateDefaultsFlag(GSLogThread) == YES)
 	{
 	  prefix = [NSString stringWithFormat: @"[thread:%x] ",
 	    GSCurrentThread()];
@@ -317,7 +318,7 @@ NSLogv (NSString* format, va_list args)
   else
 #endif
     {
-      if (GSUserDefaultsFlag(GSLogThread) == YES)
+      if (GSPrivateDefaultsFlag(GSLogThread) == YES)
 	{
 	  prefix = [NSString
 	    stringWithFormat: @"%@ %@[%d,%x] ",
