@@ -5,22 +5,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSMessagePortNameServer class reference</title>
-   $Date: 2006-05-11 08:26:54 -0600 (Thu, 11 May 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "Foundation/NSPortNameServer.h"
@@ -252,6 +252,10 @@ static void clean_up_names(void)
     }
 }
 
+- (NSPort*) portForName: (NSString*)name
+{
+  return [self portForName: name onHost: nil];
+}
 
 - (NSPort*) portForName: (NSString *)name
 		 onHost: (NSString *)host
@@ -262,10 +266,15 @@ static void clean_up_names(void)
 
   NSDebugLLog(@"NSMessagePort", @"portForName: %@ host: %@", name, host);
 
-  if ([host length] && ![host isEqual: @"*"])
+  if ([host length] > 0)
     {
-      NSDebugLLog(@"NSMessagePort", @"non-local host");
-      return nil;
+      [NSException raise: NSInvalidArgumentException
+		  format: @"Attempt to contact a named host using a "
+	@"message port name server.  This name server can only be used "
+	@"to contact processes owned by the same user on the local host "
+	@"(host name must be an empty string).  To contact processes "
+	@"owned by other users or on other hosts you must use an instance "
+	@"of the NSSocketPortNameServer class."];
     }
 
   path = [[self class] _pathForName: name];
