@@ -11,22 +11,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSNotificationCenter class reference</title>
-   $Date: 2006-03-08 04:28:59 -0700 (Wed, 08 Mar 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 #include "config.h"
@@ -241,12 +241,10 @@ typedef struct NCTbl {
   GSIMapTable		named;		/* Getting named messages only.	*/
   unsigned		lockCount;	/* Count recursive operations.	*/
   GSLazyRecursiveLock	*_lock;		/* Lock out other threads.	*/
-  BOOL			lockingDisabled;
-
-  Observation	*freeList;
-  Observation	**chunks;
-  unsigned	numChunks;
-  GSIMapTable	cache[CACHESIZE];
+  Observation		*freeList;
+  Observation		**chunks;
+  unsigned		numChunks;
+  GSIMapTable		cache[CACHESIZE];
   unsigned short	chunkIndex;
   unsigned short	cacheIndex;
 } NCTable;
@@ -407,16 +405,14 @@ static NCTable *newNCTable(void)
 
 static inline void lockNCTable(NCTable* t)
 {
-  if (t->lockingDisabled == NO)
-    [t->_lock lock];
+  [t->_lock lock];
   t->lockCount++;
 }
 
 static inline void unlockNCTable(NCTable* t)
 {
   t->lockCount--;
-  if (t->lockingDisabled == NO)
-    [t->_lock unlock];
+  [t->_lock unlock];
 }
 
 static void obsFree(Observation *o)
@@ -1057,9 +1053,6 @@ static NSNotificationCenter *default_center = nil;
    */
   unlockNCTable(TABLE);
 
-#if 0
-  NS_DURING
-#endif
   /*
    * Now send all the notifications.
    */
@@ -1069,28 +1062,21 @@ static NSNotificationCenter *default_center = nil;
       o = GSIArrayItemAtIndex(a, count).ext;
       if (o->next != 0)
 	{
-	  (*o->method)(o->observer, o->selector, notification);
+          NS_DURING
+            {
+              (*o->method)(o->observer, o->selector, notification);
+            }
+          NS_HANDLER
+            {
+              NSLog(@"Problem posting notification: %@", localException);
+            }
+          NS_ENDHANDLER
 	}
     }
   lockNCTable(TABLE);
   GSIArrayEmpty(a);
   unlockNCTable(TABLE);
 
-#if 0
-  NS_HANDLER
-    {
-      /*
-       *    If we had a problem - release memory before going on.
-       */
-      lockNCTable(TABLE);
-      GSIArrayEmpty(a);
-      unlockNCTable(TABLE);
-      RELEASE(notification);
-      [localException raise];
-    }
-  NS_ENDHANDLER
-#endif
-
   RELEASE(notification);
 }
 
@@ -1142,32 +1128,3 @@ static NSNotificationCenter *default_center = nil;
 
 @end
 
-@implementation	NSNotificationCenter (GNUstep)
-
-- (BOOL) setLockingDisabled: (BOOL)flag
-{
-  BOOL	old;
-
-  GSOnceMLog(@"This method is deprecated");
-  lockNCTable(TABLE);
-  if (self == default_center)
-    {
-      unlockNCTable(TABLE);
-      [NSException raise: NSInvalidArgumentException
-		  format: @"Can't change locking of default center."];
-    }
-  if (LOCKCOUNT > 1)
-    {
-      unlockNCTable(TABLE);
-      [NSException raise: NSInvalidArgumentException
-		  format: @"Can't change locking during post."];
-    }
-
-  old = TABLE->lockingDisabled;
-  TABLE->lockingDisabled = flag;
-  unlockNCTable(TABLE);
-  return old;
-}
-
-@end
-
