@@ -10,22 +10,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSNotificationQueue class reference</title>
-   $Date: 2006-06-29 12:01:44 -0600 (Thu, 29 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -38,6 +38,7 @@
 #include "Foundation/NSString.h"
 #include "Foundation/NSThread.h"
 
+#include "GSPrivate.h"
 /* NotificationQueueList by Richard Frith-Macdonald
    These objects are used to maintain lists of NSNotificationQueue objects.
    There is one list per NSThread, with the first object in the list stored
@@ -574,7 +575,7 @@ static inline void notifyASAP(NSNotificationQueue *q)
 }
 
 void
-GSNotifyASAP()
+GSPrivateNotifyASAP()
 {
   NotificationQueueList	*item;
 
@@ -613,7 +614,7 @@ static inline void notifyIdle(NSNotificationQueue *q)
 }
 
 void
-GSNotifyIdle()
+GSPrivateNotifyIdle()
 {
   NotificationQueueList	*item;
 
@@ -627,7 +628,7 @@ GSNotifyIdle()
 }
 
 BOOL
-GSNotifyMore()
+GSPrivateNotifyMore()
 {
   NotificationQueueList	*item;
 
