@@ -10,22 +10,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSNumber class reference</title>
-   $Date: 2006-06-04 00:42:10 -0600 (Sun, 04 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 #include <string.h>
@@ -41,6 +41,7 @@
 #include "Foundation/NSObjCRuntime.h"
 
 #include "NSConcreteNumber.h"
+#include "GSPrivate.h"
 
 @interface GSCachedBool : NSBoolNumber
 @end
@@ -152,7 +153,7 @@ GSNumberInfoFromObject(NSNumber *o)
 }
 
 unsigned int
-GSSmallHash(int n)
+GSPrivateSmallHash(int n)
 {
   return smallHashes[n + GS_SMALL];
 }
@@ -827,7 +828,7 @@ static Class	doubleNumberClass;
       switch (info->typeLevel)
 	{
 	  case 0:
-	    return [self boolValue] ? @"YES" : @"NO";
+	    return [self boolValue] ? @"1" : @"0";
 	    break;
 
 	  case 1:
@@ -2428,6 +2429,24 @@ static Class	doubleNumberClass;
   return NO;
 }
 
+- (BOOL) isEqualToValue: (NSValue*)o
+{
+  if (o == self)
+    {
+      return YES;
+    }
+  else if (o == nil)
+    {
+      return NO;
+    }
+  else if (GSObjCIsInstance(o) == YES
+    && GSObjCIsKindOf(GSObjCClass(o), abstractClass))
+    {
+      return [self isEqualToNumber: (NSNumber*)o];
+    }
+  return NO;
+}
+
 /*
  * NSCoding
  */
