@@ -9,21 +9,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSNumberFormatter class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "Foundation/NSAttributedString.h"
@@ -33,7 +34,7 @@
 #include "Foundation/NSNumberFormatter.h"
 #include "Foundation/NSString.h"
 #include "Foundation/NSUserDefaults.h"
-
+#include "Foundation/NSCharacterSet.h"
 
 @implementation NSNumberFormatter
 
@@ -45,7 +46,8 @@
 - (NSAttributedString*) attributedStringForObjectValue: (id)anObject
 				 withDefaultAttributes: (NSDictionary*)attr
 {
-  float val;
+  NSDecimalNumber *zeroNumber = [NSDecimalNumber zero];
+  NSDecimalNumber *nanNumber = [NSDecimalNumber notANumber];
 
   if (anObject == nil)
     {
@@ -55,17 +57,22 @@
     {
       return [self attributedStringForNotANumber];
     }
-  else if ([anObject intValue] == 0)
+  else if ([anObject isEqual: nanNumber])
+    {
+      return [self attributedStringForNotANumber];
+    }
+  else if ([anObject isEqual: zeroNumber])
     {
       return [self attributedStringForZero];
     }
 
-  val = [anObject intValue];
-  if ((val > 0) && (_attributesForPositiveValues))
+  if (([(NSNumber*)anObject compare: zeroNumber] == NSOrderedDescending)
+    && (_attributesForPositiveValues))
     {
       attr = _attributesForPositiveValues;
     }
-  else if ((val < 0) && (_attributesForNegativeValues))
+  else if (([(NSNumber*)anObject compare: zeroNumber] == NSOrderedAscending)
+    && (_attributesForNegativeValues))
     {
       attr = _attributesForNegativeValues;
     }
@@ -187,10 +194,11 @@
       if (range.length != 0)
         {
 	  string = AUTORELEASE([string mutableCopy]);
-	  [(NSMutableString*)string replaceOccurrencesOfString: [self thousandSeparator]
-			     withString: @""
-			     options: 0
-			     range: NSMakeRange(0, [string length])];
+	  [(NSMutableString*)string replaceOccurrencesOfString:
+	    [self thousandSeparator]
+	    withString: @""
+	    options: 0
+	    range: NSMakeRange(0, [string length])];
 	}
     }
 
@@ -223,6 +231,7 @@
   _allowsFloats = YES;
   _decimalSeparator = '.';
   _thousandSeparator = ',';
+  _hasThousandSeparators = YES;
   o = [[NSAttributedString alloc] initWithString: @""];
   [self setAttributedStringForNil: o];
   RELEASE(o);
@@ -239,19 +248,23 @@
     {
       if ([decoder containsValueForKey: @"NS.allowsfloats"])
         {
-	  [self setAllowsFloats: [decoder decodeBoolForKey: @"NS.allowsfloats"]];
+	  [self setAllowsFloats:
+	    [decoder decodeBoolForKey: @"NS.allowsfloats"]];
 	}
       if ([decoder containsValueForKey: @"NS.decimal"])
         {
-	  [self setDecimalSeparator: [decoder decodeObjectForKey: @"NS.decimal"]];
+	  [self setDecimalSeparator:
+	    [decoder decodeObjectForKey: @"NS.decimal"]];
 	}
       if ([decoder containsValueForKey: @"NS.hasthousands"])
         {
-	  [self setHasThousandSeparators: [decoder decodeBoolForKey: @"NS.hasthousands"]];
+	  [self setHasThousandSeparators:
+	    [decoder decodeBoolForKey: @"NS.hasthousands"]];
 	}
       if ([decoder containsValueForKey: @"NS.localized"])
         {
-	  [self setLocalizesFormat: [decoder decodeBoolForKey: @"NS.localized"]];
+	  [self setLocalizesFormat:
+	    [decoder decodeBoolForKey: @"NS.localized"]];
 	}
       if ([decoder containsValueForKey: @"NS.max"])
         {
@@ -263,44 +276,54 @@
 	}
       if ([decoder containsValueForKey: @"NS.nan"])
         {
-	  [self setAttributedStringForNotANumber: [decoder decodeObjectForKey: @"NS.nan"]];
+	  [self setAttributedStringForNotANumber:
+	    [decoder decodeObjectForKey: @"NS.nan"]];
 	}
       if ([decoder containsValueForKey: @"NS.negativeattrs"])
         {
-	  [self setTextAttributesForNegativeValues: [decoder decodeObjectForKey: @"NS.negativeattrs"]];
+	  [self setTextAttributesForNegativeValues:
+	    [decoder decodeObjectForKey: @"NS.negativeattrs"]];
 	}
       if ([decoder containsValueForKey: @"NS.negativeformat"])
         {
-	  [self setNegativeFormat: [decoder decodeObjectForKey: @"NS.negativeformat"]];
+	  [self setNegativeFormat:
+	    [decoder decodeObjectForKey: @"NS.negativeformat"]];
 	}
       if ([decoder containsValueForKey: @"NS.nil"])
         {
-	  [self setAttributedStringForNil: [decoder decodeObjectForKey: @"NS.nil"]];
+	  [self setAttributedStringForNil:
+	    [decoder decodeObjectForKey: @"NS.nil"]];
 	}
       if ([decoder containsValueForKey: @"NS.positiveattrs"])
         {
-	  [self setTextAttributesForPositiveValues: [decoder decodeObjectForKey: @"NS.positiveattrs"]];
+	  [self setTextAttributesForPositiveValues:
+	    [decoder decodeObjectForKey: @"NS.positiveattrs"]];
 	}
       if ([decoder containsValueForKey: @"NS.positiveformat"])
         {
-	  [self setPositiveFormat: [decoder decodeObjectForKey: @"NS.positiveformat"]];
+	  [self setPositiveFormat:
+	    [decoder decodeObjectForKey: @"NS.positiveformat"]];
 	}
       if ([decoder containsValueForKey: @"NS.rounding"])
         {
-	  [self setRoundingBehavior: [decoder decodeObjectForKey: @"NS.rounding"]];
+	  [self setRoundingBehavior:
+	    [decoder decodeObjectForKey: @"NS.rounding"]];
 	}
       if ([decoder containsValueForKey: @"NS.thousand"])
         {
-	  [self setThousandSeparator: [decoder decodeObjectForKey: @"NS.thousand"]];
+	  [self setThousandSeparator:
+	    [decoder decodeObjectForKey: @"NS.thousand"]];
 	}
       if ([decoder containsValueForKey: @"NS.zero"])
         {
-	  [self setAttributedStringForZero: [decoder decodeObjectForKey: @"NS.zero"]];
+	  [self setAttributedStringForZero:
+	    [decoder decodeObjectForKey: @"NS.zero"]];
 	}
     }
   else
     {
-      [decoder decodeValueOfObjCType: @encode(BOOL) at: &_hasThousandSeparators];
+      [decoder decodeValueOfObjCType: @encode(BOOL)
+				  at: &_hasThousandSeparators];
       [decoder decodeValueOfObjCType: @encode(BOOL) at: &_allowsFloats];
       [decoder decodeValueOfObjCType: @encode(BOOL) at: &_localizesFormat];
       [decoder decodeValueOfObjCType: @encode(unichar) at: &_thousandSeparator];
@@ -312,7 +335,8 @@
       [decoder decodeValueOfObjCType: @encode(id) at: &_attributedStringForNil];
       [decoder decodeValueOfObjCType: @encode(id)
 	                          at: &_attributedStringForNotANumber];
-      [decoder decodeValueOfObjCType: @encode(id) at: &_attributedStringForZero];
+      [decoder decodeValueOfObjCType: @encode(id)
+				  at: &_attributedStringForZero];
       [decoder decodeValueOfObjCType: @encode(id) at: &_negativeFormat];
       [decoder decodeValueOfObjCType: @encode(id) at: &_positiveFormat];
       [decoder decodeValueOfObjCType: @encode(id)
@@ -485,27 +509,233 @@
 
 - (NSString*) stringForObjectValue: (id)anObject
 {
-  NSMutableDictionary *locale;
+  NSMutableDictionary	*locale;
+  NSCharacterSet	*formattingCharacters;
+  NSCharacterSet	*placeHolders;
+  NSString		*prefix;
+  NSString		*suffix;
+  NSString		*wholeString;
+  NSString		*fracPad;
+  NSString		*fracPartString;
+  NSMutableString	*intPartString;
+  NSMutableString	*formattedNumber;
+  NSMutableString	*intPad;
+  NSRange		prefixRange;
+  NSRange		decimalPlaceRange;
+  NSRange		suffixRange;
+  NSRange		intPartRange;
+  NSDecimal		representativeDecimal;
+  NSDecimal		roundedDecimal;
+  NSDecimalNumber	*roundedNumber;
+  NSDecimalNumber	*intPart;
+  NSDecimalNumber	*fracPart;
+  int			decimalPlaces = 0;
+  BOOL			displayThousandsSeparators = NO;
+  BOOL			displayFractionalPart = NO;
+  BOOL			negativeNumber = NO;
+  NSString		*useFormat;
+
+  formattingCharacters = [NSCharacterSet
+    characterSetWithCharactersInString: @"0123456789#.,_"];
+  placeHolders = [NSCharacterSet 
+    characterSetWithCharactersInString: @"0123456789#_"];
 
   if (nil == anObject)
     return [[self attributedStringForNil] string];
+  if (![anObject isKindOfClass: [NSNumber class]])
+    return [[self attributedStringForNotANumber] string];
+  if ([anObject isEqual: [NSDecimalNumber notANumber]])
+    return [[self attributedStringForNotANumber] string];
+  if ([anObject isEqual: [NSDecimalNumber zero]])
+    return [[self attributedStringForZero] string];
+  
+  useFormat = _positiveFormat;
+  if ([(NSNumber*)anObject compare: [NSDecimalNumber zero]]
+    == NSOrderedAscending)
+    {
+      useFormat = _negativeFormat;
+      negativeNumber = YES;
+    }
 
-  /* FIXME: This is just a quick hack implementation.  */
-  NSLog(@"NSNumberFormatter-stringForObjectValue:... not fully implemented");
+  // if no format specified, use the same default that Cocoa does
+  if (nil == useFormat)
+    {
+      useFormat = negativeNumber ? @"-#,###.##" : @"#,###.##";
+    }
+
+  prefixRange = [useFormat rangeOfCharacterFromSet: formattingCharacters];
+  if (NSNotFound != prefixRange.location)
+    {
+      prefix = [useFormat substringToIndex: prefixRange.location];
+    }
+  else
+    {
+      prefix = @"";
+    }
 
   locale = [NSMutableDictionary dictionaryWithCapacity: 3];
-  if ([self hasThousandSeparators])
+  [locale setObject: @"" forKey: NSThousandsSeparator];
+  [locale setObject: @"" forKey: NSDecimalSeparator];
+
+  //should also set NSDecimalDigits?
+  
+  if ([self hasThousandSeparators]
+    && (0 != [useFormat rangeOfString:@","].length))
     {
-      [locale setObject: [self thousandSeparator] forKey: NSThousandsSeparator];
+      displayThousandsSeparators = YES;
     }
 
-  if ([self allowsFloats])
+  if ([self allowsFloats]
+    && (NSNotFound != [useFormat rangeOfString:@"." ].location))
     {
-      [locale setObject: [self decimalSeparator] forKey: NSDecimalSeparator];
-      // Should also set: NSDecimalDigits
+      decimalPlaceRange = [useFormat rangeOfString: @"."
+					   options: NSBackwardsSearch];
+      if (NSMaxRange(decimalPlaceRange) == [useFormat length])
+        {
+          decimalPlaces = 0;
+        }
+      else
+        {
+          while ([placeHolders characterIsMember:
+	    [useFormat characterAtIndex: NSMaxRange(decimalPlaceRange)]])
+            {
+              decimalPlaceRange.length++;
+              if (NSMaxRange(decimalPlaceRange) == [useFormat length])
+                break;
+            }
+          decimalPlaces=decimalPlaceRange.length -= 1;
+          decimalPlaceRange.location += 1;
+          fracPad = [useFormat substringWithRange:decimalPlaceRange];
+        } 
+      if (0 != decimalPlaces)
+        displayFractionalPart = YES;
     }
 
-  return [anObject descriptionWithLocale: locale];
+  representativeDecimal = [anObject decimalValue];
+  NSDecimalRound(&roundedDecimal, &representativeDecimal, decimalPlaces,
+    NSRoundPlain);
+  roundedNumber = [NSDecimalNumber decimalNumberWithDecimal: roundedDecimal];
+
+  /* Arguably this fiddling could be done by GSDecimalString() but I
+   * thought better to leave that behaviour as it is and provide the
+   * desired prettification here
+   */
+  if (negativeNumber)
+    roundedNumber = [roundedNumber decimalNumberByMultiplyingBy:
+      (NSDecimalNumber*)[NSDecimalNumber numberWithInt: -1]];
+  intPart = (NSDecimalNumber*)
+    [NSDecimalNumber numberWithInt: [roundedNumber intValue]];
+  fracPart = [roundedNumber decimalNumberBySubtracting: intPart];
+  intPartString
+    = AUTORELEASE([[intPart descriptionWithLocale: locale] mutableCopy]);
+  
+  //sort out the padding for the integer part
+  intPartRange = [useFormat rangeOfCharacterFromSet: placeHolders];
+  if (NSMaxRange(intPartRange) < ([useFormat length] - 1))
+    {
+      while (([placeHolders characterIsMember:
+        [useFormat characterAtIndex: NSMaxRange(intPartRange)]]
+        || [[useFormat substringFromRange:
+          NSMakeRange(NSMaxRange(intPartRange), 1)] isEqual: @","])
+        && NSMaxRange(intPartRange) < [useFormat length] - 1)
+        {
+          intPartRange.length++;
+        }
+    }
+  intPad = [[useFormat substringWithRange: intPartRange] mutableCopy];
+  [intPad replaceOccurrencesOfString: @","
+    withString: @""
+    options: 0
+    range: NSMakeRange(0, [intPad length])];
+  [intPad replaceOccurrencesOfString: @"#"
+    withString: @""
+    options: NSAnchoredSearch
+    range: NSMakeRange(0, [intPad length])];
+  if ([intPad length] > [intPartString length])
+    {
+      NSRange		ipRange;
+
+      ipRange = NSMakeRange(0, [intPad length] - [intPartString length] + 1);
+      [intPartString insertString:
+        [intPad substringWithRange: ipRange] atIndex: 0];
+      [intPartString replaceOccurrencesOfString: @"_"
+	withString: @" "
+	options: 0
+	range: NSMakeRange(0, [intPartString length])];
+      [intPartString replaceOccurrencesOfString: @"#"
+	withString: @"0"
+	options: 0
+	range: NSMakeRange(0, [intPartString length])];
+    }
+  // fix the thousands separators up
+  if (displayThousandsSeparators && [intPartString length] > 3)
+    {
+      int index = [intPartString length];
+
+      while (0 < (index -= 3))
+	{
+	  [intPartString insertString: [self thousandSeparator] atIndex: index];
+	}
+    }
+
+  formattedNumber = [intPartString mutableCopy];
+
+  //fix up the fractional part
+  if (displayFractionalPart)
+    {
+      if (0 != decimalPlaces)
+        {
+	  NSMutableString	*ms;
+
+          fracPart = [fracPart decimalNumberByMultiplyingByPowerOf10:
+	    decimalPlaces];
+          ms = [[fracPart descriptionWithLocale: locale] mutableCopy];
+          [ms replaceOccurrencesOfString: @"0"
+	    withString: @""
+	    options: (NSBackwardsSearch | NSAnchoredSearch)
+	    range: NSMakeRange(0, [ms length])];
+          if ([fracPad length] > [ms length])
+            {
+              NSRange fpRange;
+
+              fpRange = NSMakeRange([ms length],
+	        ([fracPad length] - [ms length]));
+              [ms appendString:
+		[fracPad substringWithRange: fpRange]];
+              [ms replaceOccurrencesOfString: @"#"
+		withString: @""
+		options: (NSBackwardsSearch | NSAnchoredSearch)
+		range: NSMakeRange(0, [ms length])];
+              [ms replaceOccurrencesOfString: @"#"
+		withString: @"0"
+		options: 0
+		range: NSMakeRange(0, [ms length])];
+              [ms replaceOccurrencesOfString: @"_"
+		withString: @" "
+		options: 0
+		range: NSMakeRange(0, [ms length])];
+            }
+	  fracPartString = AUTORELEASE(ms);
+        }
+      else
+        {
+          fracPartString = @"0";
+        }
+      [formattedNumber appendString: [self decimalSeparator]];
+      [formattedNumber appendString: fracPartString];
+    }
+  /*FIXME - the suffix doesn't behave the same as on Mac OS X.
+   * Our suffix is everything which follows the final formatting
+   * character.  Cocoa's suffix is everything which isn't a
+   * formatting character nor in the prefix
+   */
+  suffixRange = [useFormat rangeOfCharacterFromSet: formattingCharacters
+    options: NSBackwardsSearch];
+  suffix = [useFormat substringFromIndex: NSMaxRange(suffixRange)];
+  wholeString = [[prefix stringByAppendingString: formattedNumber]
+    stringByAppendingString: suffix];
+  [formattedNumber release];
+  return wholeString;
 }
 
 - (NSDictionary*) textAttributesForNegativeValues
