@@ -7,20 +7,21 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -28,6 +29,7 @@
 #include "Foundation/NSObject.h"
 #include "Foundation/NSFileHandle.h"
 #include "Foundation/NSDebug.h"
+#include "GSPrivate.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -77,7 +79,7 @@
         }
       else
 	{
-	  NSLog(@"Failed to create pipe ... %s", GSLastErrorStr(errno));
+	  NSLog(@"Failed to create pipe ... %@", [NSError _last]);
 	  DESTROY(self);
 	}
 #else
@@ -95,6 +97,11 @@
           writeHandle = [[NSFileHandle alloc] initWithNativeHandle: writeh
 						    closeOnDealloc: YES];
         }
+      else
+	{
+	  NSLog(@"Failed to create pipe ... %@", [NSError _last]);
+	  DESTROY(self);
+	}
 #endif
     }
   return self;
