@@ -7,26 +7,28 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSPort class reference</title>
-   $Date: 2006-02-11 11:22:57 -0700 (Sat, 11 Feb 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
 #include "Foundation/NSException.h"
 #include "Foundation/NSString.h"
+#include "Foundation/NSNotification.h"
 #include "Foundation/NSNotificationQueue.h"
 #include "Foundation/NSPort.h"
 #include "Foundation/NSPortCoder.h"
@@ -53,8 +55,8 @@
  */
 NSString * const NSPortTimeoutException = @"NSPortTimeoutException";
 
-Class	NSPort_abstract_class;
-Class	NSPort_concrete_class;
+static Class	NSPort_abstract_class;
+static Class	NSPort_concrete_class;
 
 + (id) allocWithZone: (NSZone*)aZone
 {
