@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -1435,11 +1435,11 @@
 	notPredicateWithSubpredicate: [self parseNot]];
     }
 
-  if ([self scanPredicateKeyword:@"TRUEPREDICATE"])
+  if ([self scanPredicateKeyword: @"TRUEPREDICATE"])
     {
       return [NSPredicate predicateWithValue: YES];
     }
-  if ([self scanPredicateKeyword:@"FALSEPREDICATE"])
+  if ([self scanPredicateKeyword: @"FALSEPREDICATE"])
     {
       return [NSPredicate predicateWithValue: NO];
     }
@@ -1451,7 +1451,7 @@
 {
   NSPredicate	*l = [self parseNot];
 
-  while ([self scanPredicateKeyword:@"OR"])
+  while ([self scanPredicateKeyword: @"OR"])
     {
       NSPredicate	*r = [self parseNot];
 
@@ -1483,7 +1483,7 @@
 	}
       else
         {
-	  l = [NSCompoundPredicate andPredicateWithSubpredicates: 
+	  l = [NSCompoundPredicate orPredicateWithSubpredicates: 
 	    [NSArray arrayWithObjects: l, r, nil]];
 	}
     }
@@ -1771,7 +1771,7 @@
 		  [args addObject: [self parseExpression]];
 		}
 
-	      if (![self scanString:@")" intoString:NULL])
+	      if (![self scanString: @")" intoString: NULL])
 	        {
 		  [NSException raise: NSInvalidArgumentException 
 			      format: @"Missing ) in function arguments"];
