@@ -7,16 +7,16 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -44,19 +44,306 @@
 #include "Foundation/NSUserDefaults.h"
 #include "Foundation/NSValue.h"
 #include "Foundation/NSDebug.h"
+#include "Foundation/NSNull.h"
+#include "Foundation/NSXMLParser.h"
 #include "GNUstepBase/Unicode.h"
 
 #include "GSPrivate.h"
 
+@class  GSSloppyXMLParser;
+
 extern BOOL GSScanDouble(unichar*, unsigned, double*);
 
-@class	GSMutableArray;
-@interface GSMutableArray : NSObject	// Help the compiler
-@end
 @class	GSMutableDictionary;
 @interface GSMutableDictionary : NSObject	// Help the compiler
 @end
 
+
+@interface GSXMLPListParser : NSObject
+{
+  NSXMLParser				*theParser;
+  NSMutableString			*value;
+  NSMutableArray			*stack;
+  NSString				*key;
+  BOOL					inArray;
+  BOOL					inDictionary;
+  id					plist;
+  NSPropertyListMutabilityOptions	opts;
+}
+
+- (id) initWithData: (NSData*)data
+	 mutability: (NSPropertyListMutabilityOptions)options;
+- (BOOL) parse;
+- (void) parser: (NSXMLParser *)parser
+  foundCharacters: (NSString *)string;
+- (void) parser: (NSXMLParser *)parser
+  didStartElement: (NSString *)elementName
+  namespaceURI: (NSString *)namespaceURI
+  qualifiedName: (NSString *)qualifiedName
+  attributes: (NSDictionary *)attributeDict;
+- (void) parser: (NSXMLParser *)parser
+  didEndElement: (NSString *)elementName
+  namespaceURI: (NSString *)namespaceURI
+  qualifiedName: (NSString *)qName;
+- (id) result;
+@end
+
+@implementation GSXMLPListParser
+
+- (void) dealloc
+{
+  RELEASE(key);
+  RELEASE(stack);
+  RELEASE(plist);
+  RELEASE(value);
+  RELEASE(theParser);
+  [super dealloc];
+}
+
+- (id) initWithData: (NSData*)data
+	 mutability: (NSPropertyListMutabilityOptions)options
+{
+  if ((self = [super init]) != nil)
+    {
+      stack = [[NSMutableArray alloc] initWithCapacity: 10];
+      theParser = [[GSSloppyXMLParser alloc] initWithData: data];
+      [theParser setDelegate: self];
+      opts = options;
+    }
+  return self;
+}
+
+- (void) parser: (NSXMLParser *)parser
+  foundCharacters: (NSString *)string
+{
+  string = [string stringByTrimmingSpaces];
+  if ([string length] > 0)
+    {
+      if (value == nil)
+        {
+          value = [[NSMutableString alloc] initWithCapacity: 50];
+        }
+      [value appendString: string];
+    }
+}
+
+- (void) parser: (NSXMLParser *)parser
+  didStartElement: (NSString *)elementName
+  namespaceURI: (NSString *)namespaceURI
+  qualifiedName: (NSString *)qualifiedName
+  attributes: (NSDictionary *)attributeDict
+{
+  if ([elementName isEqualToString: @"dict"] == YES)
+    {
+      NSMutableDictionary	*d;
+
+      if (key == nil)
+        {
+          key = RETAIN([NSNull null]);
+        }
+      [stack addObject: key];
+      DESTROY(key);
+      d = [[NSMutableDictionary alloc] initWithCapacity: 10];
+      [stack addObject: d];
+      RELEASE(d);
+      inDictionary = YES;
+      inArray = NO;
+    }
+  else if ([elementName isEqualToString: @"array"] == YES)
+    {
+      NSMutableArray	*a;
+
+      if (key == nil)
+        {
+          key = RETAIN([NSNull null]);
+        }
+      [stack addObject: key];
+      DESTROY(key);
+      a = [[NSMutableArray alloc] initWithCapacity: 10];
+      [stack addObject: a];
+      RELEASE(a);
+      inArray = YES;
+      inDictionary = NO;
+    }
+}
+
+- (void) parser: (NSXMLParser *)parser
+  didEndElement: (NSString *)elementName
+  namespaceURI: (NSString *)namespaceURI
+  qualifiedName: (NSString *)qName
+{
+  BOOL	inContainer = NO;
+
+  if ([elementName isEqualToString: @"dict"] == YES)
+    {
+      inContainer = YES;
+    }
+  if ([elementName isEqualToString: @"array"] == YES)
+    {
+      inContainer = YES;
+    }
+
+  if (inContainer)
+    {
+      if (opts != NSPropertyListImmutable)
+	{
+	  ASSIGN(plist, [stack lastObject]);
+	}
+      else
+        {
+	  ASSIGN(plist, [[stack lastObject] makeImmutableCopyOnFail: NO]);
+	}
+      [stack removeLastObject];
+      inArray = NO;
+      inDictionary = NO;
+      ASSIGN(key, [stack lastObject]);
+      [stack removeLastObject];
+      if ((id)key == (id)[NSNull null])
+        {
+          DESTROY(key);
+        }
+      if ([stack count] > 0)
+        {
+	  id	last;
+
+	  last = [stack lastObject];
+	  if ([last isKindOfClass: [NSArray class]] == YES)
+	    {
+	      inArray = YES;
+	    }
+	  else if ([last isKindOfClass: [NSDictionary class]] == YES)
+	    {
+	      inDictionary = YES;
+	    }
+	}
+    }
+  else if ([elementName isEqualToString: @"key"] == YES)
+    {
+      ASSIGN(key, [value makeImmutableCopyOnFail: NO]);
+      DESTROY(value);
+      return;
+    }
+  else if ([elementName isEqualToString: @"data"])
+    {
+      NSData	*d;
+
+      d = [GSMimeDocument decodeBase64:
+	     [value dataUsingEncoding: NSASCIIStringEncoding]];
+      if (opts == NSPropertyListMutableContainersAndLeaves)
+	{
+	  d = AUTORELEASE([d mutableCopy]);
+	}
+      ASSIGN(plist, d);
+      if (d == nil)
+	{
+	  [parser abortParsing];
+	  return;
+	}
+    }
+  else if ([elementName isEqualToString: @"date"])
+    {
+      id	result;
+
+      if ([value hasSuffix: @"Z"] == YES && [value length] == 20)
+	{
+	  result = [NSCalendarDate dateWithString: value
+				   calendarFormat: @"%Y-%m-%dT%H:%M:%SZ"];
+	}
+      else
+	{
+	  result = [NSCalendarDate dateWithString: value
+				   calendarFormat: @"%Y-%m-%d %H:%M:%S %z"];
+	}
+      ASSIGN(plist, result);
+    }
+  else if ([elementName isEqualToString: @"string"])
+    {
+      id	o;
+
+      if (opts == NSPropertyListMutableContainersAndLeaves)
+        {
+	  if (value == nil)
+	    {
+	      o = [NSMutableString string];
+	    }
+	  else
+	    {
+	      o = value;
+	    }
+	}
+      else
+        {
+	  if (value == nil)
+	    {
+	      o = @"";
+	    }
+	  else
+	    {
+	      o = [value makeImmutableCopyOnFail: NO];
+	    }
+	}
+      ASSIGN(plist, o);
+    }
+  else if ([elementName isEqualToString: @"integer"])
+    {
+      ASSIGN(plist, [NSNumber numberWithInt: [value intValue]]);
+    }
+  else if ([elementName isEqualToString: @"real"])
+    {
+      ASSIGN(plist, [NSNumber numberWithDouble: [value doubleValue]]);
+    }
+  else if ([elementName isEqualToString: @"true"])
+    {
+      ASSIGN(plist, [NSNumber numberWithBool: YES]);
+    }
+  else if ([elementName isEqualToString: @"false"])
+    {
+      ASSIGN(plist, [NSNumber numberWithBool: NO]);
+    }
+  else if ([elementName isEqualToString: @"plist"])
+    {
+      DESTROY(value);
+      return;
+    }
+  else // invalid tag
+    {
+      NSLog(@"unrecognized tag <%@>", elementName);
+      [parser abortParsing];
+      return;
+    }
+
+  if (inArray == YES)
+    {
+      [[stack lastObject] addObject: plist];
+    }
+  else if (inDictionary == YES)
+    {
+      if (key == nil)
+        {
+	  [parser abortParsing];
+	  return;
+	}
+      [(NSMutableDictionary*)[stack lastObject] setObject: plist forKey: key];
+      DESTROY(key);
+    }
+  DESTROY(value);
+}
+
+- (BOOL) parse
+{
+  return [theParser parse];
+}
+
+- (id) result
+{
+  return plist;
+}
+
+@end
+
+
+
+
 @interface GSBinaryPLParser : NSObject
 {
   NSPropertyListMutabilityOptions	mutability;
@@ -83,13 +370,13 @@ extern BOOL GSScanDouble(unichar*, unsigned, double*);
   id root;
 
   // Number of bytes per object table index
-  unsigned int index_size;  
+  unsigned int index_size;
   // Number of bytes per object table entry
   unsigned int offset_size;
 
   unsigned int table_start;
-  unsigned int table_size;  
-  unsigned int *table;  
+  unsigned int table_size;
+  unsigned int *table;
 }
 
 + (void) serializePropertyList: (id)aPropertyList intoData: (NSMutableData *)destination;
@@ -1635,7 +1922,8 @@ OAppend(id obj, NSDictionary *loc, unsigned lev, unsigned step,
 	  else if (x == NSPropertyListGNUstepFormat)
 	    {
 	      [dest appendBytes: "<*I" length: 3];
-	      PString([obj stringValue], dest);
+	      [dest appendData:
+	        [[obj stringValue] dataUsingEncoding: NSASCIIStringEncoding]];
 	      [dest appendBytes: ">" length: 1];
 	    }
 	  else
@@ -1654,7 +1942,8 @@ OAppend(id obj, NSDictionary *loc, unsigned lev, unsigned step,
 	  else if (x == NSPropertyListGNUstepFormat)
 	    {
 	      [dest appendBytes: "<*R" length: 3];
-	      PString([obj stringValue], dest);
+	      [dest appendData:
+	        [[obj stringValue] dataUsingEncoding: NSASCIIStringEncoding]];
 	      [dest appendBytes: ">" length: 1];
 	    }
 	  else
@@ -1878,16 +2167,35 @@ OAppend(id obj, NSDictionary *loc, unsigned lev, unsigned step,
 	  [keyArray getObjects: keys];
 	}
 
-      for (i = 0; i < numKeys; i++)
-	{
-	  if (GSObjCClass(keys[i]) == lastClass)
-	    continue;
-	  if ([keys[i] respondsToSelector: @selector(compare:)] == NO)
+      if (x == NSPropertyListXMLFormat_v1_0)
+        {
+	  /* This format can only use strings as keys.
+	   */
+	  lastClass = [NSString class];
+	  for (i = 0; i < numKeys; i++)
 	    {
-	      canCompare = NO;
-	      break;
+	      if ([keys[i] isKindOfClass: lastClass] == NO)
+	        {
+		  [NSException raise: NSInvalidArgumentException
+		    format: @"Bad key in property list: '%@'", keys[i]];
+		}
+	    }
+	}
+      else
+	{
+	  /* All keys must respond to -compare: for sorting.
+	   */
+	  for (i = 0; i < numKeys; i++)
+	    {
+	      if (GSObjCClass(keys[i]) == lastClass)
+		continue;
+	      if ([keys[i] respondsToSelector: @selector(compare:)] == NO)
+		{
+		  canCompare = NO;
+		  break;
+		}
+	      lastClass = GSObjCClass(keys[i]);
 	    }
-	  lastClass = GSObjCClass(keys[i]);
 	}
 
       if (canCompare == YES)
@@ -2071,7 +2379,7 @@ static BOOL	classInitialized = NO;
     {
       classInitialized = YES;
 
-#ifdef	HAVE_LIBXML
+#if	HAVE_LIBXML
       /*
        * Cache XML node information.
        */
@@ -2168,7 +2476,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
     {
       style = NSPropertyListXMLFormat_v1_0;
     }
-  else if (GSUserDefaultsFlag(NSWriteOldStylePropertyLists))
+  else if (GSPrivateDefaultsFlag(NSWriteOldStylePropertyLists) == YES)
     {
       style = NSPropertyListOpenStepFormat;
     }
@@ -2217,10 +2525,10 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 
       case NSPropertyListXMLFormat_v1_0:
 	return YES;
-	
+
       case NSPropertyListBinaryFormat_v1_0:
 	return YES;
-	
+
       default:
 	[NSException raise: NSInvalidArgumentException
 		    format: @"[%@ +%@]: unsupported format",
@@ -2279,9 +2587,6 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 	    {
 	      // It begins with '<?' so it is xml
 	      format = NSPropertyListXMLFormat_v1_0;
-#ifndef	HAVE_LIBXML
-	      error = @"XML format not supported ... XML support not present.";
-#endif
 	    }
 	  else
 	    {
@@ -2295,15 +2600,16 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
     {
       switch (format)
 	{
-#ifdef	HAVE_LIBXML
 	  case NSPropertyListXMLFormat_v1_0:
 	    {
+#if	HAVE_LIBXML
 	      GSXMLParser	*parser;
 	      GSXMLNode		*node;
 
 	      parser = [GSXMLParser parser];
 	      [parser substituteEntities: YES];
 	      [parser doValidityChecking: YES];
+              [parser saveMessages: YES];
 	      if ([parser parse: data] == NO || [parser parse: nil] == NO)
 		{
 		  error = @"failed to parse as valid XML matching DTD";
@@ -2317,9 +2623,28 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 		{
 		  result = nodeToObject([node firstChild], anOption, &error);
 		}
+#endif
+	      /* The libxml based parser is stricter than the fallback
+	       * parser, so if parsing failed using that, we can try again.
+	       */
+	      if (result == nil)
+	        {
+		  GSXMLPListParser *parser;
+
+		  parser = [GSXMLPListParser alloc];
+		  parser = AUTORELEASE([parser initWithData: data
+						 mutability: anOption]);
+		  if ([parser parse] == YES)
+		    {
+		      result = AUTORELEASE(RETAIN([parser result]));
+		    }
+		  else if (error == nil)
+		    {
+		      error = @"failed to parse as XML property list";
+		    }
+		}
 	    }
 	    break;
-#endif
 
 	  case NSPropertyListOpenStepFormat:
 	    {
@@ -2537,7 +2862,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
   if (offset_size == 1)
     {
       unsigned char offset;
-	
+
       [data getBytes: &offset range: NSMakeRange(table_start + index, 1)];
 
       return offset;
@@ -2545,7 +2870,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
   else if (offset_size == 2)
     {
       unsigned short offset;
-	
+
       [data getBytes: &offset range: NSMakeRange(table_start + 2*index, 2)];
 
       return NSSwapBigShortToHost(offset);
@@ -2556,7 +2881,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       int i;
       unsigned long num = 0;
       NSRange	r;
-	
+
       r = NSMakeRange(table_start + offset_size*index, offset_size);
       [data getBytes: &buffer range: r];
       for (i = 0; i < offset_size; i++)
@@ -2592,7 +2917,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       unsigned char buffer[index_size];
       int i;
       unsigned num = 0;
-	
+
       [data getBytes: &buffer range: NSMakeRange(*counter, index_size)];
       *counter += index_size;
       for (i = 0; i < index_size; i++)
@@ -2629,11 +2954,11 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
     }
   else if ((c > 0x11) && (c <= 0x13))
     {
-      unsigned len = 1 << (c - 0x10);
+      unsigned len = c - 0x0f;
       unsigned char buffer[len];
       int i;
       unsigned long num = 0;
-	
+
       [data getBytes: &buffer range: NSMakeRange(*counter, len)];
       *counter += len;
       for (i = 0; i < len; i++)
@@ -2703,7 +3028,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
         {
 	  result = [NSNumber numberWithUnsignedInt: (unsigned int)num];
 	}
-      else 
+      else
         {
 	  result = [NSNumber numberWithUnsignedLongLong: num];
 	}
@@ -3069,7 +3394,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
     }
 
   table = objc_malloc(table_size * sizeof(int));
-  
+
   objectsToDoList = [[NSMutableArray alloc] init];
   objectList = [[NSMutableArray alloc] init];
 
@@ -3106,7 +3431,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 - (void) markOffset: (unsigned int) offset for: (id)object
 {
   unsigned int oid;
-	  
+
   oid = [objectList indexOfObject: object];
   if (oid == NSNotFound)
     {
@@ -3129,10 +3454,10 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
   unsigned int i;
   unsigned char *buffer;
   unsigned int last_offset;
-  
+
   table_start = [dest length];
-  // This is a bit too much, as the length 
-  // of the last object is added. 
+  // This is a bit too much, as the length
+  // of the last object is added.
   last_offset = table_start;
 
   if (last_offset < 256)
@@ -3167,7 +3492,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       for (i = 0; i < len; i++)
         {
 	  unsigned char ci;
-	  
+
 	  ci = table[i];
 	  buffer[i] = ci;
 	}
@@ -3177,7 +3502,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       for (i = 0; i < len; i++)
         {
 	  unsigned short si;
-	  
+
 	  si = table[i];
 	  buffer[2 * i] = (si >> 8);
 	  buffer[2 * i + 1] = si % 256;
@@ -3188,7 +3513,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       for (i = 0; i < len; i++)
         {
 	  unsigned int si;
-	  
+
 	  si = table[i];
 	  buffer[3 * i] = (si >> 16);
 	  buffer[3 * i + 1] = (si >> 8) % 256;
@@ -3200,7 +3525,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       for (i = 0; i < len; i++)
         {
 	  unsigned int si;
-	  
+
 	  si = table[i];
 	  buffer[4 * i] = (si >> 24);
 	  buffer[4 * i + 1] = (si >> 16) % 256;
@@ -3224,9 +3549,9 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       meta[i] = 0;
     }
 
-  meta[6] = offset_size;  
+  meta[6] = offset_size;
   meta[7] = index_size;
-  
+
   len = [objectList count];
   meta[12] = (len >> 24);
   meta[13] = (len >> 16) % 256;
@@ -3236,8 +3561,8 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
   meta[29] = (table_start >> 16) % 256;
   meta[30] = (table_start >> 8) % 256;
   meta[31] = table_start % 256;
-  
-  [dest appendBytes: meta length: 32];      
+
+  [dest appendBytes: meta length: 32];
 }
 
 - (unsigned int) indexForObject: (id)object
@@ -3262,14 +3587,14 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       unsigned char oid;
 
       oid = index;
-      [dest appendBytes: &oid length: 1];      
+      [dest appendBytes: &oid length: 1];
     }
   else if (index_size == 2)
     {
       unsigned short oid;
 
       oid = NSSwapHostShortToBig(index);
-      [dest appendBytes: &oid length: 2];      
+      [dest appendBytes: &oid length: 2];
     }
   else if (index_size == 4)
     {
@@ -3296,7 +3621,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       code = 0x10;
       [dest appendBytes: &code length: 1];
       c = count;
-      [dest appendBytes: &c length: 1];      
+      [dest appendBytes: &c length: 1];
     }
   else if (count < 256 * 256)
     {
@@ -3320,7 +3645,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 - (void) storeData: (NSData*) data
 {
   unsigned int len;
-  unsigned char code; 
+  unsigned char code;
 
   len = [data length];
 
@@ -3343,12 +3668,12 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 {
   unsigned int len;
   BOOL ascii = YES;
-  unsigned char code; 
+  unsigned char code;
   unsigned int i;
   unichar uchar;
 
   len = [string length];
-  
+
   for (i = 0; i < len; i++)
     {
       uchar = [string characterAtIndex: i];
@@ -3394,7 +3719,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       else
         {
 	  unichar *buffer;
-	  
+
 	  code = 0x6F;
 	  [dest appendBytes: &code length: 1];
 	  buffer = objc_malloc(sizeof(unichar)*(len + 1));
@@ -3413,17 +3738,17 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 - (void) storeNumber: (NSNumber*) number
 {
   const char *type;
-  unsigned char code; 
+  unsigned char code;
 
   type = [number objCType];
 
   switch (*type)
     {
-      case 'c':	
-      case 'C':	
-      case 's':	
-      case 'S':	
-      case 'i':	
+      case 'c':
+      case 'C':
+      case 's':
+      case 'S':
+      case 'i':
       case 'I':
       case 'l':
       case 'L':
@@ -3452,7 +3777,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 	      code = 0x10;
 	      [dest appendBytes: &code length: 1];
 	      cval = (unsigned char) val;
-	      [dest appendBytes: &cval length: 1];	      
+	      [dest appendBytes: &cval length: 1];
 	    }
 	  else if (val < 256 * 256)
 	    {
@@ -3509,7 +3834,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 
 - (void) storeDate: (NSDate*) date
 {
-  unsigned char code; 
+  unsigned char code;
   double out;
 
   code = 0x33;
@@ -3520,7 +3845,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 
 - (void) storeArray: (NSArray*) array
 {
-  unsigned char code; 
+  unsigned char code;
   unsigned int len;
   unsigned int i;
 
@@ -3537,12 +3862,12 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       [dest appendBytes: &code length: 1];
       [self storeCount: len];
     }
-  
+
   for (i = 0; i < len; i++)
     {
       id obj;
       unsigned int oid;
-	  
+
       obj = [array objectAtIndex: i];
       oid = [self indexForObject: obj];
       [self storeIndex: oid];
@@ -3551,7 +3876,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 
 - (void) storeDictionary: (NSDictionary*) dict
 {
-  unsigned char code; 
+  unsigned char code;
   NSNumber *num;
   unsigned int i;
 
@@ -3574,7 +3899,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
       else
         {
 	  unsigned short si;
-      
+
 	  code = 0x81;
 	  [dest appendBytes: &code length: 1];
 	  si = NSSwapHostShortToBig((unsigned short)index);
@@ -3593,7 +3918,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 	  key = [keys objectAtIndex: i];
 	  [objects addObject: [dict objectForKey: key]];
 	}
-      
+
       if (len < 0x0F)
         {
 	  code = 0xD0 + len;
@@ -3605,22 +3930,22 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 	  [dest appendBytes: &code length: 1];
 	  [self storeCount: len];
 	}
-      
+
       for (i = 0; i < len; i++)
         {
 	  id obj;
 	  unsigned int oid;
-	  
+
 	  obj = [keys objectAtIndex: i];
 	  oid = [self indexForObject: obj];
 	  [self storeIndex: oid];
 	}
-      
+
       for (i = 0; i < len; i++)
         {
 	  id obj;
 	  unsigned int oid;
-	  
+
 	  obj = [objects objectAtIndex: i];
 	  oid = [self indexForObject: obj];
 	  [self storeIndex: oid];
@@ -3656,7 +3981,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
     {
       [self storeDictionary: object];
     }
-  else 
+  else
     {
       NSLog(@"Unknown object class %@", object);
     }
@@ -3677,7 +4002,7 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
 	  done = YES;
 	}
       NS_HANDLER
-	{ 
+	{
 	  [self cleanup];
 	  index_size += 1;
 	}
