@@ -9,22 +9,22 @@
    This file is part of the GNUstep Objective-C Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSScanner class reference</title>
-   $Date: 2006-08-25 05:47:20 -0600 (Fri, 25 Aug 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 /* We need to define _GNU_SOURCE on systems (SuSE) to get LONG_LONG_MAX.  */
@@ -140,8 +140,8 @@ typedef struct {
       GSPlaceholderStringClass = [GSPlaceholderString class];
       NSConstantStringClass = [NSString constantStringClass];
       _holder = (id)NSAllocateObject(GSPlaceholderStringClass, 0, 0);
-      externalEncoding = GetDefEncoding();
-      if (GSIsByteEncoding(externalEncoding) == YES)
+      externalEncoding = [NSString defaultCStringEncoding];
+      if (GSPrivateIsByteEncoding(externalEncoding) == YES)
 	{
 	  internalEncoding = externalEncoding;
 	}
@@ -169,7 +169,10 @@ typedef struct {
 
   if (scanner != nil)
     {
-      [scanner setLocale: GSUserDefaultsDictionaryRepresentation()];
+      NSDictionary	*loc;
+
+      loc = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
+      [scanner setLocale: loc];
     }
   return scanner;
 }
