@@ -7,22 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSSerializer class reference</title>
-   $Date: 2006-05-22 03:57:21 -0600 (Mon, 22 May 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -36,6 +36,7 @@
 #include "Foundation/NSLock.h"
 #include "Foundation/NSSet.h"
 #include "Foundation/NSThread.h"
+#include "Foundation/NSNotification.h"
 #include "Foundation/NSNotificationQueue.h"
 #include "Foundation/NSObjCRuntime.h"
 #include "Foundation/NSValue.h"
@@ -49,10 +50,6 @@
 @class	NSDataMalloc;
 @interface NSDataMalloc : NSObject	// Help the compiler
 @end
-@class	GSInlineArray;
-@class	GSMutableArray;
-@interface GSMutableArray : NSObject	// Help the compiler
-@end
 
 /*
  *	Setup for inline operation of string map tables.
