@@ -7,22 +7,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSTask class reference</title>
-   $Date: 2006-08-26 01:12:29 -0600 (Sat, 26 Aug 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -47,6 +47,7 @@
 #include "Foundation/NSTimer.h"
 #include "Foundation/NSLock.h"
 #include "Foundation/NSDebug.h"
+#include "GSPrivate.h"
 
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -921,7 +922,7 @@ pty_slave(const char* name)
 @implementation NSConcreteWindowsTask
 
 BOOL
-GSCheckTasks()
+GSPrivateCheckTasks()
 {
   BOOL	found = NO;
 
@@ -1197,7 +1198,8 @@ quotedFromString(NSString *aString)
     NULL,      			/* proc attrs */
     NULL,      			/* thread attrs */
     1,         			/* inherit handles */
-    CREATE_NO_WINDOW|DETACHED_PROCESS|CREATE_UNICODE_ENVIRONMENT,
+//    CREATE_NO_WINDOW|DETACHED_PROCESS|CREATE_UNICODE_ENVIRONMENT,
+    CREATE_NO_WINDOW|CREATE_UNICODE_ENVIRONMENT,
     envp,			/* env block */
     (const unichar*)[[self currentDirectoryPath] fileSystemRepresentation],
     &start_info,
@@ -1256,7 +1258,7 @@ quotedFromString(NSString *aString)
 @implementation NSConcreteUnixTask
 
 BOOL
-GSCheckTasks()
+GSPrivateCheckTasks()
 {
   BOOL	found = NO;
 
@@ -1276,6 +1278,7 @@ GSCheckTasks()
 
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
+	      AUTORELEASE(RETAIN(t));
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
@@ -1561,8 +1564,8 @@ GSCheckTasks()
       result = waitpid(_taskId, &_terminationStatus, WNOHANG);
       if (result < 0)
         {
-          NSLog(@"waitpid %d, result %d, error %s",
-	    _taskId, result, GSLastErrorStr(errno));
+          NSLog(@"waitpid %d, result %d, error %@",
+	    _taskId, result, [NSError _last]);
           [self _terminatedChild: -1];
         }
       else if (result == _taskId || (result > 0 && errno == 0))
@@ -1594,8 +1597,8 @@ GSCheckTasks()
 #ifdef  WAITDEBUG
       else
 	{
-	  NSLog(@"waitpid %d, result %d, error %s",
-	    _taskId, result, GSLastErrorStr(errno));
+	  NSLog(@"waitpid %d, result %d, error %@",
+	    _taskId, result, [NSError _last]);
 	}
 #endif
     }
