@@ -13,22 +13,22 @@
    This file is part of the GNUstep Objective-C Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSThread class reference</title>
-   $Date: 2006-06-04 00:42:10 -0600 (Sun, 04 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 #include "config.h"
@@ -49,11 +49,13 @@
 #include "Foundation/NSThread.h"
 #include "Foundation/NSLock.h"
 #include "Foundation/NSString.h"
+#include "Foundation/NSNotification.h"
 #include "Foundation/NSNotificationQueue.h"
 #include "Foundation/NSRunLoop.h"
 #include "Foundation/NSConnection.h"
 #include "Foundation/NSInvocation.h"
 
+#include "GSPrivate.h"
 #include "GSRunLoopCtxt.h"
 
 @interface NSAutoreleasePool (NSThread)
@@ -889,12 +891,12 @@ static NSDate *theFuture;
 #if defined(__MINGW32__)
   if (SetEvent(event) == 0)
     {
-      NSLog(@"Set event failed - %@", GSLastErrorStr(errno));
+      NSLog(@"Set event failed - %@", [NSError _last]);
     }
 #else
   if (write(outputFd, "0", 1) != 1)
     {
-      NSLog(@"Write to pipe failed - %@", GSLastErrorStr(errno));
+      NSLog(@"Write to pipe failed - %@", [NSError _last]);
     }
 #endif
 
@@ -918,12 +920,12 @@ static NSDate *theFuture;
 #if defined(__MINGW32__)
   if (ResetEvent(event) == 0)
     {
-      NSLog(@"Reset event failed - %@", GSLastErrorStr(errno));
+      NSLog(@"Reset event failed - %@", [NSError _last]);
     }
 #else
   if (read(inputFd, &c, 1) != 1)
     {
-      NSLog(@"Read pipe failed - %@", GSLastErrorStr(errno));
+      NSLog(@"Read pipe failed - %@", [NSError _last]);
     }
 #endif
 
