@@ -10,8 +10,8 @@
      This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
-   as published by the Free Software Foundation; either version 2 of
+   modify it under the terms of the GNU Lesser General Public License
+   as published by the Free Software Foundation; either version 3 of
    the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful, but
@@ -19,12 +19,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSTimeZone class reference</title>
-   $Date: 2006-06-04 00:42:10 -0600 (Sun, 04 Jun 2006) $ $Revision$
+   $Date: 2007-11-06 03:27:56 -0700 (Tue, 06 Nov 2007) $ $Revision: 25562 $
  */
 
 /* Use the system time zones if available. In other cases, use an
@@ -58,13 +59,13 @@
    ===================================
 
    Default place for the NSTimeZone directory is _time_zone_path():
-     {$(GNUSTEP_SYSTEM_ROOT)Libary/Libraries/Resources/TIME_ZONE_DIR}
+     {$(GNUSTEP_SYSTEM_LIBRARY)/Libraries/gnustep-base/Versions/1.14/Resources/TIME_ZONE_DIR)
 
    LOCAL_TIME_FILE is a text file with the name of the time zone file.
 
    ZONES_DIR is a sub-directory under TIME_ZONE_DIR
 
-   (dir) ../System/Library/Libraries/Resources/..
+   (dir) ../System/Library/Libraries/gnustep-base/Versions/1.14/Resources/..
    (dir)     NSTimeZone
    (file)      localtime {text; time zone eg Australia/Perth}
    (dir)       zones
@@ -106,7 +107,7 @@
 #include "Foundation/NSByteOrder.h"
 #include "Foundation/NSDebug.h"
 #include "GNUstepBase/GSCategories.h"
-#include "GSConfig.h"
+#include "GNUstepBase/GSConfig.h"
 #include "GSPrivate.h"
 
 #ifdef HAVE_TZHEAD
@@ -272,7 +273,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
   NSBundle *gbundle;
   if (type == nil)
     type = @"";
-  gbundle = [NSBundle bundleForLibrary: @"gnustep-base"];
+  gbundle = [NSBundle bundleForClass: [NSObject class]];
   return [gbundle pathForResource: subpath
 		           ofType: type
 		      inDirectory: TIME_ZONE_DIR];
@@ -464,15 +465,6 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 	  if (data == nil)
 	    {
 	      NSString		*fileName;
-	      const char	*str = [name UTF8String];
-
-	      /* Make sure that only time zone files are accessed.
-		 FIXME: Make this more robust. */
-	      if ((str)[0] == '/' || strchr(str, '.') != NULL)
-		{
-		  NSLog(@"Disallowed time zone name `%@'.", name);
-		  return nil;
-		}
 
 	      fileName = [NSTimeZoneClass getTimeZoneFile: name];
 	      if (fileName == nil
@@ -1468,38 +1460,22 @@ static NSMapTable	*absolutes = 0;
 
 #if	defined(__MINGW32__)
       /*
-       * Try to get timezone from windows registry.
+       * Try to get timezone from windows system call.
        */
       {
-        HKEY regkey;
+      	TIME_ZONE_INFORMATION tz;
+      	DWORD DST = GetTimeZoneInformation(&tz);
 
-        if (ERROR_SUCCESS == RegOpenKeyExA(HKEY_LOCAL_MACHINE,
-	  "SYSTEM\\CurrentControlSet\\Control\\TimeZoneInformation",
-	  0,
-	  KEY_READ,
-	  &regkey))
-          {
-            char buf[255];
-            DWORD bufsize=255;
-            DWORD type;
-
-            if (ERROR_SUCCESS == RegQueryValueExA(regkey,
-	      "StandardName",
-	      0,
-	      &type,
-	      buf,
-	      &bufsize))
-              {
-                bufsize = strlen(buf);
-                while (bufsize && isspace(buf[bufsize-1]))
-                  {
-                    bufsize--;
-                  }
-		localZoneString
-		  = [NSString stringWithCString: buf length: bufsize];
-              }
-            RegCloseKey(regkey);
-          }
+      	if (DST == TIME_ZONE_ID_DAYLIGHT)
+	  {
+	    localZoneString = [NSString stringWithCharacters: tz.DaylightName
+	      length: wcslen(tz.DaylightName)];
+	  }
+      	else
+	  {
+	    localZoneString = [NSString stringWithCharacters: tz.StandardName
+	      length: wcslen(tz.StandardName)];
+	  }
       }
 #endif
 
@@ -2111,12 +2087,6 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
   int *hour, int *minute, int *second, int *mil);
 
 
-
-/* FIXME
- * It's not unicode ... which is OK as the timezone registry
- * names are ascii ... but we ought to be consistent.
- */
-
 @implementation GSWindowsTimeZone
 
 - (NSString*) abbreviationForDate: (NSDate*)aDate
@@ -2142,209 +2112,175 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
 
 - (id) initWithName: (NSString*)name data: (NSData*)data
 {
-  HKEY  regDirKey;
-  BOOL  isNT = NO;
-  BOOL	regFound = NO;
+  HKEY     regDirKey;
+  BOOL     isNT = NO,regFound=NO;
 
-  /* Open the key in the local machine hive where
-   * the time zone data is stored. */
-  if (ERROR_SUCCESS == RegOpenKeyExA(HKEY_LOCAL_MACHINE,
-    "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
-    0,
-    KEY_READ,
-    &regDirKey))
-    {
-      isNT = YES;
-      regFound = YES;
-    }
+  /* Open the key in the local machine hive where the time zone data is stored. */
+  if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
+  	L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
+  	0,
+  	KEY_READ,
+  	&regDirKey))
+  {
+    isNT=YES;
+    regFound=YES;
+  }
   else
+  {
+    if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE,
+    	L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
+    	0,
+    	KEY_READ,
+    	&regDirKey))
     {
-      if (ERROR_SUCCESS == RegOpenKeyExA(HKEY_LOCAL_MACHINE,
-	"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones",
-	0,
-	KEY_READ,
-	&regDirKey))
-	{
-	  regFound = YES;
-	}
+    	regFound=YES;
     }
+  }
 
   if (regFound)
-    {
-      /* Iterate over all subKeys in the registry to find the right one.
-	 Unfortunately name is a localized value. The keys in the registry are
-	 unlocalized names. */
-      CHAR     achKey[255];              // buffer for subkey name
-      DWORD    cbName;                   // size of name string
-      CHAR     achClass[MAX_PATH] = "";  // buffer for class name
-      DWORD    cchClassName = MAX_PATH;  // size of class string
-      DWORD    cSubKeys=0;               // number of subkeys
-      DWORD    cbMaxSubKey;              // longest subkey size
-      DWORD    cchMaxClass;              // longest class string
-      DWORD    cValues;                  // number of values for key
-      DWORD    cchMaxValue;              // longest value name
-      DWORD    cbMaxValueData;           // longest value data
-      DWORD    cbSecurityDescriptor;     // size of security descriptor
-      FILETIME ftLastWriteTime;          // last write time
-      DWORD i;
-      DWORD retCode;
-      BOOL	tzFound = NO;
-	      
-	      /* Get the class name and the value count. */
-      retCode = RegQueryInfoKeyA(
-	regDirKey,               // key handle
-	achClass,                // buffer for class name
-	&cchClassName,           // size of class string
-	NULL,                    // reserved
-	&cSubKeys,               // number of subkeys
-	&cbMaxSubKey,            // longest subkey size
-	&cchMaxClass,            // longest class string
-	&cValues,                // number of values for this key
-	&cchMaxValue,            // longest value name
-	&cbMaxValueData,         // longest value data
-	&cbSecurityDescriptor,   // security descriptor
-	&ftLastWriteTime);       // last write time
+  {
+    	/* Iterate over all subKeys in the registry to find the right one.
+    	   Unfortunately name is a localized value. The keys in the registry are
+    	   unlocalized names. */
+	   wchar_t  achKey[255];              // buffer for subkey name
+    	DWORD    cbName;                   // size of name string 
+    	wchar_t  achClass[MAX_PATH] = L""; // buffer for class name 
+    	DWORD    cchClassName = MAX_PATH;  // size of class string 
+    	DWORD    cSubKeys=0;               // number of subkeys 
+    	DWORD    cbMaxSubKey;              // longest subkey size 
+    	DWORD    cchMaxClass;              // longest class string 
+    	DWORD    cValues;                  // number of values for key 
+    	DWORD    cchMaxValue;              // longest value name 
+    	DWORD    cbMaxValueData;           // longest value data 
+    	DWORD    cbSecurityDescriptor;     // size of security descriptor 
+    	FILETIME ftLastWriteTime;          // last write time 
+		
+		DWORD i, retCode;
+		BOOL	tzFound = NO;
+		
+		/* Get the class name and the value count. */
+	    retCode = RegQueryInfoKeyW(
+	        regDirKey,               // key handle 
+	        achClass,                // buffer for class name 
+	        &cchClassName,           // size of class string 
+	        NULL,                    // reserved 
+	        &cSubKeys,               // number of subkeys 
+	        &cbMaxSubKey,            // longest subkey size 
+	        &cchMaxClass,            // longest class string 
+	        &cValues,                // number of values for this key 
+	        &cchMaxValue,            // longest value name 
+	        &cbMaxValueData,         // longest value data 
+	        &cbSecurityDescriptor,   // security descriptor 
+	        &ftLastWriteTime);       // last write time 
 
-      if (cSubKeys && (retCode == ERROR_SUCCESS))
-	{
-	  const char *cName = [name cString];
+    	if (cSubKeys && (retCode == ERROR_SUCCESS))
+    	{
+    		wchar_t *wName = malloc(([name length]+1) * sizeof(wchar_t));
+    		if (wName)
+    		{
+    			[name getCharacters:wName];
 
-	  for (i = 0; i < cSubKeys && !tzFound; i++)
-	    {
-	      cbName = 255;
-	    
-	      retCode = RegEnumKeyExA(regDirKey,
-		i,
-		achKey,
-		&cbName,
-		NULL,
-		NULL,
-		NULL,
-		&ftLastWriteTime);
-	      if (retCode == ERROR_SUCCESS)
-		{
-		  char keyBuffer[16384];
-		  HKEY regKey;
-	    
-		  if (isNT)
-		    {
-		      sprintf(keyBuffer, "SOFTWARE\\Microsoft\\Windows NT"
-			"\\CurrentVersion\\Time Zones\\%s", achKey);
-		    }
-		  else
-		    {
-		      sprintf(keyBuffer, "SOFTWARE\\Microsoft\\Windows"
-			"\\CurrentVersion\\Time Zones\\%s", achKey);
-		    }
-	    
-		  if (ERROR_SUCCESS == RegOpenKeyExA(HKEY_LOCAL_MACHINE,
-		    keyBuffer,
-		    0,
-		    KEY_READ,
-		    &regKey))
-		    {
-		      char buf[256];
-		      char standardName[256];
-		      char daylightName[256];
-		      DWORD bufsize;
-		      DWORD type;
-
-		      /* check standardname */
-		      standardName[0]='\0';
-		      bufsize=sizeof(buf);
-		      if (ERROR_SUCCESS == RegQueryValueExA(regKey,
-			"Std",
-			0,
-			&type,
-			buf,
-			&bufsize))
-			{
-			  strcpy(standardName, buf);
-			  if (strcmp(standardName, cName) == 0)
-			    tzFound = YES;
-			}
-
-		      /* check daylightname */
-		      daylightName[0]='\0';
-		      bufsize = sizeof(buf);
-		      if (ERROR_SUCCESS == RegQueryValueExA(regKey,
-			"Dlt",
-			0,
-			&type,
-			buf,
-			&bufsize))
-			{
-			  strcpy(daylightName, buf);
-			  if (strcmp(daylightName, cName) == 0)
-			    tzFound = YES;
-			}
-
-		      if (tzFound)
-			{
-			  /* Read in the time zone data */
-			  bufsize = sizeof(buf);
-			  if (ERROR_SUCCESS == RegQueryValueExA(regKey,
-			    "TZI",
-			    0,
-			    &type,
-			    buf,
-			    &bufsize))
-			    {
-			      TZI *tzi = (void*)buf;
-			      Bias = tzi->Bias;
-			      StandardBias = tzi->StandardBias;
-			      DaylightBias = tzi->DaylightBias;
-			      StandardDate = tzi->StandardDate;
-			      DaylightDate = tzi->DaylightDate;
-			    }
-		    
-			  /* Set the standard name for the time zone. */
-			  if (strlen(standardName))
-			    {
-			      int a, b;
-
-			      ASSIGN(timeZoneName,
-				[NSString stringWithUTF8String: standardName]);
-
-			      /* Abbr generated here is IMHO a
-			       * bit suspicous but I kept it */
-			      for (a = 0, b = 0; standardName[a]; a++)
-				{
-				  if (isupper(standardName[a]))
-				    standardName[b++] = standardName[a];
+				for (i=0; i<cSubKeys && !tzFound; i++) 
+				{ 
+				   cbName = 255;
+				   
+				   retCode = RegEnumKeyExW(regDirKey, i, achKey, &cbName, NULL, NULL, NULL, &ftLastWriteTime);         
+				   if (retCode == ERROR_SUCCESS) 
+				   {
+				   	 wchar_t keyBuffer[16384];
+				   	 HKEY regKey;
+				       
+				       if (isNT)
+				       	wcscpy(keyBuffer,L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones\\");
+						 else
+						 	wcscpy(keyBuffer,L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Time Zones\\");
+				       
+				       wcscat(keyBuffer,achKey);
+	 		          if (ERROR_SUCCESS == RegOpenKeyExW(HKEY_LOCAL_MACHINE, keyBuffer, 0, KEY_READ, &regKey))
+				       {
+					      wchar_t buf[256];
+					      wchar_t standardName[256];
+					      wchar_t daylightName[256];
+					      DWORD bufsize;
+					      DWORD type;
+	
+							/* check standardname */
+							standardName[0]=L'\0';
+					      bufsize=sizeof(buf);
+					      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"Std", 0, &type, (BYTE *)buf, &bufsize))
+					      {
+					      	wcscpy(standardName,buf);
+					      	if (wcscmp(standardName,wName) == 0)
+					      		tzFound = YES;
+					   	}
+			
+							/* check daylightname */
+							daylightName[0]=L'\0';
+					      bufsize=sizeof(buf);
+					      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"Dlt", 0, &type, (BYTE *)buf, &bufsize))
+					      {
+					      	wcscpy(daylightName,buf);
+					      	if (wcscmp(daylightName,wName) == 0)
+					      		tzFound = YES;
+					   	}
+	
+							if (tzFound)
+							{
+						      /* Read in the time zone data */
+						      bufsize=sizeof(buf);
+						      if (ERROR_SUCCESS==RegQueryValueExW(regKey, L"TZI", 0, &type, (BYTE *)buf, &bufsize))
+						      {
+						          TZI *tzi = (void*)buf;
+						          Bias = tzi->Bias;
+						          StandardBias = tzi->StandardBias;
+						          DaylightBias = tzi->DaylightBias;
+						          StandardDate = tzi->StandardDate;
+						          DaylightDate = tzi->DaylightDate;
+						      }
+						      
+						      /* Set the standard name for the time zone. */
+						      if (wcslen(standardName))
+						      {
+						          int a, b;
+						          ASSIGN(timeZoneName,[NSString stringWithCharacters:standardName length:wcslen(standardName)]);
+	
+						          /* Abbr generated here is IMHO a bit suspicous but I kept it */
+						          for(a=0,b=0;standardName[a];a++)
+						          {
+						            if (iswupper(standardName[a]))
+						              standardName[b++]=standardName[a];
+						          }
+						          standardName[b]=L'\0';
+						          ASSIGN(timeZoneNameAbbr,[NSString stringWithCharacters:standardName length:wcslen(standardName)]);
+						      }
+	
+						      /* Set the daylight savings name for the time zone. */
+						      if (wcslen(daylightName))
+						      {
+						          int a,b;
+						          ASSIGN(daylightZoneName,[NSString stringWithCharacters:daylightName length:wcslen(daylightName)]);
+	
+						          /* Abbr generated here is IMHO a bit suspicous but I kept it */
+						          for(a=0,b=0;daylightName[a];a++)
+						          {
+						              if (iswupper(daylightName[a]))
+						                daylightName[b++]=daylightName[a];
+						          }
+						          daylightName[b]=L'\0';
+						          ASSIGN(daylightZoneNameAbbr,[NSString stringWithCharacters:daylightName length:wcslen(daylightName)]);
+					      	}
+					   	}
+					      RegCloseKey(regKey);
+				   	}			       
+				   }
 				}
-			      standardName[b] = 0;
-			      ASSIGN(timeZoneNameAbbr,
-				[NSString stringWithUTF8String: standardName]);
-			    }
-
-			  /* Set the daylight savings name for the time zone. */
-			  if (strlen(daylightName))
-			    {
-			      int a, b;
-
-			      ASSIGN(daylightZoneName,
-				[NSString stringWithUTF8String: daylightName]);
-
-			      /* Abbr generated here is IMHO
-			       * a bit suspicous but I kept it */
-			      for (a = 0, b = 0; daylightName[a]; a++)
-				{
-				  if (isupper(daylightName[a]))
-				    daylightName[b++] = daylightName[a];
-				}
-			      daylightName[b] = 0;
-			      ASSIGN(daylightZoneNameAbbr,
-				[NSString stringWithUTF8String: daylightName]);
-			    }
+				
+				free(wName);
 			}
-		      RegCloseKey(regKey);
-		    }			
-		}
-	    }
-	}
+    	}
       RegCloseKey(regDirKey);
-    }
+  }
+    
   return self;
 }
 
@@ -2362,12 +2298,36 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
 
   GSBreakTime(when, &year, &month, &day, &hour, &minute, &second, &mil);
 
-  // Before April or after October is Std
-  if (month < DaylightDate.wMonth || month > StandardDate.wMonth)
-    return NO;
-  // After April and before October is DST
-  if (month > DaylightDate.wMonth && month < StandardDate.wMonth)
-    return YES;
+  // Check north globe
+  if (StandardDate.wMonth >= DaylightDate.wMonth)
+    {
+      // Before April or after October is Std
+      if (month < DaylightDate.wMonth || month > StandardDate.wMonth)
+        {
+	  return NO;
+	}
+      // After April and before October is DST
+      if (month > DaylightDate.wMonth && month < StandardDate.wMonth)
+        {
+	  return YES;
+	}
+    }
+  else
+    {
+      /* check south globe 
+       * Before April or after October is DST
+       */
+      if (month < StandardDate.wMonth || month > DaylightDate.wMonth)
+        {
+	  return YES;
+	}
+      // After April and before October is Std
+      if (month > StandardDate.wMonth && month < DaylightDate.wMonth)
+        {
+	  return NO;
+	}
+  }
+
   dow = ((int)((when / 86400.0) + GREGORIAN_REFERENCE)) % 7;
   if (dow < 0)
     dow += 7;
@@ -2405,7 +2365,7 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
       if (mil >= DaylightDate.wMilliseconds)
         return YES;
       return NO;
-  }
+    }
   if (month == StandardDate.wMonth /* October */)
     {
       daylightdate = day - dow + StandardDate.wDayOfWeek;
@@ -2445,7 +2405,17 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
 
 - (NSString*) name
 {
-  return timeZoneName;
+  TIME_ZONE_INFORMATION tz;
+  DWORD DST = GetTimeZoneInformation(&tz);
+
+  if (DST == TIME_ZONE_ID_DAYLIGHT)
+    {
+      return daylightZoneName;
+    }
+  else
+    {
+      return timeZoneName;
+    }
 }
 
 - (int) secondsFromGMTForDate: (NSDate*)aDate
@@ -2495,7 +2465,7 @@ GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
 
 - (NSString*) timeZoneName
 {
-  return timeZoneName;
+  return [self name];
 }
 @end
 #endif // __MINGW32__
