@@ -9,21 +9,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSTimer class reference</title>
-   $Date: 2006-03-13 08:15:44 -0700 (Mon, 13 Mar 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
    */
 
 #include "config.h"
@@ -121,7 +122,7 @@ static Class	NSDate_class;
 /**
  * Create a timer which will fire after ti seconds and, if f is YES,
  * every ti seconds thereafter. On firing, the target object will be
- * sent a message specified by selector and with the value info as an
+ * sent a message specified by selector and with the timer as its
  * argument.<br />
  * NB. To make the timer operate, you must add it to a run loop.
  */
@@ -149,17 +150,21 @@ static Class	NSDate_class;
 				 invocation: (NSInvocation*)invocation
 				    repeats: (BOOL)f
 {
-  id t = [self timerWithTimeInterval: ti
-			  invocation: invocation
-			     repeats: f];
+  id t = [[self alloc] initWithFireDate: nil
+			       interval: ti
+				 target: invocation
+			       selector: NULL
+			       userInfo: nil
+				repeats: f];
   [[NSRunLoop currentRunLoop] addTimer: t forMode: NSDefaultRunLoopMode];
+  RELEASE(t);
   return t;
 }
 
 /**
  * Create a timer which will fire after ti seconds and, if f is YES,
  * every ti seconds thereafter. On firing, the target object will be
- * sent a message specified by selector and with the object info as an
+ * sent a message specified by selector and with the timer as its
  * argument.<br />
  * This timer will automatically be added to the current run loop and
  * will fire in the default run loop mode.
@@ -170,12 +175,14 @@ static Class	NSDate_class;
 				   userInfo: (id)info
 				    repeats: (BOOL)f
 {
-  id t = [self timerWithTimeInterval: ti
-			      target: object
-			    selector: selector
-			    userInfo: info
-			     repeats: f];
+  id t = [[self alloc] initWithFireDate: nil
+			       interval: ti
+				 target: object
+			       selector: selector
+			       userInfo: info
+				repeats: f];
   [[NSRunLoop currentRunLoop] addTimer: t forMode: NSDefaultRunLoopMode];
+  RELEASE(t);
   return t;
 }
 
