@@ -10,22 +10,22 @@
    This file is part of the GNUstep Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2006-06-16 09:21:39 -0600 (Fri, 16 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 /*
@@ -627,6 +627,10 @@ static unsigned	urlAlign;
 - (id) initWithString: (NSString*)aUrlString
 	relativeToURL: (NSURL*)aBaseUrl
 {
+  /* RFC 2396 'reserved' characters ...
+   */
+  static const char *reserved = ";/?:@&=+$,";
+
   if (aUrlString == nil)
     {
       [NSException raise: NSInvalidArgumentException
@@ -700,21 +704,24 @@ static unsigned	urlAlign;
       /*
        * Set up scheme specific parsing options.
        */
-      if (buf->scheme != 0)
+      if (buf->scheme == 0)
+        {
+	  DESTROY(self);	// Not a valid URL
+	  NS_VALRETURN(nil);
+	}
+
+      if (strcmp(buf->scheme, "file") == 0)
 	{
-	  if (strcmp(buf->scheme, "file") == 0)
-	    {
-	      usesFragments = NO;
-	      usesParameters = NO;
-	      usesQueries = NO;
-	      buf->isFile = YES;
-	    }
-	  else if (strcmp(buf->scheme, "mailto") == 0)
-	    {
-	      usesFragments = NO;
-	      usesParameters = NO;
-	      usesQueries = NO;
-	    }
+	  usesFragments = NO;
+	  usesParameters = NO;
+	  usesQueries = NO;
+	  buf->isFile = YES;
+	}
+      else if (strcmp(buf->scheme, "mailto") == 0)
+	{
+	  usesFragments = NO;
+	  usesParameters = NO;
+	  usesQueries = NO;
 	}
 
       if (canBeGeneric == YES)
@@ -876,6 +883,11 @@ static unsigned	urlAlign;
 	      if (buf->fragment == 0 && base != 0)
 		{
 		  buf->fragment = base->fragment;
+		  if (legal(buf->fragment, reserved) == NO)
+		    {
+		      [NSException raise: NSGenericException format:
+		        @"illegal character in fragment part"];
+		    }
 		}
 	    }
 
@@ -896,6 +908,11 @@ static unsigned	urlAlign;
 	      if (buf->query == 0 && base != 0)
 		{
 		  buf->query = base->query;
+		  if (legal(buf->query, reserved) == NO)
+		    {
+		      [NSException raise: NSGenericException format:
+		        @"illegal character in query part"];
+		    }
 		}
 	    }
 
@@ -916,6 +933,11 @@ static unsigned	urlAlign;
 	      if (buf->parameters == 0 && base != 0)
 		{
 		  buf->parameters = base->parameters;
+		  if (legal(buf->parameters, reserved) == NO)
+		    {
+		      [NSException raise: NSGenericException format:
+		        @"illegal character in parameters part"];
+		    }
 		}
 	    }
 
@@ -941,6 +963,11 @@ static unsigned	urlAlign;
        * Store the path.
        */
       buf->path = start;
+      if (legal(buf->path, reserved) == NO)
+	{
+	  [NSException raise: NSGenericException format:
+	    @"illegal character in path part"];
+	}
     }
   NS_HANDLER
     {
@@ -1363,33 +1390,20 @@ static unsigned	urlAlign;
 
 /**
  * Loads the resource data for the represented URL and returns the result.
- * The shouldUseCache flag determines whether an existing cached NSURLHandle
- * can be used to provide the data.
+ * The shouldUseCache flag determines whether data previously retrieved by
+ * an existing NSURLHandle can be used to provide the data, or if it should
+ * be refetched.
  */
 - (NSData*) resourceDataUsingCache: (BOOL)shouldUseCache
 {
-  NSURLHandle	*handle = [self URLHandleUsingCache: shouldUseCache];
+  NSURLHandle	*handle = [self URLHandleUsingCache: YES];
   NSData	*data;
 
   if (shouldUseCache == NO || [handle status] != NSURLHandleLoadSucceeded)
     {
-      NSRunLoop	*loop;
-      NSDate	*future;
-
-      [self loadResourceDataNotifyingClient: self
-				 usingCache: shouldUseCache];
-
-      /*
-       * Keep the runloop going until the load has completed (or failed).
-       */
-      loop = [NSRunLoop currentRunLoop];
-      future = [NSDate distantFuture];
-      while ([handle status] == NSURLHandleLoadInProgress)
-	{
-	  [loop runMode: NSDefaultRunLoopMode beforeDate: future];
-	}
+      [handle loadInForeground];
     }
-  data = [handle resourceData];
+  data = [handle availableResourceData];
   return data;
 }
 
@@ -1464,9 +1478,7 @@ static unsigned	urlAlign;
     {
       return NO;
     }
-  [self loadResourceDataNotifyingClient: self
-			     usingCache: YES];
-  if ([handle resourceData] == nil)
+  if ([handle loadInForeground] == nil)
     {
       return NO;
     }
