@@ -9,22 +9,22 @@
    This file is part of the GNUstep Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
    <title>NSURLHandle class reference</title>
-   $Date: 2006-06-16 09:21:39 -0600 (Fri, 16 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 #include "GSURLPrivate.h"
@@ -481,7 +481,7 @@ static Class		NSURLHandleClass = 0;
       NSDate	*limit;
 
       limit = [[NSDate alloc] initWithTimeIntervalSinceNow: 1.0];
-      [loop runUntilDate: limit];
+      [loop runMode: NSDefaultRunLoopMode beforeDate: limit];
       RELEASE(limit);
     }
   return _data;
@@ -526,24 +526,23 @@ static Class		NSURLHandleClass = 0;
  */
 - (NSData*) resourceData
 {
-  if (_status == NSURLHandleLoadSucceeded)
+  if (_status != NSURLHandleLoadSucceeded)
     {
-      return [self availableResourceData];
-    }
-  else if (_status == NSURLHandleLoadInProgress)
-    {
-      return nil;
-    }
-  else
-    {
-      NSData	*d = [self loadInForeground];
-
-      if (d != nil)
+      if (_status == NSURLHandleLoadInProgress)
 	{
-	  ASSIGNCOPY(_data, d);
+	  return nil;
+	}
+      else
+	{
+	  NSData	*d = [self loadInForeground];
+
+	  if (d != nil)
+	    {
+	      ASSIGNCOPY(_data, d);
+	    }
 	}
-      return _data;
     }
+  return [self availableResourceData];
 }
 
 /**
