@@ -6,21 +6,22 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSUndoManager class reference</title>
-   $Date: 2006-02-27 12:37:16 -0700 (Mon, 27 Feb 2006) $ $Revision$
+   $Date: 2007-10-11 02:00:25 -0600 (Thu, 11 Oct 2007) $ $Revision: 25524 $
 */
 
 #include "config.h"
@@ -161,13 +162,9 @@
 @implementation NSUndoManager (Private)
 - (void) _loop: (id)arg
 {
-  if (_groupsByEvent)
+  if (_groupsByEvent && _group != nil)
     {
-      if (_group != nil)
-	{
-	  [self endUndoGrouping];
-	}
-      [self beginUndoGrouping];
+      [self endUndoGrouping];
     }
   _runLoopGroupingPending = NO;
 }
@@ -217,9 +214,10 @@
     {
       RELEASE(parent);
 
-      [[NSNotificationCenter defaultCenter]
-	  postNotificationName: NSUndoManagerDidOpenUndoGroupNotification
-			object: self];
+      if (_isUndoing == NO && _isRedoing == NO)
+        [[NSNotificationCenter defaultCenter]
+	    postNotificationName: NSUndoManagerDidOpenUndoGroupNotification
+			  object: self];
     }
 }
 
@@ -333,9 +331,10 @@
   p = RETAIN([g parent]);
   _group = p;
   [g orphan];
-  [[NSNotificationCenter defaultCenter]
-      postNotificationName: NSUndoManagerWillCloseUndoGroupNotification
-		    object: self];
+  if (_isUndoing == NO && _isRedoing == NO)
+    [[NSNotificationCenter defaultCenter]
+        postNotificationName: NSUndoManagerWillCloseUndoGroupNotification
+		      object: self];
   if (p == nil)
     {
       if (_isUndoing)
@@ -881,15 +880,18 @@
   if (_modes != newModes)
     {
       ASSIGN(_modes, newModes);
-      [[NSRunLoop currentRunLoop] cancelPerformSelector: @selector(_loop:)
-						 target: self
-					       argument: nil];
-      [[NSRunLoop currentRunLoop] performSelector: @selector(_loop:)
-	target: self
-	argument: nil
-	order: NSUndoCloseGroupingRunLoopOrdering
-	modes: _modes];
-      _runLoopGroupingPending = YES;
+      if (_runLoopGroupingPending)
+      {
+	NSRunLoop *runLoop = [NSRunLoop currentRunLoop];
+        [runLoop cancelPerformSelector: @selector(_loop:)
+				target: self
+			      argument: nil];
+        [runLoop performSelector: @selector(_loop:)
+			  target: self
+			argument: nil
+			   order: NSUndoCloseGroupingRunLoopOrdering
+			   modes: _modes];
+      }
     }
 }
 
