@@ -8,8 +8,8 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
-   as published by the Free Software Foundation; either version 2 of
+   modify it under the terms of the GNU Lesser General Public License
+   as published by the Free Software Foundation; either version 3 of
    the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful, but
@@ -17,12 +17,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 
    <title>NSZone class reference</title>
-   $Date: 2006-06-03 13:34:12 -0600 (Sat, 03 Jun 2006) $ $Revision$
+   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
 */
 
 /*  Design goals:
@@ -1702,12 +1703,12 @@ NSCreateZone (size_t start, size_t gran, BOOL canFree)
       newZone = (NSZone*)zone;
     }
 
-    [gnustep_global_lock lock];
-    newZone->next = zone_list;
-    zone_list = newZone;
-    [gnustep_global_lock unlock];
+  [gnustep_global_lock lock];
+  newZone->next = zone_list;
+  zone_list = newZone;
+  [gnustep_global_lock unlock];
 
-    return newZone;
+  return newZone;
 }
 
 /**
