@@ -7,24 +7,24 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #include "config.h"
 #include "Foundation/NSString.h"
 
-
 #include "Foundation/NSArray.h"
 #include "Foundation/NSException.h"
 
@@ -72,11 +72,6 @@ NSString *NSThreadWillExitNotification = @"NSThreadWillExitNotification";
 /*
  * Port Notifications
  */
-NSString *PortBecameInvalidNotification = @"PortBecameInvalidNotification";
-
-NSString *InPortClientBecameInvalidNotification = @"InPortClientBecameInvalidNotification";
-
-NSString *InPortAcceptedClientNotification = @"InPortAcceptedClientNotification";
 
 NSString *NSPortDidBecomeInvalidNotification = @"NSPortDidBecomeInvalidNotification";
 
@@ -121,11 +116,6 @@ NSString *NSShowNonLocalizedStrings = @"NSShowNonLocalizedStrings";
 NSString *NSLoadedClasses = @"NSLoadedClasses";
 
 
-/* Stream */
-NSString *StreamException = @"StreamException";
-
-
-
 /* Standard domains */
 NSString *NSArgumentDomain = @"NSArgumentDomain";
 
@@ -240,7 +230,7 @@ NSString *NSClassDescriptionNeededForClassNotification = @"NSClassDescriptionNee
  */
 
 void
-GSBuildStrings()
+GSPrivateBuildStrings()
 {
   static Class	NSStringClass = 0;
 
@@ -258,8 +248,6 @@ GSBuildStrings()
       [NSStringClass performSelector: @selector(initialize)];
 
       GS_REPLACE_CONSTANT_STRING(GSNetworkNotificationCenterType);
-      GS_REPLACE_CONSTANT_STRING(InPortAcceptedClientNotification);
-      GS_REPLACE_CONSTANT_STRING(InPortClientBecameInvalidNotification);
       GS_REPLACE_CONSTANT_STRING(NSAMPMDesignation);
       GS_REPLACE_CONSTANT_STRING(NSArgumentDomain);
       GS_REPLACE_CONSTANT_STRING(NSBundleDidLoadNotification);
@@ -322,8 +310,6 @@ GSBuildStrings()
       GS_REPLACE_CONSTANT_STRING(NSWeekDayNameArray);
       GS_REPLACE_CONSTANT_STRING(NSWillBecomeMultiThreadedNotification);
       GS_REPLACE_CONSTANT_STRING(NSYearMonthWeekDesignations);
-      GS_REPLACE_CONSTANT_STRING(PortBecameInvalidNotification);
-      GS_REPLACE_CONSTANT_STRING(StreamException);
     }
 }
 
