@@ -6,16 +6,16 @@
    Date: 2006
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -50,6 +50,10 @@
 #define	PF_LOCAL	PF_UNIX
 #endif
 
+#ifndef	socklen_t
+#define	socklen_t	uint32_t
+#endif
+
 /** 
  * The concrete subclass of NSInputStream that reads from a file
  */
@@ -288,8 +292,10 @@ static void setNonblocking(int fd)
 - (void) dealloc
 {
   if ([self _isOpened])
-    [self close];
-  RELEASE(_path);
+    {
+      [self close];
+    }
+  DESTROY(_path);
   [super dealloc];
 }
 
@@ -317,9 +323,15 @@ static void setNonblocking(int fd)
 
   readLen = read((intptr_t)_loopID, buffer, len);
   if (readLen < 0 && errno != EAGAIN && errno != EINTR)
-    [self _recordError];
+    {
+      [self _recordError];
+      readLen = -1;
+    }
   else if (readLen == 0)
-    [self _setStatus: NSStreamStatusAtEnd];
+    {
+      [self _setStatus: NSStreamStatusAtEnd];
+      [self _sendEvent: NSStreamEventEndEncountered];
+    }
   return readLen;
 }
 
@@ -356,6 +368,7 @@ static void setNonblocking(int fd)
   if (fd < 0)
     {  
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   _loopID = (void*)(intptr_t)fd;
@@ -400,7 +413,7 @@ static void setNonblocking(int fd)
 
 - (void) setSibling: (GSOutputStream*)sibling
 {
-  ASSIGN(_sibling, sibling);
+  _sibling = sibling;
 }
 
 - (void) setPassive: (BOOL)passive
@@ -422,8 +435,11 @@ static void setNonblocking(int fd)
 - (void) dealloc
 {
   if ([self _isOpened])
-    [self close];
-  RELEASE(_sibling);
+    {
+      [self close];
+    }
+  [(GSSocketOutputStream*)_sibling setSibling: nil];
+  _sibling = nil;
   [super dealloc];
 }
 
@@ -468,6 +484,7 @@ static void setNonblocking(int fd)
 	      return;
 	    }
           [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
           return;
         }
     }
@@ -512,9 +529,15 @@ static void setNonblocking(int fd)
 
   readLen = read((intptr_t)_loopID, buffer, len);
   if (readLen < 0 && errno != EAGAIN && errno != EINTR)
-    [self _recordError];
+    {
+      [self _recordError];
+      readLen = -1;
+    }
   else if (readLen == 0)
-    [self _setStatus: NSStreamStatusAtEnd];
+    {
+      [self _setStatus: NSStreamStatusAtEnd];
+      [self _sendEvent: NSStreamEventEndEncountered];
+    }
   return readLen;
 }
 
@@ -541,6 +564,7 @@ static void setNonblocking(int fd)
       socklen_t len = sizeof(error);
       unsigned i = [_modes count];
 
+      AUTORELEASE(RETAIN(self));
       while (i-- > 0)
 	{
 	  [_runloop removeStream: self mode: [_modes objectAtIndex: i]];
@@ -701,7 +725,9 @@ static void setNonblocking(int fd)
 - (void) dealloc
 {
   if ([self _isOpened])
-    [self close];
+    {
+      [self close];
+    }
   RELEASE(_path);
   [super dealloc];
 }
@@ -752,9 +778,10 @@ static void setNonblocking(int fd)
   else
     flag = flag | O_TRUNC;
   fd = open([_path fileSystemRepresentation], flag, mode);
-  if (fd< 0)
+  if (fd < 0)
     {  // make an error
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   _loopID = (void*)(intptr_t)fd;
@@ -811,7 +838,7 @@ static void setNonblocking(int fd)
 
 - (void) setSibling: (GSInputStream*)sibling
 {
-  ASSIGN(_sibling, sibling);
+  _sibling = sibling;
 }
 
 - (void) setPassive: (BOOL)passive
@@ -832,8 +859,11 @@ static void setNonblocking(int fd)
 - (void) dealloc
 {
   if ([self _isOpened])
-    [self close];
-  RELEASE(_sibling);
+    {
+      [self close];
+    }
+  [(GSSocketInputStream*)_sibling setSibling: nil];
+  _sibling = nil;
   [super dealloc];
 }
 
@@ -913,6 +943,7 @@ static void setNonblocking(int fd)
 	      return;
 	    }
           [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
           return;
         }
     }
@@ -947,6 +978,7 @@ static void setNonblocking(int fd)
       int result;
       unsigned i = [_modes count];
 
+      AUTORELEASE(RETAIN(self));
       while (i-- > 0)
 	{
 	  [_runloop removeStream: self mode: [_modes objectAtIndex: i]];
@@ -1405,7 +1437,9 @@ static void setNonblocking(int fd)
 - (void) dealloc
 {
   if ([self _isOpened])
-    [self close];
+    {
+      [self close];
+    }
   [super dealloc];
 }
 
@@ -1425,12 +1459,28 @@ static void setNonblocking(int fd)
 
 - (void) open
 {
-  int bindReturn = bind((int)(intptr_t)_loopID, [self serverAddr], [self sockLen]);
-  int listenReturn = listen((intptr_t)_loopID, SOCKET_BACKLOG);
+  int bindReturn;
+  int listenReturn;
 
-  if (bindReturn < 0 || listenReturn)
+#ifndef	BROKEN_SO_REUSEADDR
+  /*
+   * Under decent systems, SO_REUSEADDR means that the port can be reused
+   * immediately that this process exits.  Under some it means
+   * that multiple processes can serve the same port simultaneously.
+   * We don't want that broken behavior!
+   */
+  int	status = 1;
+
+  setsockopt((int)(intptr_t)_loopID, SOL_SOCKET, SO_REUSEADDR,
+    (char *)&status, sizeof(status));
+#endif
+
+  bindReturn = bind((int)(intptr_t)_loopID, [self serverAddr], [self sockLen]);
+  listenReturn = listen((intptr_t)_loopID, SOCKET_BACKLOG);
+  if (bindReturn < 0 || listenReturn < 0)
     {
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   setNonblocking((intptr_t)_loopID);
