@@ -7,18 +7,19 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #define	_FILE_OFFSET_BITS 64
@@ -107,16 +108,23 @@ static NSString*	NotificationKey = @"NSFileHandleNotificationKey";
     }
   else
     {
-      DWORD readBytes=-1;
-      if (ReadFile((HANDLE)_get_osfhandle(descriptor), buf, len, &readBytes, NULL)) {
-      	return readBytes;
-      } else {
-      	DWORD err = GetLastError();
-      	if (err == ERROR_BROKEN_PIPE || err == ERROR_HANDLE_EOF) {
-      		return readBytes;
-      	}
-      	return -1;
-      }
+      DWORD readBytes = -1;
+
+      if (ReadFile((HANDLE)_get_osfhandle(descriptor), buf, len,
+        &readBytes, NULL))
+        {
+          return readBytes;
+        }
+      else
+        {
+          DWORD err = GetLastError();
+
+          if (err == ERROR_BROKEN_PIPE || err == ERROR_HANDLE_EOF)
+            {
+              return readBytes;
+            }
+          return -1;
+        }
     }
   return len;
 }
@@ -792,7 +800,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == INVALID_SOCKET)
     {
-      NSLog(@"unable to create socket - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to create socket - %@", [NSError _last]);
       RELEASE(self);
       return nil;
     }
@@ -806,8 +814,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       if (bind(net, (struct sockaddr *)&lsin, sizeof(lsin)) == SOCKET_ERROR)
 	{
-	  NSLog(@"unable to bind to port %s:%d - %s", inet_ntoa(lsin.sin_addr),
-	    GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+	  NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(lsin.sin_addr),
+	    GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
 	  (void) closesocket(net);
 	  RELEASE(self);
 	  return nil;
@@ -825,9 +833,9 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  if (WSAGetLastError() != WSAEWOULDBLOCK)
 	    {
-	      NSLog(@"unable to make connection to %s:%d - %s",
+	      NSLog(@"unable to make connection to %s:%d - %@",
 		inet_ntoa(sin.sin_addr),
-		GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+		GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
 	      RELEASE(self);
 	      return nil;
 	    }
@@ -891,7 +899,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if ((net = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) == INVALID_SOCKET)
     {
-      NSLog(@"unable to create socket - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to create socket - %@", [NSError _last]);
       RELEASE(self);
       return nil;
     }
@@ -908,8 +916,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (bind(net, (struct sockaddr *)&sin, sizeof(sin)) == SOCKET_ERROR)
     {
-      NSLog(@"unable to bind to port %s:%d - %s", inet_ntoa(sin.sin_addr),
-	GSSwapBigI16ToHost(sin.sin_port), GSLastErrorStr(errno));
+      NSLog(@"unable to bind to port %s:%d - %@", inet_ntoa(sin.sin_addr),
+	GSSwapBigI16ToHost(sin.sin_port), [NSError _last]);
       (void) closesocket(net);
       RELEASE(self);
       return nil;
@@ -917,7 +925,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (listen(net, 256) == SOCKET_ERROR)
     {
-      NSLog(@"unable to listen on port - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to listen on port - %@", [NSError _last]);
       (void) closesocket(net);
       RELEASE(self);
       return nil;
@@ -925,7 +933,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(net, (struct sockaddr*)&sin, &size) == SOCKET_ERROR)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
       (void) closesocket(net);
       RELEASE(self);
       return nil;
@@ -1092,8 +1100,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
       if (_fstat(desc, &sbuf) != 0)
 	{
-          NSLog(@"unable to get status of descriptor %d - %s",
-	    desc, GSLastErrorStr(errno));
+          NSLog(@"unable to get status of descriptor %d - %@",
+	    desc, [NSError _last]);
 	}
       else
 	{
@@ -1225,7 +1233,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
   if (readInfo)
     {
-    [self receivedEventRead];
+      [self receivedEventRead];
     }
 }
 
@@ -1293,8 +1301,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (len < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to read from descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to read from descriptor - %@",
+                  [NSError _last]];
     }
   return d;
 }
@@ -1318,8 +1326,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (len < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to read from descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to read from descriptor - %@",
+                  [NSError _last]];
     }
   return d;
 }
@@ -1344,8 +1352,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       if (got < 0)
 	{
 	  [NSException raise: NSFileHandleOperationException
-		      format: @"unable to read from descriptor - %s",
-		      GSLastErrorStr(errno)];
+		      format: @"unable to read from descriptor - %@",
+		      [NSError _last]];
 	}
       [d setLength: got];
     }
@@ -1367,8 +1375,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	  else if (got < 0)
 	    {
 	      [NSException raise: NSFileHandleOperationException
-			  format: @"unable to read from descriptor - %s",
-			  GSLastErrorStr(errno)];
+			  format: @"unable to read from descriptor - %@",
+			  [NSError _last]];
 	    }
 	}
       while (len > 0 && got > 0);
@@ -1414,8 +1422,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (rval < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"unable to write to descriptor - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"unable to write to descriptor - %@",
+                  [NSError _last]];
     }
 }
 
@@ -1520,8 +1528,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
   return (unsigned long long)result;
 }
@@ -1544,8 +1552,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
   return (unsigned long long)result;
 }
@@ -1568,8 +1576,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   if (result < 0)
     {
       [NSException raise: NSFileHandleOperationException
-                  format: @"failed to move to offset in file - %s",
-                  GSLastErrorStr(errno)];
+                  format: @"failed to move to offset in file - %@",
+                  [NSError _last]];
     }
 }
 
@@ -1740,6 +1748,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   NSRunLoop	*l;
   NSArray	*modes;
+  void		*e;
+  int		t;
 
   if (descriptor < 0)
     {
@@ -1754,34 +1764,25 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	NSFileHandleNotificationMonitorModes];
     }
 
+  e = (void*)(event ? (uintptr_t)event : (uintptr_t)descriptor);
+  t = event ? ET_HANDLE : ET_TRIGGER;
+
   if (modes && [modes count])
     {
       unsigned int	i;
 
       for (i = 0; i < [modes count]; i++)
 	{
-	if (event)
-	  [l removeEvent: (void*)(uintptr_t)event
-		    type: ET_HANDLE
-		 forMode: [modes objectAtIndex: i]
-		     all: YES];
-	else
-	  [l removeEvent:0
-		    type: ET_TRIGGER
+	  [l removeEvent: e
+		    type: t
 		 forMode: [modes objectAtIndex: i]
 		     all: YES];
         }
     }
   else
     {
-    if (event)
-      [l removeEvent: (void*)(uintptr_t)event
-	        type: ET_HANDLE
-	     forMode: NSDefaultRunLoopMode
-                 all: YES];
-	else             
-      [l removeEvent:0
-	        type: ET_TRIGGER
+      [l removeEvent: e
+	        type: t
 	     forMode: NSDefaultRunLoopMode
                  all: YES];
     }
@@ -1791,6 +1792,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   NSRunLoop	*l;
   NSArray	*modes;
+  void		*e;
+  int		t;
 
   if (descriptor < 0)
     {
@@ -1806,22 +1809,25 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       modes=(NSArray*)[info objectForKey: NSFileHandleNotificationMonitorModes];
     }
 
+  e = (void*)(event ? (uintptr_t)event : (uintptr_t)descriptor);
+  t = event ? ET_HANDLE : ET_TRIGGER;
+
   if (modes && [modes count])
     {
       unsigned int	i;
 
       for (i = 0; i < [modes count]; i++)
 	{
-          [l removeEvent: (void*)(uintptr_t)event
-	            type: event ? ET_HANDLE : ET_TRIGGER
+          [l removeEvent: e
+	            type: t
 	         forMode: [modes objectAtIndex: i]
                      all: YES];
         }
     }
   else
     {
-      [l removeEvent: (void*)(uintptr_t)event
-                type: event ? ET_HANDLE : ET_TRIGGER
+      [l removeEvent: e
+                type: t
 	     forMode: NSDefaultRunLoopMode
                  all: YES];
     }
@@ -1830,6 +1836,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 - (void) watchReadDescriptorForModes: (NSArray*)modes;
 {
   NSRunLoop	*l;
+  void		*e;
+  int		t;
 
   if (descriptor < 0)
     {
@@ -1838,20 +1846,18 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   l = [NSRunLoop currentRunLoop];
   [self setNonBlocking: YES];
+
+  e = (void*)(event ? (uintptr_t)event : (uintptr_t)descriptor);
+  t = event ? ET_HANDLE : ET_TRIGGER;
+
   if (modes && [modes count])
     {
       unsigned int	i;
 
       for (i = 0; i < [modes count]; i++)
 	{
-	  if (event)
-	  [l addEvent: (void*)(uintptr_t)event
-		 type: ET_HANDLE
-	      watcher: self
-	      forMode: [modes objectAtIndex: i]];
-	  else
-	    [l addEvent:0
-		 type: ET_TRIGGER
+	  [l addEvent: e
+		 type: t
 	      watcher: self
 	      forMode: [modes objectAtIndex: i]];
         }
@@ -1859,14 +1865,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
   else
     {
-	  if (event)
-      [l addEvent: (void*)(uintptr_t)event
-	     type: ET_HANDLE
-	  watcher: self
-	  forMode: NSDefaultRunLoopMode];
-	  else
-      [l addEvent:0
-	     type: ET_TRIGGER
+      [l addEvent: e
+	     type: t
 	  watcher: self
 	  forMode: NSDefaultRunLoopMode];
     }
@@ -1883,26 +1883,32 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       NSMutableDictionary	*info = [writeInfo objectAtIndex: 0];
       NSRunLoop			*l = [NSRunLoop currentRunLoop];
       NSArray			*modes = nil;
+      void			*e;
+      int			t;
 
       modes = [info objectForKey: NSFileHandleNotificationMonitorModes];
 
       [self setNonBlocking: YES];
+
+      e = (void*)(event ? (uintptr_t)event : (uintptr_t)descriptor);
+      t = event ? ET_HANDLE : ET_TRIGGER;
+
       if (modes && [modes count])
 	{
 	  unsigned int	i;
 
 	  for (i = 0; i < [modes count]; i++)
 	    {
-	      [l addEvent: (void*)(uintptr_t)event
-		     type: event ? ET_HANDLE : ET_TRIGGER
+	      [l addEvent: e
+		     type: t
 		  watcher: self
 		  forMode: [modes objectAtIndex: i]];
 	    }
 	}
       else
 	{
-	  [l addEvent: (void*)(uintptr_t)event
-		 type: event ? ET_HANDLE : ET_TRIGGER
+	  [l addEvent: e
+		 type: t
 	      watcher: self
 	      forMode: NSDefaultRunLoopMode];
 	}
@@ -1927,8 +1933,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  NSString	*s;
 
-	  s = [NSString stringWithFormat: @"Accept attempt failed - %s",
-		GSLastErrorStr(errno)];
+	  s = [NSString stringWithFormat: @"Accept attempt failed - %@",
+	    [NSError _last]];
 	  [readInfo setObject: s forKey: GSFileHandleNotificationError];
 	}
       else
@@ -1996,8 +2002,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
             {
 	      NSString	*s;
 
-	      s = [NSString stringWithFormat: @"Read attempt failed - %s",
-		    GSLastErrorStr(errno)];
+	      s = [NSString stringWithFormat: @"Read attempt failed - %@",
+		[NSError _last]];
 	      [readInfo setObject: s forKey: GSFileHandleNotificationError];
 	      [self postReadNotification];
 	    }
@@ -2005,8 +2011,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
             {
 	      NSString	*s;
 
-	      s = [NSString stringWithFormat: @"Read attempt failed - %s",
-		    GSLastErrorStr(errno)];
+	      s = [NSString stringWithFormat: @"Read attempt failed - %@",
+		[NSError _last]];
 	      [readInfo setObject: s forKey: GSFileHandleNotificationError];
 	      [self postReadNotification];
 	    }
@@ -2038,15 +2044,25 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     || operation == GSSOCKSConnect)
     { // Connection attempt completed.
       int	result;
+      int	rval;
       unsigned	len = sizeof(result);
 
-      if (getsockopt((SOCKET)_get_osfhandle(descriptor), SOL_SOCKET, SO_ERROR,
-        (char*)&result, &len) == 0 && result != 0)
+      rval = getsockopt((SOCKET)_get_osfhandle(descriptor), SOL_SOCKET,
+        SO_ERROR, (char*)&result, &len);
+      if (rval != 0)
         {
           NSString	*s;
 
-          s = [NSString stringWithFormat: @"Connect attempt failed - %s",
-		GSLastErrorStr(result)];
+          s = [NSString stringWithFormat: @"Connect attempt failed - %@",
+	    [NSError _last]];
+          [info setObject: s forKey: GSFileHandleNotificationError];
+	}
+      else if (result != 0)
+        {
+          NSString	*s;
+
+          s = [NSString stringWithFormat: @"Connect attempt failed - %@",
+	    [NSError _systemError: result]];
           [info setObject: s forKey: GSFileHandleNotificationError];
         }
       else
@@ -2080,7 +2096,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	          NSString	*s;
 
 	          s = [NSString stringWithFormat:
-	        	@"Write attempt failed - %s", GSLastErrorStr(errno)];
+		    @"Write attempt failed - %@", [NSError _last]];
 	          [info setObject: s forKey: GSFileHandleNotificationError];
 	          [self postWriteNotification];
 	        }
@@ -2110,52 +2126,71 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       [self setNonBlocking: YES];
     }
-  if (isSocket) {
-  if (WSAEnumNetworkEvents((SOCKET)_get_osfhandle(descriptor), 
-    event, &ocurredEvents) == SOCKET_ERROR)
+  if (isSocket)
     {
-      NSLog(@"Error getting event type %d", WSAGetLastError());
-      abort();
+      if (WSAEnumNetworkEvents((SOCKET)_get_osfhandle(descriptor), 
+	event, &ocurredEvents) == SOCKET_ERROR)
+	{
+	  NSLog(@"Error getting event type %d", WSAGetLastError());
+	  abort();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_CONNECT)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"Connect on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_CONNECT;
+	  [self receivedEventWrite];
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_ACCEPT)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"Accept on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_ACCEPT;
+	  [self receivedEventRead];
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_WRITE)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"Write on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_WRITE;
+	  [self receivedEventWrite];
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_READ)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"Read on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_READ;
+	  [self receivedEventRead];
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_OOB)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"OOB on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_OOB;
+	  [self receivedEventRead];
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents & FD_CLOSE)
+	{
+	  NSDebugMLLog(@"NSFileHandle", @"Close on %x", extra);
+	  ocurredEvents.lNetworkEvents ^= FD_CLOSE;
+	  if ([writeInfo count] > 0)
+	    {
+	      [self receivedEventWrite];
+	    }
+	  else
+	    {
+	      [self receivedEventRead];
+	    }
+	  GSPrivateNotifyASAP();
+	}
+      if (ocurredEvents.lNetworkEvents)
+	{
+	  NSLog(@"Event not get %d", ocurredEvents.lNetworkEvents);
+	  abort();      
+	}
     }
-  if (ocurredEvents.lNetworkEvents & FD_CONNECT)
+  else
     {
-      NSDebugMLLog(@"NSFileHandle", @"Connect on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_CONNECT;
-      [self receivedEventWrite];
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents & FD_ACCEPT)
-    {
-      NSDebugMLLog(@"NSFileHandle", @"Accept on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_ACCEPT;
-      [self receivedEventRead];
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents & FD_WRITE)
-    {
-      NSDebugMLLog(@"NSFileHandle", @"Write on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_WRITE;
-      [self receivedEventWrite];
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents & FD_READ)
-    {
-      NSDebugMLLog(@"NSFileHandle", @"Read on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_READ;
-      [self receivedEventRead];
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents & FD_OOB)
-    {
-      NSDebugMLLog(@"NSFileHandle", @"OOB on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_OOB;
-      [self receivedEventRead];
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents & FD_CLOSE)
-    {
-      NSDebugMLLog(@"NSFileHandle", @"Close on %x", extra);
-      ocurredEvents.lNetworkEvents ^= FD_CLOSE;
       if ([writeInfo count] > 0)
 	{
 	  [self receivedEventWrite];
@@ -2164,23 +2199,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	{
 	  [self receivedEventRead];
 	}
-      GSNotifyASAP();
-    }
-  if (ocurredEvents.lNetworkEvents)
-    {
-      NSLog(@"Event not get %d", ocurredEvents.lNetworkEvents);
-      abort();      
-    }
-    } else {
-      if ([writeInfo count] > 0)
-	{
-	  [self receivedEventWrite];
-	}
-      else
-	{
-	  [self receivedEventRead];
-	}
-      GSNotifyASAP();
+      GSPrivateNotifyASAP();
     }
 }
 
@@ -2193,7 +2212,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
 - (void) setAddr: (struct sockaddr_in *)sin
 {
-  address = [[NSString alloc] initWithCString: (char*)inet_ntoa(sin->sin_addr)];
+  address = [[NSString alloc] initWithUTF8String:
+    (char*)inet_ntoa(sin->sin_addr)];
   service = [[NSString alloc] initWithFormat: @"%d",
     (int)GSSwapBigI16ToHost(sin->sin_port)];
   protocol = @"tcp";
@@ -2217,21 +2237,26 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       unsigned long	dummy;
 
-      if (isSocket != YES) {
-        // Not a file and not a socket, must be a pipe
-		DWORD mode;
-		if (flag)
-			mode = PIPE_NOWAIT;
-		else
-			mode = PIPE_WAIT;
-		if (SetNamedPipeHandleState((HANDLE)_get_osfhandle(descriptor), &mode, NULL, NULL)) {
-			isNonBlocking = flag;
-		} else {
+      if (isSocket != YES)
+        {                        // Not a file and not a socket, must be a pipe
+          DWORD mode;
+
+          if (flag)
+            mode = PIPE_NOWAIT;
+          else
+            mode = PIPE_WAIT;
+          if (SetNamedPipeHandleState((HANDLE)_get_osfhandle(descriptor),
+            &mode, NULL, NULL))
+            {
+              isNonBlocking = flag;
+            }
+          else
+            {
 	      NSLog(@"unable to set pipe non-blocking mode - %d",
 		GetLastError());
-		}
-        return;
-      }
+            }
+          return;
+        }
 
       if (flag)
 	{
@@ -2239,8 +2264,8 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	  if (ioctlsocket((SOCKET)_get_osfhandle(descriptor), FIONBIO, &dummy)
 	    == SOCKET_ERROR)
 	    {
-	      NSLog(@"unable to set non-blocking mode - %s",
-		GSLastErrorStr(errno));
+	      NSLog(@"unable to set non-blocking mode - %@",
+		[NSError _last]);
 	    }
 	  else
 	    isNonBlocking = flag;
@@ -2251,8 +2276,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	  if (ioctlsocket((SOCKET)_get_osfhandle(descriptor), FIONBIO, &dummy)
 	    == SOCKET_ERROR)
 	    {
-	      NSLog(@"unable to set blocking mode - %s",
-		GSLastErrorStr(errno));
+	      NSLog(@"unable to set blocking mode - %@", [NSError _last]);
 	    }
 	  else
 	    isNonBlocking = flag;
@@ -2273,11 +2297,11 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == SOCKET_ERROR)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
-      str = [NSString stringWithCString: (char*)inet_ntoa(sin.sin_addr)];
+      str = [NSString stringWithUTF8String: (char*)inet_ntoa(sin.sin_addr)];
     }
   return str;
 }
@@ -2290,7 +2314,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == SOCKET_ERROR)
     {
-      NSLog(@"unable to get socket name - %s", GSLastErrorStr(errno));
+      NSLog(@"unable to get socket name - %@", [NSError _last]);
     }
   else
     {
