@@ -6,16 +6,16 @@
    Date: 2006
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
@@ -44,6 +44,7 @@
 #include <Foundation/NSDebug.h>
 
 #include "../GSStream.h"
+#include "../GSPrivate.h"
 
 #define	BUFFERSIZE	(BUFSIZ*64)
 
@@ -269,7 +270,7 @@ static void setNonblocking(SOCKET fd)
   unsigned long	dummy = 1;
 
   if (ioctlsocket(fd, FIONBIO, &dummy) == SOCKET_ERROR)
-    NSLog(@"unable to set non-blocking mode - %s",GSLastErrorStr(errno));
+    NSLog(@"unable to set non-blocking mode - %@", [NSError _last]);
 }
 
 @implementation GSFileInputStream
@@ -325,6 +326,7 @@ static void setNonblocking(SOCKET fd)
   if (h == INVALID_HANDLE_VALUE)
     {
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   [self _setLoopID: (void*)h];
@@ -764,6 +766,7 @@ static void setNonblocking(SOCKET fd)
 	&& WSAGetLastError() != WSAEWOULDBLOCK)
         {// make an error
           [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
           return;
         }
       // waiting on writable, as an indication of opened
@@ -857,6 +860,7 @@ static void setNonblocking(SOCKET fd)
   else if (readLen == 0)
     {
       [self _setStatus: NSStreamStatusAtEnd];
+      [self _sendEvent: NSStreamEventEndEncountered];
     }
   else 
     {
@@ -872,6 +876,7 @@ static void setNonblocking(SOCKET fd)
 
 - (void) _dispatch
 {
+  AUTORELEASE(RETAIN(self));
   /*
    * Windows only permits a single event to be associated with a socket
    * at any time, but the runloop system only allows an event handle to
@@ -1016,7 +1021,7 @@ static void setNonblocking(SOCKET fd)
     {
       _peerAddr.sin_family = AF_INET;
       _peerAddr.sin_port = htons(port);
-      _peerAddr.sin_addr.s_addr = inet_addr(addr_c);
+      _peerAddr.sin_addr.s_addr = addr_c ? inet_addr(addr_c) : INADDR_NONE;
       if (_peerAddr.sin_addr.s_addr == INADDR_NONE)   // error
 	{
 	  DESTROY(self);
@@ -1076,6 +1081,7 @@ static void setNonblocking(SOCKET fd)
   if (h == INVALID_HANDLE_VALUE)
     {
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   else if (_shouldAppend == NO)
@@ -1083,6 +1089,9 @@ static void setNonblocking(SOCKET fd)
       if (SetEndOfFile(h) == 0)	// Truncate to current file pointer (0)
 	{
           [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
+          CloseHandle(h);
+	  return;
 	}
     }
   [self _setLoopID: (void*)h];
@@ -1538,6 +1547,7 @@ static void setNonblocking(SOCKET fd)
 	&& WSAGetLastError() != WSAEWOULDBLOCK)
         {// make an error
           [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
           return;
         }
       // waiting on writable, as an indication of opened
@@ -1602,6 +1612,7 @@ static void setNonblocking(SOCKET fd)
 
 - (void) _dispatch
 {
+  AUTORELEASE(RETAIN(self));
   /*
    * Windows only permits a single event to be associated with a socket
    * at any time, but the runloop system only allows an event handle to
@@ -1752,7 +1763,7 @@ static void setNonblocking(SOCKET fd)
     {
       _peerAddr.sin_family = AF_INET;
       _peerAddr.sin_port = htons(port);
-      _peerAddr.sin_addr.s_addr = inet_addr(addr_c);
+      _peerAddr.sin_addr.s_addr = addr_c ? inet_addr(addr_c) : INADDR_NONE;
       if (_peerAddr.sin_addr.s_addr == INADDR_NONE)   // error
 	{
 	  DESTROY(self);
@@ -1866,8 +1877,8 @@ static void setNonblocking(SOCKET fd)
   if (handle == INVALID_HANDLE_VALUE)
     {
       [NSException raise: NSInternalInconsistencyException
-		  format: @"Unable to open named pipe '%@'... %s",
-	path, GSLastErrorStr(GetLastError())];
+		  format: @"Unable to open named pipe '%@'... %@",
+	path, [NSError _last]];
     }
 
   // the type of the stream does not matter, since we are only using the fd
@@ -2212,12 +2223,25 @@ done:
 
 - (void) open
 {
-  int bindReturn = bind(_sock, [self serverAddr], [self sockLen]);
-  int listenReturn = listen(_sock, SOCKET_BACKLOG);
+  int	bindReturn;
+  int	listenReturn;
+#ifndef	BROKEN_SO_REUSEADDR
+  int	status = 1;
 
-  if (bindReturn < 0 || listenReturn)
+  /*
+   * Under decent systems, SO_REUSEADDR means that the port can be reused
+   * immediately that this process exits.  Under some it means
+   * that multiple processes can serve the same port simultaneously.
+   * We don't want that broken behavior!
+   */
+  setsockopt(_sock, SOL_SOCKET, SO_REUSEADDR, (char *)&status, sizeof(status));
+#endif
+  bindReturn = bind(_sock, [self serverAddr], [self sockLen]);
+  listenReturn = listen(_sock, SOCKET_BACKLOG);
+  if (bindReturn < 0 || listenReturn < 0)
     {
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
   setNonblocking(_sock);
@@ -2347,7 +2371,7 @@ done:
   [super init];
   _serverAddr.sin_family = AF_INET;
   _serverAddr.sin_port = htons(port);
-  _serverAddr.sin_addr.s_addr = inet_addr(addr_c);
+  _serverAddr.sin_addr.s_addr = addr_c ? inet_addr(addr_c) : INADDR_NONE;
   _sock = socket(AF_INET, SOCK_STREAM, 0);
   if (_serverAddr.sin_addr.s_addr == INADDR_NONE || _loopID < 0)   // error
     {
@@ -2431,6 +2455,7 @@ done:
   if (handle == INVALID_HANDLE_VALUE)
     {
       [self _recordError];
+      [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
 
@@ -2453,6 +2478,7 @@ done:
       else if (errno != ERROR_IO_PENDING)
 	{
 	  [self _recordError];
+          [self _sendEvent: NSStreamEventErrorOccurred];
 	  return;
 	}
     }
