@@ -825,16 +825,16 @@ static BOOL snuggleStart(NSString *t)
       NSString		*s = [a objectForKey: @"BaseType"];
 
       [str appendString: @"        <arg type=\""];
-      [str appendString: s];
+      [str appendString: escapeType(s)];
       s = [a objectForKey: @"Prefix"];
       if (s != nil)
 	{
-	  [str appendString: s];
+	  [str appendString: escapeType(s)];
 	}
       s = [a objectForKey: @"Suffix"];
       if (s != nil)
 	{
-	  [str appendString: s];
+	  [str appendString: escapeType(s)];
 	}
       [str appendString: @"\">"];
       [str appendString: [a objectForKey: @"Name"]];
@@ -2334,8 +2334,8 @@ static BOOL snuggleStart(NSString *t)
 
 	  [str appendString: @"<?xml version=\"1.0\"?>\n"];
 	  [str appendString: @"<!DOCTYPE gsdoc PUBLIC "];
-	  [str appendString: @"\"-//GNUstep//DTD gsdoc 1.0.1//EN\" "];
-	  [str appendString: @"\"http://www.gnustep.org/gsdoc-1_0_1.xml\">\n"];
+	  [str appendString: @"\"-//GNUstep//DTD gsdoc 1.0.3//EN\" "];
+	  [str appendString: @"\"http://www.gnustep.org/gsdoc-1_0_3.xml\">\n"];
 	  [str appendString: @"<gsdoc base=\""];
 	  [str appendString: [name lastPathComponent]];
 	  /*
