@@ -40,11 +40,12 @@
 #endif
 
 static BOOL	is_daemon = NO;		/* Currently running as daemon.	 */
+static BOOL	auto_stop = NO;		/* Should we shut down when unused? */
 static char	ebuf[2048];
 
 #ifdef HAVE_SYSLOG
 
-static int	log_priority;
+static int	log_priority = LOG_DEBUG;
 
 static void
 gdnc_log (int prio)
@@ -642,6 +643,14 @@ ihandler(int sig)
 	  [self removeObserversForClients: table];
 	  NSFreeMapTable(table);
 	}
+
+      if (auto_stop == YES && NSCountMapTable(connections) == 0)
+        {
+	  /* If there is nothing else using this process, and this is not
+	   * a daemon, then we can quietly terminate.
+	   */
+          exit(EXIT_SUCCESS);
+	}
     }
   return nil;
 }
@@ -1047,6 +1056,10 @@ main(int argc, char** argv, char** env)
       printf("--verbose\tMore verbose debug output\n");
       exit(EXIT_SUCCESS);
     }
+  if ([[pInfo arguments] containsObject: @"--auto"] == YES)
+    {
+      auto_stop = YES;
+    }
   if ([[pInfo arguments] containsObject: @"--daemon"] == YES)
     {
       subtask = NO;
@@ -1106,6 +1119,7 @@ main(int argc, char** argv, char** env)
 
     for (sym = 0; sym < NSIG; sym++)
       {
+	if (sym == SIGABRT) continue;
 #ifdef	SIGPROF
 	if (sym == SIGPROF) continue;
 #endif
