@@ -13,7 +13,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -69,7 +69,7 @@ id process_plist(NSData *inputData)
   NSString *string = nil;
 
   // Initialize a string with the contents of the file.
-  string = [NSString stringWithCString: (char *)[inputData bytes]];
+  string = [NSString stringWithUTF8String: (char *)[inputData bytes]];
 
   // Convert the string into a property list.  If there is a parsing error
   // the property list interpreter will throw an exception.
